@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2006 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * extending class to class t3lib_basicFileFunctions
  *
- * $Id: class.t3lib_extfilefunc.php,v 1.14 2004/09/13 22:57:17 typo3 Exp $
+ * $Id: class.t3lib_extfilefunc.php 1657 2006-07-26 17:54:15Z mundaun $
  * Revised for TYPO3 3.6 May/2004 by Kasper Skaarhoj
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
@@ -38,23 +38,23 @@
  *
  *
  *  105: class t3lib_extFileFunctions extends t3lib_basicFileFunctions
- *  149:     function start($fileCmds)
- *  179:     function init_actionPerms($setup)
- *  211:     function processData()
- *  268:     function printLogErrorMessages($redirect='')
- *  326:     function findRecycler($theFile)
- *  355:     function writeLog($action,$error,$details_nr,$details,$data)
+ *  151:     function start($fileCmds)
+ *  181:     function init_actionPerms($setup)
+ *  213:     function processData()
+ *  270:     function printLogErrorMessages($redirect='')
+ *  328:     function findRecycler($theFile)
+ *  357:     function writeLog($action,$error,$details_nr,$details,$data)
  *
  *              SECTION: File operation functions
- *  382:     function func_delete($cmds)
- *  449:     function func_copy($cmds)
- *  539:     function func_move($cmds)
- *  632:     function func_rename($cmds)
- *  678:     function func_newfolder($cmds)
- *  708:     function func_newfile($cmds)
- *  745:     function func_edit($cmds)
- *  777:     function func_upload($cmds)
- *  815:     function func_unzip($cmds)
+ *  384:     function func_delete($cmds)
+ *  451:     function func_copy($cmds)
+ *  542:     function func_move($cmds)
+ *  637:     function func_rename($cmds)
+ *  683:     function func_newfolder($cmds)
+ *  713:     function func_newfile($cmds)
+ *  750:     function func_edit($cmds)
+ *  782:     function func_upload($cmds)
+ *  821:     function func_unzip($cmds)
  *
  * TOTAL FUNCTIONS: 15
  * (This index is automatically created/updated by the extension "extdeveval")
@@ -136,6 +136,8 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 	var $PHPFileFunctions = 0;			// If set, all fileoperations are done by the default PHP-functions. This is necessary under windows! On UNIX the system commands by exec() can be used unless safe_mode is enabled
 	var $dont_use_exec_commands = 0;	// This is necessary under windows!
 
+		// Internal, dynamic:
+	var $internalUploadMap = array();	// Will contain map between upload ID and the final filename
 
 
 
@@ -300,7 +302,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 			$lines[] = '
 					<tr>
 						<td colspan="2" align="center"><br />'.
-						'<form action=""><input type="submit" value="Continue" onclick="'.htmlspecialchars('document.location=\''.$redirect.'\';return false;').'" /></form>'.
+						'<form action=""><input type="submit" value="Continue" onclick="'.htmlspecialchars('window.location.href=\''.$redirect.'\';return false;').'" /></form>'.
 						'</td>
 					</tr>';
 
@@ -376,7 +378,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 	/**
 	 * Deleting files and folders (action=4)
 	 *
-	 * @param	array		$cmds['data'] is the the file/folder to delete
+	 * @param	array		$cmds['data'] is the file/folder to delete
 	 * @return	boolean		Returns true upon success
 	 */
 	function func_delete($cmds)	{
@@ -411,12 +413,12 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 				} else $this->writelog(4,1,112,'You are not allowed to delete files','');
 				// FINISHED deleting file
 
-			} elseif (@is_dir($theFile) && !$this->dont_use_exec_commands) {	// if we're deleting a folder
+			} elseif (@is_dir($theFile)) {	// if we're deleting a folder
 				if ($this->actionPerms['deleteFolder'])	{
 					$theFile = $this->is_directory($theFile);
 					if ($theFile)	{
 						if ($this->checkPathAgainstMounts($theFile))	{	// I choose not to append '/' to $theFile here as this will prevent us from deleting mounts!! (which makes sense to me...)
-							if ($this->actionPerms['deleteFolderRecursively'])	{
+							if ($this->actionPerms['deleteFolderRecursively'] && !$this->dont_use_exec_commands)	{
 									// No way to do this under windows
 								$cmd = 'rm -Rf "'.$theFile.'"';
 								exec($cmd);		// This is a quite critical command...
@@ -443,7 +445,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 	/**
 	 * Copying files and folders (action=2)
 	 *
-	 * @param	array		$cmds['data'] is the the file/folder to copy. $cmds['target'] is the path where to copy to. $cmds['altName'] (boolean): If set, another filename is found in case the target already exists
+	 * @param	array		$cmds['data'] is the file/folder to copy. $cmds['target'] is the path where to copy to. $cmds['altName'] (boolean): If set, another filename is found in case the target already exists
 	 * @return	string		Returns the new filename upon success
 	 */
 	function func_copy($cmds)	{
@@ -482,6 +484,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 									$cmd = 'cp "'.$theFile.'" "'.$theDestFile.'"';
 									exec($cmd);
 								}
+								t3lib_div::fixPermissions($theDestFile);
 								clearstatcache();
 								if (@is_file($theDestFile))	{
 									$this->writelog(2,0,1,'File "%s" copied to "%s"',Array($theFile,$theDestFile));
@@ -533,7 +536,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 	/**
 	 * Moving files and folders (action=3)
 	 *
-	 * @param	array		$cmds['data'] is the the file/folder to move. $cmds['target'] is the path where to move to. $cmds['altName'] (boolean): If set, another filename is found in case the target already exists
+	 * @param	array		$cmds['data'] is the file/folder to move. $cmds['target'] is the path where to move to. $cmds['altName'] (boolean): If set, another filename is found in case the target already exists
 	 * @return	string		Returns the new filename upon success
 	 */
 	function func_move($cmds)	{
@@ -603,6 +606,8 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 										rename($theFile, $theDestFile);
 									} else {
 										$cmd = 'mv "'.$theFile.'" "'.$theDestFile.'"';
+										$errArr = array();
+										$retVar = 0;
 										exec($cmd,$errArr,$retVar);
 									}
 									clearstatcache();
@@ -737,7 +742,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 	}
 
 	/**
-	 * Editing textfiles or foldes (action=9)
+	 * Editing textfiles or folders (action=9)
 	 *
 	 * @param	array		$cmds['data'] is the new content. $cmds['target'] is the target (file or dir)
 	 * @return	boolean		Returns true on success
@@ -793,6 +798,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 									t3lib_div::upload_copy_move($theFile,$theNewFile);
 									clearstatcache();
 									if (@is_file($theNewFile))	{
+										$this->internalUploadMap[$id] = $theNewFile;
 										$this->writelog(1,0,1,'Uploading file "%s" to "%s"',Array($theName,$theNewFile, $id));
 										return $theNewFile;
 									} else $this->writelog(1,1,100,'Uploaded file could not be moved! Write-permission problem in "%s"?',Array($theTarget.'/'));
@@ -802,7 +808,7 @@ class t3lib_extFileFunctions extends t3lib_basicFileFunctions	{
 					} else $this->writelog(1,1,104,'The uploaded file exceeds the size-limit of %s bytes',Array($this->maxUploadFileSize*1024));
 				} else $this->writelog(1,1,105,'You are not allowed to upload files!','');
 			} else $this->writelog(1,2,106,'The uploaded file did not exist!','');
-		}
+		} else $this->writelog(1,2,106,'No file was uploaded!','');
 	}
 
 	/**
