@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2006 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Contains a class with Extension Management functions
  *
- * $Id: class.t3lib_extmgm.php,v 1.17.2.1 2004/09/19 20:17:09 mundaun Exp $
+ * $Id: class.t3lib_extmgm.php 1421 2006-04-10 09:27:15Z mundaun $
  * Revised for TYPO3 3.6 July/2003 by Kasper Skaarhoj
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
@@ -37,50 +37,51 @@
  *
  *
  *
- *  113: class t3lib_extMgm
+ *  114: class t3lib_extMgm
  *
  *              SECTION: PATHS and other evaluation
- *  130:     function isLoaded($key,$exitOnError=0)
- *  146:     function extPath($key,$script='')
- *  164:     function extRelPath($key)
- *  181:     function siteRelPath($key)
- *  193:     function getCN($key)
+ *  131:     function isLoaded($key,$exitOnError=0)
+ *  147:     function extPath($key,$script='')
+ *  165:     function extRelPath($key)
+ *  182:     function siteRelPath($key)
+ *  194:     function getCN($key)
  *
  *              SECTION: Adding BACKEND features
- *  226:     function addTCAcolumns($table,$columnArray,$addTofeInterface=0)
- *  250:     function addToAllTCAtypes($table,$str,$specificTypesList='',$position='')
- *  312:     function allowTableOnStandardPages($table)
- *  329:     function addModule($main,$sub='',$position='',$path='')
- *  391:     function insertModuleFunction($modname,$className,$classPath,$title,$MM_key='function')
- *  409:     function addPageTSConfig($content)
- *  423:     function addUserTSConfig($content)
- *  438:     function addLLrefForTCAdescr($tca_descr_key,$file_ref)
+ *  227:     function addTCAcolumns($table,$columnArray,$addTofeInterface=0)
+ *  251:     function addToAllTCAtypes($table,$str,$specificTypesList='',$position='')
+ *  309:     function allowTableOnStandardPages($table)
+ *  326:     function addModule($main,$sub='',$position='',$path='')
+ *  389:     function insertModuleFunction($modname,$className,$classPath,$title,$MM_key='function',$WS='')
+ *  408:     function addPageTSConfig($content)
+ *  422:     function addUserTSConfig($content)
+ *  437:     function addLLrefForTCAdescr($tca_descr_key,$file_ref)
  *
  *              SECTION: Adding SERVICES features
- *  480:     function addService($extKey, $serviceType, $serviceKey, $info)
- *  545:     function findService($serviceType, $serviceSubType='', $excludeServiceKeys=array())
- *  616:     function deactivateService($serviceType, $serviceKey)
+ *  479:     function addService($extKey, $serviceType, $serviceKey, $info)
+ *  547:     function findService($serviceType, $serviceSubType='', $excludeServiceKeys=array())
+ *  618:     function deactivateService($serviceType, $serviceKey)
  *
  *              SECTION: Adding FRONTEND features
- *  655:     function addPlugin($itemArray,$type='list_type')
- *  680:     function addPiFlexFormValue($piKeyToMatch,$value)
- *  700:     function addToInsertRecords($table,$content_table='tt_content',$content_field='records')
- *  731:     function addPItoST43($key,$classFile='',$prefix='',$type='list_type',$cached=0)
- *  806:     function addStaticFile($extKey,$path,$title)
- *  825:     function addTypoScriptSetup($content)
- *  839:     function addTypoScriptConstants($content)
- *  856:     function addTypoScript($key,$type,$content,$afterStaticUid=0)
+ *  657:     function addPlugin($itemArray,$type='list_type')
+ *  682:     function addPiFlexFormValue($piKeyToMatch,$value)
+ *  702:     function addToInsertRecords($table,$content_table='tt_content',$content_field='records')
+ *  733:     function addPItoST43($key,$classFile='',$prefix='',$type='list_type',$cached=0)
+ *  808:     function addStaticFile($extKey,$path,$title)
+ *  827:     function addTypoScriptSetup($content)
+ *  841:     function addTypoScriptConstants($content)
+ *  858:     function addTypoScript($key,$type,$content,$afterStaticUid=0)
  *
  *              SECTION: INTERNAL EXTENSION MANAGEMENT:
- *  919:     function typo3_loadExtensions()
+ *  921:     function typo3_loadExtensions()
  *  998:     function _makeIncludeHeader($key,$file)
  * 1019:     function isCacheFilesAvailable($cacheFilePrefix)
  * 1032:     function isLocalconfWritable()
  * 1045:     function cannotCacheFilesWritable($cacheFilePrefix)
  * 1069:     function currentCacheFiles()
  * 1092:     function writeCacheFiles($extensions,$cacheFilePrefix)
+ * 1130:     function removeCacheFiles()
  *
- * TOTAL FUNCTIONS: 31
+ * TOTAL FUNCTIONS: 32
  * (This index is automatically created/updated by the extension "extdeveval")
  *
  */
@@ -257,27 +258,23 @@ class t3lib_extMgm {
 		if (trim($str) && is_array($TCA[$table]) && is_array($TCA[$table]['types']))	{
 			foreach($TCA[$table]['types'] as $k => $v)	{
 				if (!$specificTypesList || t3lib_div::inList($specificTypesList,$k))	{
-
-
-
-					if ($insert) {
+					if ($insert)	{
 						$append=true;
 						$showItem = t3lib_div::trimExplode(',',$TCA[$table]['types'][$k]['showitem'],1);
-						foreach($showItem as $key => $fieldInfo) {
+						foreach($showItem as $key => $fieldInfo)	{
 
 							$parts = explode(';',$fieldInfo);
 							$theField = trim($parts[0]);
 							$palette = trim($parts[0]).';;'.trim($parts[2]);
 
 								// insert before: find exact field name or palette with number
-							if (in_array($theField, $positionArr) OR in_array($palette, $positionArr) OR
-								in_array('before:'.$theField, $positionArr) OR in_array('before:'.$palette, $positionArr))	{
+							if (in_array($theField, $positionArr) || in_array($palette, $positionArr) || in_array('before:'.$theField, $positionArr) || in_array('before:'.$palette, $positionArr))	{
 								$showItem[$key]=trim($str).', '.$fieldInfo;
 								$append=false;
 								break;
 							}
 								// insert after
-							if (in_array('after:'.$theField, $positionArr) OR in_array('after:'.$palette, $positionArr))	{
+							if (in_array('after:'.$theField, $positionArr) || in_array('after:'.$palette, $positionArr))	{
 								$showItem[$key]=$fieldInfo.', '.trim($str);
 								$append=false;
 								break;
@@ -385,15 +382,17 @@ class t3lib_extMgm {
 	 * @param	string		Class path
 	 * @param	string		Title of module
 	 * @param	string		Menu array key - default is "function"
+	 * @param	string		Workspace conditions. Blank means all workspaces, any other string can be a comma list of "online", "offline" and "custom"
 	 * @return	void
 	 * @see t3lib_SCbase::mergeExternalItems()
 	 */
-	function insertModuleFunction($modname,$className,$classPath,$title,$MM_key='function')	{
+	function insertModuleFunction($modname,$className,$classPath,$title,$MM_key='function',$WS='')	{
 		global $TBE_MODULES_EXT;
 		$TBE_MODULES_EXT[$modname]['MOD_MENU'][$MM_key][$className]=array(
 			'name' => $className,
 			'path' => $classPath,
 			'title' => $title,
+			'ws' => $WS
 		);
 	}
 
@@ -471,8 +470,8 @@ class t3lib_extMgm {
 	 * Adds a service to the global services array
 	 *
 	 * @param	string		Extension key
-	 * @param	string		Service type
-	 * @param	string		Service key
+	 * @param	string		Service type, cannot be prefixed "tx_"
+	 * @param	string		Service key, must be prefixed "tx_" or "user_"
 	 * @param	array		Service description array
 	 * @return	void
 	 * @author	Ren Fritz <r.fritz@colorcube.de>
@@ -483,7 +482,10 @@ class t3lib_extMgm {
 		// even not available services will be included to make it possible to give the admin a feedback of non-available services.
 		// but maybe it's better to move non-available services to a different array??
 
-		if ($serviceType AND substr($serviceType,0,3)!='tx_' AND substr($serviceKey,0,3)=='tx_' AND is_array($info))	{
+		if ($serviceType AND
+			!t3lib_div::isFirstPartOfStr($serviceType, 'tx_') AND
+			(t3lib_div::isFirstPartOfStr($serviceKey, 'tx_') OR t3lib_div::isFirstPartOfStr($serviceKey, 'user_')) AND
+			is_array($info))	{
 
 			$info['priority'] = max(0,min(100,$info['priority']));
 
@@ -543,7 +545,7 @@ class t3lib_extMgm {
 	 * @author	Ren Fritz <r.fritz@colorcube.de>
 	 */
 	function findService($serviceType, $serviceSubType='', $excludeServiceKeys=array()) {
-		global $T3_SERVICES;
+		global $T3_SERVICES, $T3_VAR, $TYPO3_CONF_VARS;
 
 		$serviceKey = FALSE;
 		$serviceInfo = FALSE;
@@ -798,7 +800,7 @@ tt_content.'.$key.$prefix.' {
 	 * Usage: 3
 	 *
 	 * @param	string		$extKey is of course the extension key
-	 * @param	string		$path is the path where the template files (fixed names) include_static.txt (integer list of uids from the table "static_templates"), constants.txt, setup.txt and editorcfg.txt is found (relative to extPath, eg. 'static/')
+	 * @param	string		$path is the path where the template files (fixed names) include_static.txt (integer list of uids from the table "static_templates"), constants.txt, setup.txt, editorcfg.txt, and include_static_file.txt is found (relative to extPath, eg. 'static/'). The file include_static_file.txt, allows you to include other static templates defined in files, from your static template, and thus corresponds to the field 'include_static_file' in the sys_template table. The syntax for this is a commaseperated list of static templates to include, like:  EXT:css_styled_content/static/,EXT:da_newsletter_subscription/static/,EXT:cc_random_image/pi2/static/
 	 * @param	string		$title is the title in the selector box.
 	 * @return	void
 	 * @see addTypoScript()
@@ -937,53 +939,51 @@ tt_content.'.$key.$prefix.' {
 			if ($TYPO3_CONF_VARS['EXT']['extCache'] && t3lib_extMgm::isCacheFilesAvailable($cacheFilePrefix))	{
 					// Return cache file prefix:
 				$extensions['_CACHEFILE'] = $cacheFilePrefix;
-			} else {
-				// ... but if not, configure...
+			} else {	// ... but if not, configure...
+
+					// Prepare reserved filenames:
+				$files = t3lib_div::trimExplode(',','ext_localconf.php,ext_tables.php,ext_tables.sql,ext_tables_static+adt.sql,ext_typoscript_constants.txt,ext_typoscript_editorcfg.txt,ext_typoscript_setup.txt',1);
+
+					// Traverse extensions and check their existence:
+				clearstatcache();	// Clear file state cache to make sure we get good results from is_dir()
 				$temp_extensions = array_unique(t3lib_div::trimExplode(',',$rawExtList,1));
-				while(list(,$temp_extKey)=each($temp_extensions))	{
-					if (@is_dir(PATH_site.'typo3conf/ext/'.$temp_extKey))	{
-						$extensions[$temp_extKey]=array('type'=>'L','siteRelPath'=>'typo3conf/ext/'.$temp_extKey.'/','typo3RelPath'=>'../typo3conf/ext/'.$temp_extKey.'/');
-					} elseif (@is_dir(PATH_site.TYPO3_mainDir.'ext/'.$temp_extKey))	{
-						$extensions[$temp_extKey]=array('type'=>'G','siteRelPath'=>TYPO3_mainDir.'ext/'.$temp_extKey.'/','typo3RelPath'=>'ext/'.$temp_extKey.'/');
-					} elseif (@is_dir(PATH_site.TYPO3_mainDir.'sysext/'.$temp_extKey))	{
-						$extensions[$temp_extKey]=array('type'=>'S','siteRelPath'=>TYPO3_mainDir.'sysext/'.$temp_extKey.'/','typo3RelPath'=>'sysext/'.$temp_extKey.'/');
+				foreach($temp_extensions as $temp_extKey)	{
+						// Check local, global and system locations:
+					if (@is_dir(PATH_site.'typo3conf/ext/'.$temp_extKey.'/'))	{
+						$extensions[$temp_extKey] = array('type'=>'L', 'siteRelPath'=>'typo3conf/ext/'.$temp_extKey.'/', 'typo3RelPath'=>'../typo3conf/ext/'.$temp_extKey.'/');
+					} elseif (@is_dir(PATH_site.TYPO3_mainDir.'ext/'.$temp_extKey.'/'))	{
+						$extensions[$temp_extKey] = array('type'=>'G', 'siteRelPath'=>TYPO3_mainDir.'ext/'.$temp_extKey.'/', 'typo3RelPath'=>'ext/'.$temp_extKey.'/');
+					} elseif (@is_dir(PATH_site.TYPO3_mainDir.'sysext/'.$temp_extKey.'/'))	{
+						$extensions[$temp_extKey] = array('type'=>'S', 'siteRelPath'=>TYPO3_mainDir.'sysext/'.$temp_extKey.'/', 'typo3RelPath'=>'sysext/'.$temp_extKey.'/');
 					}
 
-					$files = t3lib_div::trimExplode(',','
-						ext_localconf.php,
-						ext_tables.php,
-						ext_tables.sql,
-						ext_tables_static+adt.sql,
-						ext_typoscript_constants.txt,
-						ext_typoscript_editorcfg.txt,
-						ext_typoscript_setup.txt
-					',1);
-					reset($files);
-					while(list(,$fName)=each($files))	{
-						$temp_filename = PATH_site.$extensions[$temp_extKey]['siteRelPath'].trim($fName);
-						if (is_array($extensions[$temp_extKey]) && @is_file($temp_filename))	{
-							$extensions[$temp_extKey][$fName]=$temp_filename;
+						// If extension was found, check for reserved filenames:
+					if (isset($extensions[$temp_extKey]))	{
+						foreach($files as $fName) 	{
+							$temp_filename = PATH_site.$extensions[$temp_extKey]['siteRelPath'].trim($fName);
+							if (is_array($extensions[$temp_extKey]) && @is_file($temp_filename))	{
+								$extensions[$temp_extKey][$fName] = $temp_filename;
+							}
 						}
 					}
 				}
 				unset($extensions['_CACHEFILE']);
 
-
-				// write cache?
-				if ($TYPO3_CONF_VARS['EXT']['extCache'])	{
+					// write cache?
+				if ($TYPO3_CONF_VARS['EXT']['extCache'] &&
+						@is_dir(PATH_site.TYPO3_mainDir.'sysext/') &&
+						@is_dir(PATH_site.TYPO3_mainDir.'ext/'))	{	// Must also find global and system extension directories to exist, otherwise caching cannot be allowed (since it is most likely a temporary server problem). This might fix a rare, unrepeatable bug where global/system extensions are not loaded resulting in fatal errors if that is cached!
 					$wrError = t3lib_extMgm::cannotCacheFilesWritable($cacheFilePrefix);
 					if ($wrError)	{
-//debug('Cannot write cache files: '.$wrError.'. Disabling the cache...');
 						$TYPO3_CONF_VARS['EXT']['extCache']=0;
 					} else {
-						// Write cache files:
+							// Write cache files:
 						$extensions = t3lib_extMgm::writeCacheFiles($extensions,$cacheFilePrefix);
 					}
 				}
 			}
 		}
 
-#debug($extensions);
 		return $extensions;
 	}
 
@@ -1121,6 +1121,25 @@ $TYPO3_LOADED_EXT = unserialize(stripslashes(\''.addslashes(serialize($extension
 
 		return $extensions;
 	}
+
+	/**
+	 * Unlink (delete) cache files
+	 *
+	 * @return	integer		Number of deleted files.
+	 */
+	function removeCacheFiles()	{
+		$cacheFiles = t3lib_extMgm::currentCacheFiles();
+		$out = 0;
+		if (is_array($cacheFiles))	{
+			reset($cacheFiles);
+			foreach($cacheFiles as $cfile)	{
+				@unlink($cfile);
+				clearstatcache();
+				$out++;
+			}
+		}
+		return $out;
+	}
 }
 
 ?>
