@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2006 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Class to setup values in localconf.php and verify the TYPO3 DB tables/fields
  *
- * $Id: class.t3lib_install.php,v 1.20.2.1 2004/09/19 20:17:09 mundaun Exp $
+ * $Id: class.t3lib_install.php 1642 2006-07-16 09:50:49Z stfl_wolfgang $
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
  */
@@ -37,27 +37,27 @@
  *
  *
  *   83: class t3lib_install
- *  111:     function t3lib_install()
+ *  108:     function t3lib_install()
  *
  *              SECTION: Writing to localconf.php
- *  134:     function setValueInLocalconfFile(&$line_array, $variable, $value)
- *  184:     function writeToLocalconf_control($inlines='')
- *  236:     function checkForBadString($string)
- *  249:     function slashValueForSingleDashes($value)
+ *  132:     function setValueInLocalconfFile(&$line_array, $variable, $value)
+ *  183:     function writeToLocalconf_control($inlines='',$absFullPath='')
+ *  253:     function checkForBadString($string)
+ *  266:     function slashValueForSingleDashes($value)
  *
  *              SECTION: SQL
- *  274:     function getFieldDefinitions_sqlContent($sqlContent)
- *  318:     function getFieldDefinitions_sqlContent_parseTypes(&$total)
- *  365:     function getFieldDefinitions_database()
- *  409:     function getDatabaseExtra($FDsrc, $FDcomp, $onlyTableList='')
- *  454:     function getUpdateSuggestions($diffArr,$keyList='extra,diff')
- *  550:     function assembleFieldDefinition($row)
- *  579:     function getStatementArray($sqlcode,$removeNonSQL=0,$query_regex='')
- *  619:     function getCreateTables($statements, $insertCountFlag=0)
- *  643:     function getTableInsertStatements($statements, $table)
- *  663:     function performUpdateQueries($arr,$keyArr)
- *  679:     function getListOfTables()
- *  695:     function generateUpdateDatabaseForm_checkboxes($arr,$label,$checked=1,$iconDis=0,$currentValue=array(),$cVfullMsg=0)
+ *  291:     function getFieldDefinitions_sqlContent($sqlContent)
+ *  359:     function getFieldDefinitions_sqlContent_parseTypes(&$total)
+ *  406:     function getFieldDefinitions_database()
+ *  450:     function getDatabaseExtra($FDsrc, $FDcomp, $onlyTableList='')
+ *  496:     function getUpdateSuggestions($diffArr,$keyList='extra,diff')
+ *  589:     function assembleFieldDefinition($row)
+ *  611:     function getStatementArray($sqlcode,$removeNonSQL=0,$query_regex='')
+ *  649:     function getCreateTables($statements, $insertCountFlag=0)
+ *  683:     function getTableInsertStatements($statements, $table)
+ *  704:     function performUpdateQueries($arr,$keyArr)
+ *  720:     function getListOfTables()
+ *  736:     function generateUpdateDatabaseForm_checkboxes($arr,$label,$checked=1,$iconDis=0,$currentValue=array(),$cVfullMsg=0)
  *
  * TOTAL FUNCTIONS: 17
  * (This index is automatically created/updated by the extension "extdeveval")
@@ -82,11 +82,11 @@ require_once(PATH_t3lib.'class.t3lib_sqlparser.php');
  */
 class t3lib_install {
 
+
 		// External, Static
 	var $updateIdentity = '';					// Set to string which identifies the script using this class.
 	var $deletedPrefixKey = 'zzz_deleted_';		// Prefix used for tables/fields when deleted/renamed.
 	var $dbUpdateCheckboxPrefix = 'TYPO3_INSTALL[database_update]';	// Prefix for checkbox fields when updating database.
-	var $mysqlVersion = '3.22';					// 3.22 or 3.23. If set to 3.23, then the expected format of the incoming sql-dump files are changed. 'DEFAULT' and 'NOT NULL' are reversed in order and 'DEFAULT' is lowercase. Also auto_incremented fields are without default definitions.
 	var $localconf_addLinesOnly = 0;			// If this is set, modifications to localconf.php is done by adding new lines to the array only. If unset, existing values are recognized and changed.
 	var $localconf_editPointToken = 'INSTALL SCRIPT EDIT POINT TOKEN - all lines after this points may be changed by the install script!';		// If set and addLinesOnly is disabled, lines will be change only if they are after this token (on a single line!) in the file
 	var $allowUpdateLocalConf = 0;		// If true, this class will allow the user to update the localconf.php file. Is set true in the init.php file.
@@ -100,9 +100,6 @@ class t3lib_install {
 	var $touchedLine = 0;				// updated with line in localconf.php file that was changed.
 
 
-
-
-
 	/**
 	 * Constructor function
 	 *
@@ -120,6 +117,7 @@ class t3lib_install {
 	 *
 	 * Writing to localconf.php
 	 *
+
 	 **************************************/
 
 	/**
@@ -178,11 +176,14 @@ class t3lib_install {
 	 * Writes or returns lines from localconf.php
 	 *
 	 * @param	array		Array of lines to write back to localconf.php. Possibly
+	 * @param	string		Absolute path of alternative file to use (Notice: this path is not validated in terms of being inside 'TYPO3 space')
 	 * @return	mixed		If $inlines is not an array it will return an array with the lines from localconf.php. Otherwise it will return a status string, either "continue" (updated) or "nochange" (not updated)
 	 * @see setValueInLocalconfFile()
 	 */
-	function writeToLocalconf_control($inlines='')	{
-		$writeToLocalconf_dat['file'] = PATH_typo3conf.'localconf.php';
+	function writeToLocalconf_control($inlines='',$absFullPath='')	{
+		$tmpExt = '.TMP.php';
+		$writeToLocalconf_dat['file'] = $absFullPath ? $absFullPath : PATH_typo3conf.'localconf.php';
+		$writeToLocalconf_dat['tmpfile'] = $writeToLocalconf_dat['file'].$tmpExt;
 
 			// Checking write state of localconf.php:
 		if (!$this->allowUpdateLocalConf)	{
@@ -193,7 +194,7 @@ class t3lib_install {
 		}
 
 				// Splitting localconf.php file into lines:
-		$lines = explode(chr(10),trim(t3lib_div::getUrl($writeToLocalconf_dat['file'])));
+		$lines = explode(chr(10),str_replace(chr(13),'',trim(t3lib_div::getUrl($writeToLocalconf_dat['file']))));
 		$writeToLocalconf_dat['endLine'] = array_pop($lines);	// Getting "? >" ending.
 
 			// Checking if "updated" line was set by this tool - if so remove old line.
@@ -210,14 +211,30 @@ class t3lib_install {
 			array_push($inlines,$writeToLocalconf_dat['endLine']);
 
 			if ($this->setLocalconf)	{
-				t3lib_div::writeFile($writeToLocalconf_dat['file'],implode(chr(10),$inlines));
-
-				if (strcmp(t3lib_div::getUrl($writeToLocalconf_dat['file']), implode(chr(10),$inlines)))	{
-					die('typo3temp/localconf.php was NOT updated properly (written content didn\'t match file content) - maybe write access problem?');
+				$success = FALSE;
+				if (!t3lib_div::writeFile($writeToLocalconf_dat['tmpfile'],implode(chr(10),$inlines)))	{
+					$msg = 'typo3conf/localconf.php'.$tmpExt.' could not be written - maybe a write access problem?';
 				}
+				elseif (strcmp(t3lib_div::getUrl($writeToLocalconf_dat['tmpfile']), implode(chr(10),$inlines)))	{
+					@unlink($writeToLocalconf_dat['tmpfile']);
+					$msg = 'typo3conf/localconf.php'.$tmpExt.' was NOT written properly (written content didn\'t match file content) - maybe a disk space problem?';
+				}
+				elseif (!@copy($writeToLocalconf_dat['tmpfile'],$writeToLocalconf_dat['file']))	{
+					$msg = 'typo3conf/localconf.php could not be replaced by typo3conf/localconf.php'.$tmpExt.' - maybe a write access problem?';
+				}
+				else {
+					@unlink($writeToLocalconf_dat['tmpfile']);
+					$success = TRUE;
+					$msg = 'Configuration written to typo3conf/localconf.php';
+				}
+				$this->messages[]= $msg;
 
-				$this->messages[]= 'Configuration written to typo3conf/localconf.php';
-				return 'continue';
+				if ($success)	{
+					return 'continue';
+				} else {
+					t3lib_div::sysLog($msg, 'Core', 3);
+					return 'nochange';
+				}
 			} else {
 				return 'nochange';
 			}
@@ -234,9 +251,7 @@ class t3lib_install {
 	 * @see setValueInLocalconfFile()
 	 */
 	function checkForBadString($string)	{
-		if (ereg('['.chr(10).chr(13).']',$string)){
-			return FALSE;
-		} else return TRUE;
+		return preg_match('/['.chr(10).chr(13).']/',$string) ? FALSE : TRUE;
 	}
 
 	/**
@@ -247,7 +262,11 @@ class t3lib_install {
 	 * @see setValueInLocalconfFile()
 	 */
 	function slashValueForSingleDashes($value)	{
-		return str_replace("'","\'",str_replace('\\','\\\\',$value));
+		$value = str_replace("'.chr(10).'", '###INSTALL_TOOL_LINEBREAK###', $value);
+		$value = str_replace("'","\'",str_replace('\\','\\\\',$value));
+		$value = str_replace('###INSTALL_TOOL_LINEBREAK###', "'.chr(10).'", $value);
+
+		return $value;
 	}
 
 
@@ -274,28 +293,52 @@ class t3lib_install {
 	function getFieldDefinitions_sqlContent($sqlContent)	{
 		$lines = t3lib_div::trimExplode(chr(10), $sqlContent,1);
 		$isTable = '';
+		$total = Array();
 
 		foreach($lines as $value)	{
 			if ($value[0]!='#')	{
 				if (!$isTable)	{
 					$parts = explode(' ',$value);
 					if ($parts[0]=='CREATE' && $parts[1]=='TABLE')	{
-						$isTable = $parts[2];
+						$isTable = str_replace( '`', '', $parts[2]);
 						if (TYPO3_OS=='WIN') { 	// tablenames are always lowercase on windows!
 							$isTable = strtolower($isTable);
 						}
 					}
 				} else {
 					if (substr($value,0,1)==')' && substr($value,-1)==';')	{
+						$ttype = array();
+						preg_match('/(ENGINE|TYPE)=([a-zA-Z]*)/',$value,$ttype);
+						$total[$isTable]['extra']['ttype'] = $ttype[2];
 						$isTable = '';
 					} else {
-						$lineV = ereg_replace(',$','',$value);
+						$lineV = preg_replace('/,$/','',$value);
+						$lineV = str_replace('UNIQUE KEY', 'UNIQUE', $lineV);
 						$parts = explode(' ',$lineV,2);
+
+							// Make sure there is no default value when auto_increment is set
+						if(stristr($parts[1],'auto_increment'))	{
+							$parts[1] = preg_replace('/ default \'0\'/i','',$parts[1]);
+						}
+							// "default" is always lower-case
+						if(strstr($parts[1], ' DEFAULT '))	{
+							$parts[1] = str_replace(' DEFAULT ', ' default ', $parts[1]);
+						}
+
+							// Change order of "default" and "null" statements
+						$parts[1] = preg_replace('/(.*) (default .*) (NOT NULL)/', '$1 $3 $2', $parts[1]);
+						$parts[1] = preg_replace('/(.*) (default .*) (NULL)/', '$1 $3 $2', $parts[1]);
+
+							// Remove double blanks
+						$parts[1] = preg_replace('/([^ ]+)[ ]+([^ ]+)/', '$1 $2', $parts[1]);
+
 						if ($parts[0]!='PRIMARY' && $parts[0]!='KEY' && $parts[0]!='UNIQUE')	{
-							$total[$isTable]['fields'][$parts[0]] = $parts[1];
+							$key = str_replace('`', '', $parts[0]);
+							$total[$isTable]['fields'][$key] = $parts[1];
 						} else {
 							$newParts = explode(' ',$parts[1],2);
-							$total[$isTable]['keys'][($parts[0]=='PRIMARY'?$parts[0]:$newParts[0])] = $lineV;
+							$key = str_replace('`', '', ($parts[0]=='PRIMARY'?$parts[0]:$newParts[0]));
+							$total[$isTable]['keys'][$key] = str_replace('`', '', $lineV);
 						}
 					}
 				}
@@ -323,35 +366,37 @@ class t3lib_install {
 				// Init SQL parser:
 			$sqlParser = t3lib_div::makeInstance('t3lib_sqlparser');
 			foreach($total as $table => $cfg)	{
-				foreach($cfg['fields'] as $fN => $fType)	{
-					$orig_fType = $fType;
-					$fInfo = $sqlParser->parseFieldDef($fType);
+				if (is_array($cfg['fields'])) {
+					foreach($cfg['fields'] as $fN => $fType)	{
+						$orig_fType = $fType;
+						$fInfo = $sqlParser->parseFieldDef($fType);
 
-					switch($fInfo['fieldType'])	{
-						case 'char':
-						case 'varchar':
-							$newSize = round($fInfo['value']*$mSize);
+						switch($fInfo['fieldType'])	{
+							case 'char':
+							case 'varchar':
+								$newSize = round($fInfo['value']*$mSize);
 
-							if ($newSize <= 255)	{
-								$fInfo['value'] = $newSize;
-							} else {
-								$fInfo = array(
-									'fieldType' => 'text',
-									'featureIndex' => array(
-										'NOTNULL' => array(
-											'keyword' => 'NOT NULL'
+								if ($newSize <= 255)	{
+									$fInfo['value'] = $newSize;
+								} else {
+									$fInfo = array(
+										'fieldType' => 'text',
+										'featureIndex' => array(
+											'NOTNULL' => array(
+												'keyword' => 'NOT NULL'
+											)
 										)
-									)
-								);
-							}
-						break;
-						case 'tinytext':
-							$fInfo['fieldType'] = 'text';
-						break;
-					}
+									);
+								}
+							break;
+							case 'tinytext':
+								$fInfo['fieldType'] = 'text';
+							break;
+						}
 
-					$total[$table]['fields'][$fN] = $sqlParser->compileFieldCfg($fInfo);
-					if ($sqlParser->parse_error)	die($sqlParser->parse_error);
+						$total[$table]['fields'][$fN] = $sqlParser->compileFieldCfg($fInfo);
+						if ($sqlParser->parse_error)	die($sqlParser->parse_error);
+					}
 				}
 			}
 		}
@@ -404,9 +449,10 @@ class t3lib_install {
 	 * @param	array		Field definitions, source (from getFieldDefinitions_sqlContent())
 	 * @param	array		Field definitions, comparison. (from getFieldDefinitions_database())
 	 * @param	string		Table names (in list) which is the ONLY one observed.
+	 * @param	boolean		If set, this function ignores NOT NULL statements of the sql file field definition when comparing current field definition from database with field definition from sql file. This way, NOT NULL statements will be executed when the field is initially created, but the sql parser will never complain about missing NOT NULL statements afterwards.
 	 * @return	array		Returns an array with 1) all elements from $FSsrc that is not in $FDcomp (in key 'extra') and 2) all elements from $FSsrc that is difference from the ones in $FDcomp
 	 */
-	function getDatabaseExtra($FDsrc, $FDcomp, $onlyTableList='')	{
+	function getDatabaseExtra($FDsrc, $FDcomp, $onlyTableList='',$ignoreNotNullWhenComparing=true)	{
 		$extraArr = array();
 		$diffArr = array();
 
@@ -420,10 +466,11 @@ class t3lib_install {
 						$keyTypes = explode(',','fields,keys');
 						foreach($keyTypes as $theKey)	{
 							if (is_array($info[$theKey]))	{
-								foreach($info[$theKey] as $fieldN => $fieldC) {
+								foreach($info[$theKey] as $fieldN => $fieldC)	{
+									$fieldN = str_replace('`','',$fieldN);
 									if (!isset($FDcomp[$table][$theKey][$fieldN]))	{
 										$extraArr[$table][$theKey][$fieldN] = $fieldC;
-									} elseif (strcmp($FDcomp[$table][$theKey][$fieldN], $fieldC)){
+									} elseif (strcmp($FDcomp[$table][$theKey][$fieldN], $ignoreNotNullWhenComparing?str_replace(' NOT NULL', '', $fieldC):$fieldC))	{
 										$diffArr[$table][$theKey][$fieldN] = $fieldC;
 										$diffArr_cur[$table][$theKey][$fieldN] = $FDcomp[$table][$theKey][$fieldN];
 									}
@@ -482,7 +529,7 @@ class t3lib_install {
 										$statement = 'ALTER TABLE '.$table.' ADD '.$fN.' '.$fV.';';
 										$statements['add'][md5($statement)] = $statement;
 									}
-								} elseif ($theKey=='diff') {
+								} elseif ($theKey=='diff')	{
 									$statement = 'ALTER TABLE '.$table.' CHANGE '.$fN.' '.$fN.' '.$fV.';';
 									$statements['change'][md5($statement)] = $statement;
 									$statements['change_currentValue'][md5($statement)] = $diffArr['diff_currentValues'][$table]['fields'][$fN];
@@ -493,11 +540,7 @@ class t3lib_install {
 					if (is_array($info['keys']))	{
 						foreach($info['keys'] as $fN => $fV) {
 							if ($info['whole_table'])	{
-								if ($fN=='PRIMARY')	{
-									$whole_table[] = $fV;
-								} else {
-									$whole_table[] = $fV;
-								}
+								$whole_table[] = $fV;
 							} else {
 								if ($theKey=='extra')	{
 									if ($remove)	{
@@ -530,7 +573,8 @@ class t3lib_install {
 							list($count) = $GLOBALS['TYPO3_DB']->sql_fetch_row($res);
 							$statements['tables_count'][md5($statement)] = $count?'Records in table: '.$count:'';
 						} else {
-							$statement = 'CREATE TABLE '.$table." (\n".implode(",\n",$whole_table)."\n) TYPE=MyISAM;";
+							$statement = 'CREATE TABLE '.$table." (\n".implode(",\n",$whole_table)."\n)";
+							$statement .= ($info['extra']['ttype']) ? ' TYPE='.$info['extra']['ttype'].';' : ';';
 							$statements['create_table'][md5($statement)]=$statement;
 						}
 					}
@@ -549,22 +593,15 @@ class t3lib_install {
 	 */
 	function assembleFieldDefinition($row)	{
 		$field[] = $row['Type'];
-		if ($this->mysqlVersion=='3.23' && !$row['Null'])	{
-			$field[] = 'NOT NULL';
-		}
+		// if (!$row['Null'])	{ $field[] = 'NOT NULL'; }
 		if (!strstr($row['Type'],'blob') && !strstr($row['Type'],'text'))	{
-			// This code will return a default value if the sql-file version is not 3.23 and the field is not auto-incremented. In 3.23 files, the auto-incremented fields do not have a default definition.
-			// Furthermore if the file is 3.22 the default value of auto-incremented fields are expected to always be zero which is why the default value is passed through intval(). Thus it should work on 3.23 MySQL servers.
-			if ($this->mysqlVersion!='3.23' || !stristr($row['Extra'],'auto_increment'))	{
-				$field[] = ($this->mysqlVersion=='3.23'?'default':'DEFAULT')." '".(stristr($row['Extra'],'auto_increment')?intval($row['Default']):addslashes($row['Default']))."'";
+				// Add a default value if the field is not auto-incremented (these fields never have a default definition).
+			if (!stristr($row['Extra'],'auto_increment'))	{
+				$field[] = 'default '."'".(addslashes($row['Default']))."'";
 			}
 		}
-		if ($this->mysqlVersion!='3.23' && !$row['Null'])	{
-			$field[] = 'NOT NULL';
-		}
-		if ($row['Extra'])	{
-			$field[] = $row['Extra'];
-		}
+		if ($row['Extra'])	{ $field[] = $row['Extra']; }
+
 		return implode(' ',$field);
 	}
 
@@ -584,12 +621,10 @@ class t3lib_install {
 		$statementArrayPointer = 0;
 
 		foreach($sqlcodeArr as $line => $linecontent)	{
-
-				// Setting MySQL Version if necessary.
-			if (substr(trim($linecontent),0,16)=='# Server version')	{
-				$this->mysqlVersion = doubleval(substr(trim($linecontent),16));
-			}
 			$is_set = 0;
+			if(stristr($linecontent,'auto_increment')) {
+				$linecontent = eregi_replace(' default \'0\'','',$linecontent);
+			}
 
 			if (!$removeNonSQL || (strcmp(trim($linecontent),'') && substr(trim($linecontent),0,1)!='#' && substr(trim($linecontent),0,2)!='--'))	{		// '--' is seen as mysqldump comments from server version 3.23.49
 				$statementArray[$statementArrayPointer].= $linecontent;
@@ -618,18 +653,28 @@ class t3lib_install {
 	 */
 	function getCreateTables($statements, $insertCountFlag=0)	{
 		$crTables = array();
+		$insertCount = array();
 		foreach($statements as $line => $linecontent)	{
-			if (eregi('^create[[:space:]]*table[[:space:]]*([[:alnum:]_]*)',substr($linecontent,0,100),$reg))	{
+			$reg = array();
+			if (eregi('^create[[:space:]]*table[[:space:]]*[`]?([[:alnum:]_]*)[`]?',substr($linecontent,0,100),$reg))	{
 				$table = trim($reg[1]);
 				if ($table)	{
-					if (TYPO3_OS=='WIN')	{$table=strtolower($table);}	// tablenames are always lowercase on windows!
+					if (TYPO3_OS=='WIN')	{ $table=strtolower($table); }	// table names are always lowercase on Windows!
+					$sqlLines = explode(chr(10), $linecontent);
+					foreach($sqlLines as $k=>$v)	{
+						if(stristr($v,'auto_increment')) {
+							$sqlLines[$k] = eregi_replace(' default \'0\'','',$v);
+						}
+					}
+					$linecontent = implode(chr(10), $sqlLines);
 					$crTables[$table] = $linecontent;
 				}
-			} elseif ($insertCountFlag && eregi('^insert[[:space:]]*into[[:space:]]*([[:alnum:]_]*)',substr($linecontent,0,100),$reg))	{
+			} elseif ($insertCountFlag && eregi('^insert[[:space:]]*into[[:space:]]*[`]?([[:alnum:]_]*)[`]?',substr($linecontent,0,100),$reg))	{
 				$nTable = trim($reg[1]);
 				$insertCount[$nTable]++;
 			}
 		}
+
 		return array($crTables,$insertCount);
 	}
 
@@ -643,7 +688,8 @@ class t3lib_install {
 	function getTableInsertStatements($statements, $table)	{
 		$outStatements=array();
 		foreach($statements as $line => $linecontent)	{
-			if (eregi('^insert[[:space:]]*into[[:space:]]*([[:alnum:]_]*)',substr($linecontent,0,100),$reg))	{
+			$reg = array();
+			if (preg_match('/^insert[[:space:]]*into[[:space:]]*[`]?([[:alnum:]_]*)[`]?/i',substr($linecontent,0,100),$reg))	{
 				$nTable = trim($reg[1]);
 				if ($nTable && !strcmp($table,$nTable))	{
 					$outStatements[]=$linecontent;
@@ -699,19 +745,19 @@ class t3lib_install {
 				$ico = '';
 				if ($iconDis)	{
 					if (stristr($string,' user_'))	{
-						$ico.= '<img src="'.$this->backPath.'t3lib/gfx/icon_warning.gif" width="18" height="16" align="top" alt="" /><strong>(USER) </strong>';
+						$ico.= '<img src="'.$this->backPath.'gfx/icon_warning.gif" width="18" height="16" align="top" alt="" /><strong>(USER) </strong>';
 					}
 					if (stristr($string,' app_'))	{
-						$ico.= '<img src="'.$this->backPath.'t3lib/gfx/icon_warning.gif" width="18" height="16" align="top" alt="" /><strong>(APP) </strong>';
+						$ico.= '<img src="'.$this->backPath.'gfx/icon_warning.gif" width="18" height="16" align="top" alt="" /><strong>(APP) </strong>';
 					}
 					if (stristr($string,' ttx_') || stristr($string,' tx_'))	{
-						$ico.= '<img src="'.$this->backPath.'t3lib/gfx/icon_warning.gif" width="18" height="16" align="top" alt="" /><strong>(EXT) </strong>';
+						$ico.= '<img src="'.$this->backPath.'gfx/icon_warning.gif" width="18" height="16" align="top" alt="" /><strong>(EXT) </strong>';
 					}
 				}
 				$out[]='
 					<tr>
-						<td valign="top"><input type="checkbox" name="'.$this->dbUpdateCheckboxPrefix.'['.$key.']" value="1"'.($checked?' checked="checked"':'').' /></td>
-						<td nowrap="nowrap">'.nl2br($ico.htmlspecialchars($string)).'</td>
+						<td valign="top"><input type="checkbox" id="db-'.$key.'" name="'.$this->dbUpdateCheckboxPrefix.'['.$key.']" value="1"'.($checked?' checked="checked"':'').' /></td>
+						<td nowrap="nowrap"><label for="db-'.$key.'">'.nl2br($ico.htmlspecialchars($string)).'</label></td>
 					</tr>';
 				if (isset($currentValue[$key]))	{
 					$out[]='
