@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the Typo3 project. The Typo3 project is
@@ -27,7 +27,7 @@
 /**
  * Parent class for "Services" classes
  *
- * $Id: class.t3lib_svbase.php,v 1.14.2.1 2004/09/19 20:17:10 mundaun Exp $
+ * $Id: class.t3lib_svbase.php 1421 2006-04-10 09:27:15Z mundaun $
  * TODO: temp files are not removed
  *
  * @author	Ren Fritz <r.fritz@colorcube.de>
@@ -37,55 +37,53 @@
  *
  *
  *
- *  127: class t3lib_svbase
+ *  125: class t3lib_svbase
  *
  *              SECTION: Get service meta information
  *  191:     function getServiceInfo()
- *  199:     function getServiceKey()
- *  207:     function getServiceTitle()
- *  220:     function getServiceOption($optionName, $defaultValue='', $includeDefaultConfig=TRUE)
+ *  201:     function getServiceKey()
+ *  211:     function getServiceTitle()
+ *  224:     function getServiceOption($optionName, $defaultValue='', $includeDefaultConfig=TRUE)
  *
  *              SECTION: Error handling
- *  255:     function devLog($msg, $severity=0, $dataVar=FALSE)
- *  269:     function errorPush($errNum=T3_ERR_SV_GENERAL, $errMsg='Unspecified error occured')
- *  284:     function errorPull()
- *  296:     function getLastError()
- *  311:     function getLastErrorMsg()
- *  326:     function getErrorMsgArray()
- *  344:     function getLastErrorArray()
- *  353:     function resetErrors()
+ *  259:     function devLog($msg, $severity=0, $dataVar=FALSE)
+ *  273:     function errorPush($errNum=T3_ERR_SV_GENERAL, $errMsg='Unspecified error occured')
+ *  288:     function errorPull()
+ *  300:     function getLastError()
+ *  315:     function getLastErrorMsg()
+ *  330:     function getErrorMsgArray()
+ *  348:     function getLastErrorArray()
+ *  357:     function resetErrors()
  *
  *              SECTION: General service functions
- *  373:     function checkExec($progList)
- *  395:     function deactivateService()
- *  401:     function available()
+ *  377:     function checkExec($progList)
+ *  401:     function deactivateService()
  *
  *              SECTION: IO tools
- *  439:     function checkInputFile ($absFile)
- *  460:     function readFile ($absFile, $length=0)
- *  485:     function writeFile ($content, $absFile='')
- *  511:     function tempFile ($filePrefix)
- *  529:     function registerTempFile ($absFile)
- *  539:     function unlinkTempFiles ()
+ *  427:     function checkInputFile ($absFile)
+ *  448:     function readFile ($absFile, $length=0)
+ *  473:     function writeFile ($content, $absFile='')
+ *  499:     function tempFile ($filePrefix)
+ *  517:     function registerTempFile ($absFile)
+ *  527:     function unlinkTempFiles ()
  *
  *              SECTION: IO input
- *  561:     function setInput ($content, $type='')
- *  575:     function setInputFile ($absFile, $type='')
- *  588:     function getInput ()
- *  603:     function getInputFile ($createFile='')
+ *  549:     function setInput ($content, $type='')
+ *  563:     function setInputFile ($absFile, $type='')
+ *  576:     function getInput ()
+ *  591:     function getInputFile ($createFile='')
  *
  *              SECTION: IO output
- *  628:     function setOutputFile ($absFile)
- *  638:     function getOutput ()
- *  652:     function getOutputFile ($absFile='')
+ *  616:     function setOutputFile ($absFile)
+ *  626:     function getOutput ()
+ *  640:     function getOutputFile ($absFile='')
  *
  *              SECTION: Service implementation
- *  676:     function init()
- *  700:     function reset()
- *  715:     function __destruct()
- *  721:     function process($content='', $type='', $conf=array())
+ *  664:     function init()
+ *  688:     function reset()
+ *  703:     function __destruct()
  *
- * TOTAL FUNCTIONS: 32
+ * TOTAL FUNCTIONS: 30
  * (This index is automatically created/updated by the extension "extdeveval")
  *
  */
@@ -186,6 +184,8 @@ class t3lib_svbase {
 
 
 	/**
+	 * Returns internal information array for service
+	 *
 	 * @return	array		service description array
 	 */
 	function getServiceInfo() {
@@ -194,6 +194,8 @@ class t3lib_svbase {
 
 
 	/**
+	 * Returns the service key of the service
+	 *
 	 * @return	string		service key
 	 */
 	function getServiceKey() {
@@ -202,6 +204,8 @@ class t3lib_svbase {
 
 
 	/**
+	 * Returns the title of the service
+	 *
 	 * @return	string		service title
 	 */
 	function getServiceTitle() {
@@ -225,7 +229,7 @@ class t3lib_svbase {
 		$svOptions = $TYPO3_CONF_VARS['SVCONF'][$this->info['serviceType']];
 
 		if(isset($svOptions[$this->info['serviceKey']][$optionName])) {
-			$config = $svOptions['default'][$optionName];
+			$config = $svOptions[$this->info['serviceKey']][$optionName];
 		} elseif($includeDefaultConfig AND isset($svOptions['default'][$optionName])) {
 			$config = $svOptions['default'][$optionName];
 		}
@@ -269,7 +273,7 @@ class t3lib_svbase {
 	function errorPush($errNum=T3_ERR_SV_GENERAL, $errMsg='Unspecified error occured') {
 		array_push($this->error, array('nr'=>$errNum, 'msg'=>$errMsg));
 
-		if (is_object($GLOBALS["TT"])) {
+		if (is_object($GLOBALS['TT'])) {
 			$GLOBALS['TT']->setTSlogMessage($errMsg,2);
 		}
 
@@ -371,6 +375,8 @@ class t3lib_svbase {
 	 * @return	boolean		return FALSE if one program was not found
 	 */
 	function checkExec($progList) {
+		global $T3_VAR, $TYPO3_CONF_VARS;
+
 		$ret = TRUE;
 
 		require_once(PATH_t3lib.'class.t3lib_exec.php');
@@ -397,29 +403,11 @@ class t3lib_svbase {
 	}
 
 
-/**
-function available()	{
-	global $AB,$BE_USER,$LANG,$BACK_PATH,$TCA_DESCR,$TCA,$CLIENT,$TYPO3_CONF_VARS;
 
-	// check if the service is available at runtime
-	// the sense of this method is that the service might need some more information to check the availablity
 
-	/ *
 
-	$excludeServiceKeys='';
-	while (is_object($serviceObj = t3lib_div::makeInstanceService('anyService','', $excludeServiceKeys))) {
-		if ($serviceObj->available('some special parm to check availablity')) {
-			break;
-		}
-		$excludeServiceKeys .= ','.$serviceObj->getServiceKey;
-		unset($serviceObj);
-	}
 
-	* /
 
-	return TRUE;
-}
-*/
 
 
 	/***************************************
@@ -437,7 +425,7 @@ function available()	{
 	 * @return	string		File name or FALSE.
 	 */
 	function checkInputFile ($absFile)	{
-		if(@is_file($absFile)) {
+		if(t3lib_div::isAllowedAbsPath($absFile) && @is_file($absFile)) {
 			if(@is_readable($absFile)) {
 				return $absFile;
 			} else {
@@ -489,7 +477,7 @@ function available()	{
 			$absFile = $this->tempFile($this->prefixId);
 		}
 
-		if($absFile) {
+		if($absFile && t3lib_div::isAllowedAbsPath($absFile)) {
 			if ($fd = @fopen($absFile,'wb')) {
 				@fwrite($fd, $content);
 				@fclose($fd);
@@ -554,9 +542,9 @@ function available()	{
 	/**
 	 * Set the input content for service processing.
 	 *
-	 * @param	mixed
-	 * @param	[type]		$type: ...
-	 * @return	[type]		...
+	 * @param	mixed		Input content (going into ->inputContent)
+	 * @param	string		The type of the input content (or file). Might be the same as the service subtypes.
+	 * @return	void
 	 */
 	function setInput ($content, $type='') {
 		$this->inputContent = $content;
@@ -569,8 +557,8 @@ function available()	{
 	 * Set the input file name for service processing.
 	 *
 	 * @param	string		file name
-	 * @param	[type]		$type: ...
-	 * @return	[type]		...
+	 * @param	string		The type of the input content (or file). Might be the same as the service subtypes.
+	 * @return	void
 	 */
 	function setInputFile ($absFile, $type='') {
 		$this->inputContent = '';
@@ -581,7 +569,7 @@ function available()	{
 
 	/**
 	 * Get the input content.
-	 * Will be read from input file if needed.
+	 * Will be read from input file if needed. (That is if ->inputContent is empty and ->inputFile is not)
 	 *
 	 * @return	mixed
 	 */
@@ -623,7 +611,7 @@ function available()	{
 	 * Set the output file name.
 	 *
 	 * @param	string		file name
-	 * @return	[type]		...
+	 * @return	void
 	 */
 	function setOutputFile ($absFile) {
 		$this->outputFile = $absFile;
@@ -644,9 +632,9 @@ function available()	{
 
 
 	/**
-	 * Get the output file name.
+	 * Get the output file name. If no output file is set, the ->out buffer is written to the file given by input parameter filename
 	 *
-	 * @param	[type]		$absFile: ...
+	 * @param	string		Absolute filename to write to
 	 * @return	mixed
 	 */
 	function getOutputFile ($absFile='') {
@@ -718,7 +706,7 @@ function available()	{
 
 
 	/* every service type has it's own API
-	function process($content='', $type='', $conf=array())	{
+	function process($content='', $type='', $conf=array())	{	//
 	}
 	*/
 
