@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Contains base class for creating a browsable array/page/folder tree in HTML
  *
- * $Id: class.t3lib_treeview.php,v 1.18 2004/09/13 22:57:19 typo3 Exp $
+ * $Id: class.t3lib_treeview.php 1421 2006-04-10 09:27:15Z mundaun $
  * Revised for TYPO3 3.6 November/2003 by Kasper Skaarhoj
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
@@ -39,49 +39,49 @@
  *
  *
  *  115: class t3lib_treeView
- *  267:     function init($clause='', $orderByFields='')
- *  298:     function setTreeName($treeName='')
- *  312:     function addField($field,$noCheck=0)
- *  326:     function reset()
+ *  270:     function init($clause='', $orderByFields='')
+ *  301:     function setTreeName($treeName='')
+ *  315:     function addField($field,$noCheck=0)
+ *  329:     function reset()
  *
  *              SECTION: output
- *  346:     function getBrowsableTree()
- *  407:     function printTree($treeArr='')
+ *  349:     function getBrowsableTree()
+ *  412:     function printTree($treeArr='')
  *
  *              SECTION: rendering parts
- *  460:     function PMicon($row,$a,$c,$nextCount,$exp)
- *  482:     function PM_ATagWrap($icon,$cmd,$bMark='')
- *  504:     function wrapTitle($title,$row,$bank=0)
- *  517:     function wrapIcon($icon,$row)
- *  528:     function addTagAttributes($icon,$attr)
- *  540:     function wrapStop($str,$row)
+ *  467:     function PMicon($row,$a,$c,$nextCount,$exp)
+ *  489:     function PM_ATagWrap($icon,$cmd,$bMark='')
+ *  511:     function wrapTitle($title,$row,$bank=0)
+ *  524:     function wrapIcon($icon,$row)
+ *  535:     function addTagAttributes($icon,$attr)
+ *  547:     function wrapStop($str,$row)
  *
  *              SECTION: tree handling
- *  568:     function expandNext($id)
- *  578:     function initializePositionSaving()
- *  605:     function savePosition()
+ *  575:     function expandNext($id)
+ *  585:     function initializePositionSaving()
+ *  612:     function savePosition()
  *
  *              SECTION: Functions that might be overwritten by extended classes
- *  634:     function getRootIcon($rec)
- *  647:     function getIcon($row)
- *  666:     function getTitleStr($row,$titleLen=30)
- *  678:     function getTitleAttrib($row)
- *  688:     function getId($row)
- *  698:     function getJumpToParam($row)
+ *  641:     function getRootIcon($rec)
+ *  654:     function getIcon($row)
+ *  673:     function getTitleStr($row,$titleLen=30)
+ *  685:     function getTitleAttrib($row)
+ *  695:     function getId($row)
+ *  705:     function getJumpToParam($row)
  *
  *              SECTION: tree data buidling
- *  731:     function getTree($uid, $depth=999, $depthData='',$blankLineCode='')
+ *  739:     function getTree($uid, $depth=999, $depthData='',$blankLineCode='',$subCSSclass='')
  *
  *              SECTION: Data handling
- *  828:     function getCount($uid)
- *  855:     function getRootRecord($uid)
- *  868:     function getRecord($uid)
- *  885:     function getDataInit($parentId)
- *  915:     function getDataCount(&$res)
- *  932:     function getDataNext(&$res)
- *  960:     function getDataFree(&$res)
- *  980:     function setDataFromArray(&$dataArr,$traverse=FALSE,$pid=0)
- * 1009:     function setDataFromTreeArray(&$treeArr, &$treeLookupArr)
+ *  839:     function getCount($uid)
+ *  865:     function getRootRecord($uid)
+ *  878:     function getRecord($uid)
+ *  898:     function getDataInit($parentId,$subCSSclass='')
+ *  929:     function getDataCount(&$res)
+ *  947:     function getDataNext(&$res,$subCSSclass='')
+ *  986:     function getDataFree(&$res)
+ * 1006:     function setDataFromArray(&$dataArr,$traverse=FALSE,$pid=0)
+ * 1035:     function setDataFromTreeArray(&$treeArr, &$treeLookupArr)
  *
  * TOTAL FUNCTIONS: 31
  * (This index is automatically created/updated by the extension "extdeveval")
@@ -122,6 +122,8 @@ class t3lib_treeView {
 	var $ext_IconMode = false;		// If true, no context menu is rendered on icons. If set to "titlelink" the icon is linked as the title is.
 	var $addSelfId = 0;				// If set, the id of the mounts will be added to the internal ids array
 	var $title='no title';			// Used if the tree is made of records (not folders for ex.)
+	var $showDefaultTitleAttribute = FALSE;		// If true, a default title attribute showing the UID of the record is shown. This cannot be enabled by default because it will destroy many applications where another title attribute is in fact applied later.
+	var $highlightPagesWithVersions = TRUE;		// If true, pages containing child records which has versions will be highlighted in yellow. This might be too expensive in terms of processing power.
 
 	/**
 	 * Needs to be initialized with $GLOBALS['BE_USER']
@@ -212,6 +214,7 @@ class t3lib_treeView {
 
 	/**
 	 * If true, HTML code is also accumulated in ->tree array during rendering of the tree.
+	 * If 2, then also the icon prefix code (depthData) is stored
 	 */
 	var $makeHTML=1;
 
@@ -380,15 +383,17 @@ class t3lib_treeView {
 			}
 
 			if (is_array($rootRec))	{
+				$uid = $rootRec['uid'];		// In case it was swapped inside getRecord due to workspaces.
+
 					// Add the root of the mount to ->tree
-				$this->tree[]=array('HTML'=>$firstHtml,'row'=>$rootRec,'bank'=>$this->bank);
+				$this->tree[]=array('HTML'=>$firstHtml, 'row'=>$rootRec, 'bank'=>$this->bank);
 
 					// If the mount is expanded, go down:
 				if ($isOpen)	{
 						// Set depth:
 					$depthD='<img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/ol/blank.gif','width="18" height="16"').' alt="" />';
 					if ($this->addSelfId)	$this->ids[] = $uid;
-					$this->getTree($uid,999,$depthD);
+					$this->getTree($uid,999,$depthD,'',$rootRec['_SUBCSSCLASS']);
 				}
 
 					// Add tree:
@@ -421,9 +426,11 @@ class t3lib_treeView {
 
 		foreach($treeArr as $k => $v)	{
 			$idAttr = htmlspecialchars($this->domIdPrefix.$this->getId($v['row']).'_'.$v['bank']);
+
 			$out.='
 				<tr>
-					<td id="'.$idAttr.'">'.
+					<td id="'.$idAttr.'"'.
+						($v['row']['_CSSCLASS'] ? ' class="'.$v['row']['_CSSCLASS'].'"' : '').'>'.
 						$v['HTML'].
 						$this->wrapTitle($this->getTitleStr($v['row'],$titleLen),$v['row'],$v['bank']).
 					'</td>
@@ -502,7 +509,7 @@ class t3lib_treeView {
 	 * @access private
 	 */
 	function wrapTitle($title,$row,$bank=0)	{
-		$aOnClick = 'return jumpTo(\''.$this->getJumpToParam($row).'\',this,\''.$this->domIdPrefix.$this->getId($row).'_'.$bank.'\');';
+		$aOnClick = 'return jumpTo(\''.$this->getJumpToParam($row).'\',this,\''.$this->domIdPrefix.$this->getId($row).'\','.$bank.');';
 		return '<a href="#" onclick="'.htmlspecialchars($aOnClick).'">'.$title.'</a>';
 	}
 
@@ -646,9 +653,9 @@ class t3lib_treeView {
 	 */
 	function getIcon($row) {
 		if ($this->iconPath && $this->iconName) {
-			$icon = '<img'.t3lib_iconWorks::skinImg('',$this->iconPath.$this->iconName,'width="18" height="16"').' alt="" />';
+			$icon = '<img'.t3lib_iconWorks::skinImg('',$this->iconPath.$this->iconName,'width="18" height="16"').' alt=""'.($this->showDefaultTitleAttribute ? ' title="UID: '.$row['uid'].'"':'').' />';
 		} else {
-			$icon = t3lib_iconWorks::getIconImage($this->table,$row,$this->backPath,'align="top" class="c-recIcon"');
+			$icon = t3lib_iconWorks::getIconImage($this->table,$row,$this->backPath,'align="top" class="c-recIcon"'.($this->showDefaultTitleAttribute ? ' title="UID: '.$row['uid'].'"':''));
 		}
 
 		return $this->wrapIcon($icon,$row);
@@ -726,9 +733,10 @@ class t3lib_treeView {
 	 * @param	integer		Max depth (recursivity limit)
 	 * @param	string		HTML-code prefix for recursive calls.
 	 * @param	string		? (internal)
+	 * @param	string		CSS class to use for <td> sub-elements
 	 * @return	integer		The count of items on the level
 	 */
-	function getTree($uid, $depth=999, $depthData='',$blankLineCode='')	{
+	function getTree($uid, $depth=999, $depthData='',$blankLineCode='',$subCSSclass='')	{
 
 			// Buffer for id hierarchy is reset:
 		$this->buffer_idH=array();
@@ -739,12 +747,12 @@ class t3lib_treeView {
 		$HTML='';
 		$a=0;
 
-		$res = $this->getDataInit($uid);
+		$res = $this->getDataInit($uid,$subCSSclass);
 		$c = $this->getDataCount($res);
 		$crazyRecursionLimiter = 999;
 
 			// Traverse the records:
-		while ($crazyRecursionLimiter>0 && $row = $this->getDataNext($res))	{
+		while ($crazyRecursionLimiter>0 && $row = $this->getDataNext($res,$subCSSclass))	{
 			$a++;
 			$crazyRecursionLimiter--;
 
@@ -760,16 +768,18 @@ class t3lib_treeView {
 			}
 
 				// Accumulate the id of the element in the internal arrays
-			$this->ids[]=$idH[$row['uid']]['uid']=$row['uid'];
-			$this->ids_hierarchy[$depth][]=$row['uid'];
+			$this->ids[]=$idH[$row['uid']]['uid'] = $row['uid'];
+			$this->ids_hierarchy[$depth][] = $row['uid'];
 
 				// Make a recursive call to the next level
+			$HTML_depthData = $depthData.'<img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/ol/'.$LN.'.gif','width="18" height="16"').' alt="" />';
 			if ($depth>1 && $this->expandNext($newID) && !$row['php_tree_stop'])	{
 				$nextCount=$this->getTree(
 						$newID,
 						$depth-1,
-						$this->makeHTML?$depthData.'<img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/ol/'.$LN.'.gif','width="18" height="16"').' alt="" />':'',
-						$blankLineCode.','.$LN
+						$this->makeHTML ? $HTML_depthData : '',
+						$blankLineCode.','.$LN,
+						$row['_SUBCSSCLASS']
 					);
 				if (count($this->buffer_idH))	$idH[$row['uid']]['subrow']=$this->buffer_idH;
 				$exp=1;	// Set "did expand" flag
@@ -789,6 +799,7 @@ class t3lib_treeView {
 			$this->tree[$treeKey] = Array(
 				'row'=>$row,
 				'HTML'=>$HTML,
+				'HTML_depthData' => $this->makeHTML==2 ? $HTML_depthData : '',
 				'invertedDepth'=>$depth,
 				'blankLineCode'=>$blankLineCode,
 				'bank' => $this->bank
@@ -833,11 +844,10 @@ class t3lib_treeView {
 			$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery(
 						'count(*)',
 						$this->table,
-						$this->parentField.'="'.$GLOBALS['TYPO3_DB']->quoteStr($uid, $this->table).'"'.
+						$this->parentField.'='.$GLOBALS['TYPO3_DB']->fullQuoteStr($uid, $this->table).
 							t3lib_BEfunc::deleteClause($this->table).
-							$this->clause,	// whereClauseMightContainGroupOrderBy
-						'',
-						$this->orderByFields
+							t3lib_BEfunc::versioningPlaceholderClause($this->table).
+							$this->clause	// whereClauseMightContainGroupOrderBy
 					);
 			$row = $GLOBALS['TYPO3_DB']->sql_fetch_row($res);
 			return $row[0];
@@ -869,7 +879,9 @@ class t3lib_treeView {
 		if (is_array($this->data)) {
 			return $this->dataLookup[$uid];
 		} else {
-			return t3lib_befunc::getRecord($this->table,$uid);
+			$row = t3lib_befunc::getRecordWSOL($this->table,$uid);
+
+			return $row;
 		}
 	}
 
@@ -879,10 +891,11 @@ class t3lib_treeView {
 	 * For arrays: This will return key to the ->dataLookup array
 	 *
 	 * @param	integer		parent item id
+	 * @param	string		Class for sub-elements. 
 	 * @return	mixed		data handle (Tables: An sql-resource, arrays: A parentId integer. -1 is returned if there were NO subLevel.)
 	 * @access private
 	 */
-	function getDataInit($parentId) {
+	function getDataInit($parentId,$subCSSclass='') {
 		if (is_array($this->data)) {
 			if (!is_array($this->dataLookup[$parentId][$this->subLevelID])) {
 				$parentId = -1;
@@ -894,8 +907,9 @@ class t3lib_treeView {
 			$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery(
 						implode(',',$this->fieldArray),
 						$this->table,
-						$this->parentField.'="'.$GLOBALS['TYPO3_DB']->quoteStr($parentId, $this->table).'"'.
+						$this->parentField.'='.$GLOBALS['TYPO3_DB']->fullQuoteStr($parentId, $this->table).
 							t3lib_BEfunc::deleteClause($this->table).
+							t3lib_BEfunc::versioningPlaceholderClause($this->table).
 							$this->clause,	// whereClauseMightContainGroupOrderBy
 						'',
 						$this->orderByFields
@@ -925,28 +939,40 @@ class t3lib_treeView {
 	 * Getting the tree data: next entry
 	 *
 	 * @param	mixed		data handle
+	 * @param	string		CSS class for sub elements (workspace related)
 	 * @return	array		item data array OR FALSE if end of elements.
 	 * @access private
 	 * @see getDataInit()
 	 */
-	function getDataNext(&$res){
+	function getDataNext(&$res,$subCSSclass=''){
 		if (is_array($this->data)) {
 			if ($res<0) {
 				$row=FALSE;
 			} else {
 				list(,$row) = each($this->dataLookup[$res][$this->subLevelID]);
 
-				/*
-				if (!is_array($row))	{
-					$row=FALSE;
-				} else {
-					unset($row['subLevel']);
+					// Passing on default <td> class for subelements:
+				if (is_array($row) && $subCSSclass!=='')	{
+					$row['_CSSCLASS'] = $row['_SUBCSSCLASS'] = $subCSSclass;
 				}
-				*/
 			}
 			return $row;
 		} else {
-			return @$GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
+			$row = @$GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
+			t3lib_BEfunc::workspaceOL($this->table, $row, $this->BE_USER->workspace);
+
+				// Passing on default <td> class for subelements:
+			if (is_array($row) && $subCSSclass!=='')	{
+
+				if ($this->table==='pages' && $this->highlightPagesWithVersions && !isset($row['_CSSCLASS']) && count(t3lib_BEfunc::countVersionsOfRecordsOnPage($this->BE_USER->workspace, $row['uid'], TRUE)))	{
+					$row['_CSSCLASS'] = 'ver-versions';
+				}
+
+				if (!isset($row['_CSSCLASS']))	$row['_CSSCLASS'] = $subCSSclass;
+				if (!isset($row['_SUBCSSCLASS']))	$row['_SUBCSSCLASS'] = $subCSSclass;
+			}
+
+			return $row;
 		}
 	}
 
