@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2006 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Class with template object that is responsible for generating the template
  *
- * $Id: class.t3lib_tstemplate.php,v 1.18 2004/09/13 22:57:19 typo3 Exp $
+ * $Id: class.t3lib_tstemplate.php 1495 2006-06-02 09:29:25Z masi $
  * Revised for TYPO3 3.6 July/2003 by Kasper Skaarhoj
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
@@ -37,43 +37,45 @@
  *
  *
  *
- *  107: class t3lib_TStemplate
- *  209:     function init()
- *  247:     function getCurrentPageData()
- *  264:     function matching($cc)
- *  288:     function start($theRootLine)
+ *  109: class t3lib_TStemplate
+ *  211:     function init()
+ *  249:     function getCurrentPageData()
+ *  266:     function matching($cc)
+ *  290:     function start($theRootLine)
  *
  *              SECTION: Fetching TypoScript code text for the Template Hierarchy
- *  403:     function runThroughTemplates($theRootLine,$start_template_uid=0)
- *  450:     function processTemplate($row, $idList,$pid,$templateID='',$templateParent='')
- *  565:     function includeStaticTypoScriptSources($idList,$templateID,$pid,$row)
- *  626:     function addExtensionStatics($idList,$templateID,$pid,$row)
- *  659:     function prependStaticExtra($subrow)
+ *  406:     function runThroughTemplates($theRootLine,$start_template_uid=0)
+ *  459:     function processTemplate($row, $idList,$pid,$templateID='',$templateParent='')
+ *  580:     function includeStaticTypoScriptSources($idList,$templateID,$pid,$row)
+ *  642:     function addExtensionStatics($idList,$templateID,$pid,$row)
+ *  675:     function prependStaticExtra($subrow)
+ *  688:     function versionOL(&$row)
  *
  *              SECTION: Parsing TypoScript code text from Template Records into PHP array
- *  695:     function generateConfig()
- *  859:     function procesIncludes()
- *  883:     function mergeConstantsFromPageTSconfig($constArray)
- *  912:     function flattenSetup($setupArray, $prefix, $resourceFlag)
- *  936:     function substituteConstants($all)
+ *  725:     function generateConfig()
+ *  891:     function procesIncludes()
+ *  915:     function mergeConstantsFromPageTSconfig($constArray)
+ *  944:     function flattenSetup($setupArray, $prefix, $resourceFlag)
+ *  968:     function substituteConstants($all)
+ *  991:     function substituteConstantsCallBack($matches)
  *
  *              SECTION: Various API functions, used from elsewhere in the frontend classes
- *  974:     function splitConfArray($conf,$splitCount)
- * 1051:     function getFileName($fileFromSetup)
- * 1108:     function extractFromResources($res,$file)
- * 1136:     function checkFile($name,$menuArr)
- * 1153:     function printTitle($title,$no_title=0,$titleFirst=0)
- * 1176:     function fileContent($fName)
- * 1196:     function wrap($content,$wrap)
- * 1210:     function removeQueryString($url)
- * 1227:     function sortedKeyList($setupArr, $acceptOnlyProperties=FALSE)
+ * 1022:     function splitConfArray($conf,$splitCount)
+ * 1099:     function getFileName($fileFromSetup)
+ * 1156:     function extractFromResources($res,$file)
+ * 1184:     function checkFile($name,$menuArr)
+ * 1201:     function printTitle($title,$no_title=0,$titleFirst=0)
+ * 1224:     function fileContent($fName)
+ * 1244:     function wrap($content,$wrap)
+ * 1258:     function removeQueryString($url)
+ * 1275:     function sortedKeyList($setupArr, $acceptOnlyProperties=FALSE)
  *
  *              SECTION: Functions for creating links
- * 1274:     function linkData($page,$oTarget,$no_cache,$script,$overrideArray='',$addParams='',$typeOverride='')
- * 1393:     function getFromMPmap($pageId=0)
- * 1429:     function initMPmap_create($id,$MP_array=array(),$level=0)
+ * 1322:     function linkData($page,$oTarget,$no_cache,$script,$overrideArray='',$addParams='',$typeOverride='')
+ * 1449:     function getFromMPmap($pageId=0)
+ * 1485:     function initMPmap_create($id,$MP_array=array(),$level=0)
  *
- * TOTAL FUNCTIONS: 26
+ * TOTAL FUNCTIONS: 28
  * (This index is automatically created/updated by the extension "extdeveval")
  *
  */
@@ -207,7 +209,7 @@ class t3lib_TStemplate	{
 	 * @see tslib_fe::initTemplate()
 	 */
 	function init()	{
-	 		// $this->whereClause is used only to select templates from sys_template.
+			// $this->whereClause is used only to select templates from sys_template.
 			// $GLOBALS['SIM_EXEC_TIME'] is used so that we're able to simulate a later time as a test...
 		$this->whereClause='AND deleted=0 ';
 		if (!$GLOBALS['TSFE']->showHiddenRecords)	{
@@ -346,7 +348,7 @@ class t3lib_TStemplate	{
 				if ($this->tt_track)	$GLOBALS['TT']->setTSlogMessage('TS template size, serialized: '.strlen(serialize($this->setup)).' bytes');
 
 				$rowSumHash = md5('ROWSUM:'.serialize($this->rowSum));
-				t3lib_pageSelect::storeHash($rowSumHash, serialize($cc['all']), 'TMPL CONDITIONS - ALL');
+				t3lib_pageSelect::storeHash($rowSumHash, serialize($cc['all']), 'TMPL CONDITIONS - AL');
 			}
 				// Add rootLine
 			$cc['rootLine'] = $this->rootLine;
@@ -361,6 +363,7 @@ class t3lib_TStemplate	{
 					'tstamp' => $GLOBALS['EXEC_TIME']
 				);
 				$GLOBALS['TYPO3_DB']->exec_DELETEquery('cache_pagesection', 'page_id='.intval($GLOBALS['TSFE']->id).' AND mpvar_hash='.t3lib_div::md5int($GLOBALS['TSFE']->MP));
+
 				$GLOBALS['TYPO3_DB']->exec_INSERTquery('cache_pagesection', $insertFields);
 			}
 				// If everything OK.
@@ -415,8 +418,11 @@ class t3lib_TStemplate	{
 				$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'sys_template', 'uid='.intval($this->nextLevel).' '.$this->whereClause);
 				$this->nextLevel = 0;
 				if ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res))	{
-					$this->processTemplate($row,'sys_'.$row['uid'],$this->absoluteRootLine[$a]['uid'],'sys_'.$row['uid']);
-					$this->outermostRootlineIndexWithTemplate=$a;
+					$this->versionOL($row);
+					if (is_array($row))	{
+						$this->processTemplate($row,'sys_'.$row['uid'],$this->absoluteRootLine[$a]['uid'],'sys_'.$row['uid']);
+						$this->outermostRootlineIndexWithTemplate=$a;
+					}
 				}
 				$GLOBALS['TYPO3_DB']->sql_free_result($res);
 			}
@@ -427,8 +433,11 @@ class t3lib_TStemplate	{
 
 			$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'sys_template', 'pid='.intval($this->absoluteRootLine[$a]['uid']).$addC.' '.$this->whereClause,'','sorting',1);
 			if ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res))	{
-				$this->processTemplate($row,'sys_'.$row['uid'],$this->absoluteRootLine[$a]['uid'],'sys_'.$row['uid']);
-				$this->outermostRootlineIndexWithTemplate=$a;
+				$this->versionOL($row);
+				if (is_array($row))	{
+					$this->processTemplate($row,'sys_'.$row['uid'],$this->absoluteRootLine[$a]['uid'],'sys_'.$row['uid']);
+					$this->outermostRootlineIndexWithTemplate=$a;
+				}
 			}
 			$GLOBALS['TYPO3_DB']->sql_free_result($res);
 			$this->rootLine[] = $this->absoluteRootLine[$a];
@@ -485,7 +494,10 @@ class t3lib_TStemplate	{
 				if ($id && !t3lib_div::inList($idList,'sys_'.$id))	{	// if $id is not allready included ...
 					$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'sys_template', 'uid='.$id.' '.$this->whereClause);
 					if ($subrow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res))	{	// there was a template, then we fetch that
-						$this->processTemplate($subrow,$idList.',sys_'.$id,$pid, 'sys_'.$id,$templateID);
+						$this->versionOL($subrow);
+						if (is_array($subrow))	{
+							$this->processTemplate($subrow,$idList.',sys_'.$id,$pid, 'sys_'.$id,$templateID);
+						}
 					}
 					$GLOBALS['TYPO3_DB']->sql_free_result($res);
 				}
@@ -495,7 +507,10 @@ class t3lib_TStemplate	{
 					if (!t3lib_div::inList($idList,'sys_'.$id))	{	// if $id is not allready included ...
 						$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'sys_template', 'uid='.intval($id).' '.$this->whereClause);
 						if ($subrow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res))	{	// there was a template, then we fetch that
-							$this->processTemplate($subrow,$idList.',sys_'.$id,$pid, 'sys_'.$id,$templateID);
+							$this->versionOL($subrow);
+							if (is_array($subrow))	{
+								$this->processTemplate($subrow,$idList.',sys_'.$id,$pid, 'sys_'.$id,$templateID);
+							}
 						}
 						$GLOBALS['TYPO3_DB']->sql_free_result($res);
 					}
@@ -597,6 +612,7 @@ class t3lib_TStemplate	{
 								'config'=>		@is_file($ISF_filePath.'setup.txt')		?t3lib_div::getUrl($ISF_filePath.'setup.txt'):'',
 								'editorcfg'=>	@is_file($ISF_filePath.'editorcfg.txt')	?t3lib_div::getUrl($ISF_filePath.'editorcfg.txt'):'',
 								'include_static'=>	@is_file($ISF_filePath.'include_static.txt')?implode(',',array_unique(t3lib_div::intExplode(',',t3lib_div::getUrl($ISF_filePath.'include_static.txt')))):'',
+								'include_static_file'=>	@is_file($ISF_filePath.'include_static_file.txt')?implode(',',array_unique(explode(',',t3lib_div::getUrl($ISF_filePath.'include_static_file.txt')))):'',
 								'title' => 		$ISF_file,
 								'uid' => 		$mExtKey
 							);
@@ -663,6 +679,20 @@ class t3lib_TStemplate	{
 		return $subrow;
 	}
 
+	/**
+	 * Creating versioning overlay of a sys_template record. This will use either frontend or backend overlay functionality depending on environment.
+	 *
+	 * @param	array		Row to overlay.
+	 * @return	void		Row is passed by reference.
+	 */
+	function versionOL(&$row)	{
+		if (is_object($GLOBALS['TSFE']))	{	// Frontend:
+			$GLOBALS['TSFE']->sys_page->versionOL('sys_template',$row);
+		} else {	// Backend:
+			t3lib_BEfunc::workspaceOL('sys_template',$row);
+		}
+	}
+
 
 
 
@@ -836,6 +866,8 @@ class t3lib_TStemplate	{
 					$tN = $this->setup[$theKey.'.']['typeNum'];
 					if (isset($tN))	{
 						$this->setup['types.'][$tN] = $theKey;
+					} elseif(!$this->setup['types.'][0])	{	// If there is no type 0 yet and typeNum was not set, we use the current object as the default
+						$this->setup['types.'][0] = $theKey;
 					}
 				}
 			}
@@ -935,15 +967,31 @@ class t3lib_TStemplate	{
 	 */
 	function substituteConstants($all)	{
 		if ($this->tt_track)	$GLOBALS['TT']->setTSlogMessage('Constants to substitute: '.count($this->flatSetup));
-		reset($this->flatSetup);
-		while (list($const,$val)=each($this->flatSetup))	{
-			if (!is_array($val))	{
-				$all = str_replace('{$'.$const.'}',$val,$all);
+
+		$noChange = false;
+		// recursive substitution of constants (up to 10 nested levels)
+		for ($i = 0; $i < 10 && !$noChange; $i++) {
+			$old_all = $all;
+			$all = preg_replace_callback('/\{\$(.[^}]*)\}/', array($this, 'substituteConstantsCallBack'), $all);
+			if ($old_all == $all) {
+				$noChange = true;
 			}
 		}
+
 		return $all;
 	}
 
+	/**
+	 * Call back method for preg_replace_callback in substituteConstants
+	 *
+	 * @param	array		Regular expression matches
+	 * @return	string		Replacement
+	 * @see substituteConstants()
+	 */
+	function substituteConstantsCallBack($matches) {
+		// replace {$CONST} if found in $this->flatSetup, else leave unchanged
+		return isset($this->flatSetup[$matches[1]]) && !is_array($this->flatSetup[$matches[1]]) ? $this->flatSetup[$matches[1]] : $matches[0];
+	}
 
 
 
@@ -1050,7 +1098,12 @@ class t3lib_TStemplate	{
 	 */
 	function getFileName($fileFromSetup)	{
 		$file = trim($fileFromSetup);
-		if (!$file)	return;
+		if (!$file)	{
+			return;
+		} elseif (strstr($file,'../'))	{
+			if ($this->tt_track)	$GLOBALS['TT']->setTSlogMessage('File path "'.$file.'" contained illegal string "../"!',3);
+			return;
+		}
 			// cache
 		$hash = md5($file);
 		if (isset($this->fileCache[$hash]))	{
@@ -1072,6 +1125,7 @@ class t3lib_TStemplate	{
 
 			// find
 		if (strstr($file,'/')) {	// here it is manual media
+			if(!strcmp(substr($file,0,6),'media/')) $file = 'typo3/sysext/cms/tslib/'.$file;
 			if (@is_file($this->getFileName_backPath.$file))	{
 				$outFile = $file;
 				$fileInfo = t3lib_div::split_fileref($outFile);
@@ -1130,7 +1184,7 @@ class t3lib_TStemplate	{
 	 * @param	string		Property name in the menu array
 	 * @param	array		Menu array to traverse
 	 * @return	array		Modified menu array
-	 * @depreciated
+	 * @deprecated
 	 * @internal
 	 */
 	function checkFile($name,$menuArr)	{
@@ -1323,7 +1377,15 @@ class t3lib_TStemplate	{
 		$LD['no_cache'] = (trim($page['no_cache']) || $no_cache) ? '&no_cache=1' : '';
 
 			// linkVars
-		$LD['linkVars'] = $GLOBALS['TSFE']->linkVars.$addParams;
+		if ($GLOBALS['TSFE']->config['config']['uniqueLinkVars']) {
+			if ($addParams) {
+				$LD['linkVars'] = t3lib_div::implodeArrayForUrl('',t3lib_div::explodeUrl2Array($GLOBALS['TSFE']->linkVars.$addParams));
+			} else {
+				$LD['linkVars'] = $GLOBALS['TSFE']->linkVars;
+			}
+		} else {
+			$LD['linkVars'] = $GLOBALS['TSFE']->linkVars.$addParams;
+		}
 
 			// If simulateStaticDocuments is enabled:
 		if ($GLOBALS['TSFE']->config['config']['simulateStaticDocuments'])	{
@@ -1360,7 +1422,7 @@ class t3lib_TStemplate	{
 		$LD['url'] = $GLOBALS['TSFE']->absRefPrefix.$LD['url'];
 
 			// If the special key 'sectionIndex_uid' (added 'manually' in tslib/menu.php to the page-record) is set, then the link jumps directly to a section on the page.
-		$LD['sectionIndex'] = $page['sectionIndex_uid'] ? '#'.$page['sectionIndex_uid'] : '';
+		$LD['sectionIndex'] = $page['sectionIndex_uid'] ? '#c'.$page['sectionIndex_uid'] : '';
 
 			// Compile the normal total url
 		$LD['totalURL']= $this->removeQueryString($LD['url'].$LD['type'].$LD['no_cache'].$LD['linkVars'].$GLOBALS['TSFE']->getMethodUrlIdToken).$LD['sectionIndex'];
