@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -30,7 +30,7 @@
  * Provides links to registered shortcuts
  * If the 'cms' extension is loaded you will also have a field for entering page id/alias which will be found/edited
  *
- * $Id: alt_shortcut.php,v 1.17 2004/09/13 22:57:21 typo3 Exp $
+ * $Id: alt_shortcut.php 1421 2006-04-10 09:27:15Z mundaun $
  * Revised for TYPO3 3.6 2/2003 by Kasper Skaarhoj
  * XHTML compliant output
  *
@@ -41,21 +41,25 @@
  *
  *
  *
- *   82: class SC_alt_shortcut
- *  118:     function preinit()
- *  143:     function preprocess()
- *  200:     function init()
- *  234:     function main()
- *  335:     function editLoadedFunc()
- *  396:     function editPageIdFunc()
- *  438:     function printContent()
+ *   86: class SC_alt_shortcut
+ *  125:     function preinit()
+ *  152:     function preprocess()
+ *  234:     function init()
+ *  275:     function main()
+ *  452:     function editLoadedFunc()
+ *  532:     function editPageIdFunc()
+ *  586:     function printContent()
+ *
+ *              SECTION: WORKSPACE FUNCTIONS:
+ *  611:     function workspaceSelector()
  *
  *              SECTION: OTHER FUNCTIONS:
- *  466:     function mIconFilename($Ifilename,$backPath)
- *  479:     function getIcon($modName)
- *  503:     function itemLabel($inlabel,$modName,$M_modName='')
+ *  686:     function mIconFilename($Ifilename,$backPath)
+ *  702:     function getIcon($modName)
+ *  726:     function itemLabel($inlabel,$modName,$M_modName='')
+ *  748:     function getLinkedPageId($url)
  *
- * TOTAL FUNCTIONS: 10
+ * TOTAL FUNCTIONS: 12
  * (This index is automatically created/updated by the extension "extdeveval")
  *
  */
@@ -93,13 +97,11 @@ class SC_alt_shortcut {
 
 		// Internal, static:
 	var $loadModules;		// Modules object
-	var $doc;				// Document template object
-	var $nGroups;			// Number of groups
-	var $nGlobals;			// Number of globals
+	var $doc;			// Document template object
 
 		// Internal, dynamic:
 	var $content;			// Accumulation of output HTML (string)
-	var $lines;				// Accumulation of table cells (array)
+	var $lines;			// Accumulation of table cells (array)
 
 	var $editLoaded;		// Flag for defining whether we are editing
 	var $editError;			// Can contain edit error message
@@ -108,6 +110,11 @@ class SC_alt_shortcut {
 	var $theEditRec;		// Page record to be edited
 	var $editPage;			// Page alias or id to be edited
 	var $selOpt;			// Select options.
+	var $searchFor;			// Text to search for...
+	var $groupLabels=array();	// Labels of all groups. If value is 1, the system will try to find a label in the locallang array.
+
+	var $alternativeTableUid = array();	// Array with key 0/1 being table/uid of record to edit. Internally set.
+
 
 
 	/**
@@ -126,6 +133,8 @@ class SC_alt_shortcut {
 
 		$this->deleteCategory = t3lib_div::_GP('deleteCategory');
 		$this->editPage = t3lib_div::_GP('editPage');
+		$this->changeWorkspace = t3lib_div::_GP('changeWorkspace');
+		$this->changeWorkspacePreview = t3lib_div::_GP('changeWorkspacePreview');
 		$this->editName = t3lib_div::_GP('editName');
 		$this->editGroup = t3lib_div::_GP('editGroup');
 		$this->whichItem = t3lib_div::_GP('whichItem');
@@ -142,6 +151,30 @@ class SC_alt_shortcut {
 	 */
 	function preprocess()	{
 		global $BE_USER;
+		$description = '';	// Default description
+		$url = urldecode($this->URL);
+
+			// Lookup the title of this page and use it as default description
+		$page_id = $this->getLinkedPageId($url);
+		if (t3lib_div::testInt($page_id))	{
+			if (preg_match('/\&edit\[(.*)\]\[(.*)\]=edit/',$url,$matches))	{
+					// Edit record
+				$description = '';	// TODO: Set something useful
+			} else {
+					// Page listing
+				$pageRow = t3lib_BEfunc::getRecord('pages',$page_id);
+				if (count($pageRow))	{
+						// If $page_id is an integer, set the description to the title of that page
+					$description = $pageRow['title'];
+				}
+			}
+		} else {
+			if (preg_match('/\/$/', $page_id))	{
+					// If $page_id is a string and ends with a slash, assume it is a fileadmin reference and set the description to the basename of that path
+				$description = basename($page_id);
+			}
+		}
+
 
 			// Adding a shortcut being set from another frame
 		if ($this->modName && $this->URL)	{
@@ -149,7 +182,8 @@ class SC_alt_shortcut {
 				'userid' => $BE_USER->user['uid'],
 				'module_name' => $this->modName.'|'.$this->M_modName,
 				'url' => $this->URL,
-				'sorting' => time()
+				'description' => $description,
+				'sorting' => time(),
 			);
 			$GLOBALS['TYPO3_DB']->exec_INSERTquery('sys_be_shortcuts', $fields_values);
 		}
@@ -217,11 +251,18 @@ class SC_alt_shortcut {
 				top.goToModule(modName);
 			}
 			function editSh(uid)	{	//
-				document.location="alt_shortcut.php?editShortcut="+uid;
+				window.location.href="alt_shortcut.php?editShortcut="+uid;
 			}
 			function submitEditPage(id)	{	//
-				document.location="alt_shortcut.php?editPage="+top.rawurlencode(id);
+				window.location.href="alt_shortcut.php?editPage="+top.rawurlencode(id);
 			}
+			function changeWorkspace(workspaceId)	{	//
+				window.location.href="alt_shortcut.php?changeWorkspace="+top.rawurlencode(workspaceId);
+			}
+			function changeWorkspacePreview(newstate)	{	//
+				window.location.href="alt_shortcut.php?changeWorkspacePreview="+newstate;
+			}
+
 			');
 		$this->content.=$this->doc->startPage('Shortcut frame');
 	}
@@ -232,59 +273,112 @@ class SC_alt_shortcut {
 	 * @return	void
 	 */
 	function main()	{
-		global $BE_USER,$LANG;
+		global $BE_USER,$LANG,$TCA;
 
-			// Setting groups and globals
-		$this->nGroups=4;
-		$this->nGlobals=5;
+			// By default, 5 groups are set
+		$this->groupLabels=array(
+			1 => 1,
+			2 => 1,
+			3 => 1,
+			4 => 1,
+			5 => 1,
+		);
 
-		$globalGroups=array(-100);
 		$shortCutGroups = $BE_USER->getTSConfig('options.shortcutGroups');
-		for($a=1;$a<=$this->nGlobals;$a++)	{
-			if ($BE_USER->isAdmin() || strcmp($shortCutGroups['properties'][$a],''))	{
-				$globalGroups[]=-$a;
+		if (is_array($shortCutGroups['properties']) && count($shortCutGroups['properties']))	{
+			foreach ($shortCutGroups['properties'] as $k=>$v)	{
+				if (strcmp('',$v) && strcmp('0',$v))	{
+					$this->groupLabels[$k] = (string)$v;
+				} elseif ($BE_USER->isAdmin())	{
+					unset($this->groupLabels[$k]);
+				}
 			}
 		}
 
+			// List of global groups that will be loaded. All global groups have negative IDs.
+		$globalGroups = -100;	// Group -100 is kind of superglobal and can't be changed.
+		if (count($this->groupLabels))	{
+			$globalGroups .= ','.implode(',',array_keys($this->groupLabels));
+			$globalGroups = str_replace(',',',-',$globalGroups);	// Ugly hack to make the UIDs negative - is there any better solution?
+		}
+
 			// Fetching shortcuts to display for this user:
-		$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'sys_be_shortcuts', '((userid='.$BE_USER->user['uid'].' AND sc_group>=0) OR sc_group IN ('.implode(',',$globalGroups).'))', '', 'sc_group,sorting');
+		$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'sys_be_shortcuts', '((userid='.$BE_USER->user['uid'].' AND sc_group>=0) OR sc_group IN ('.$globalGroups.'))', '', 'sc_group,sorting');
 
 			// Init vars:
 		$this->lines=array();
+		$this->linesPre=array();
 		$this->editSC_rec='';
 		$this->selOpt=array();
 		$formerGr='';
 
 			// Traverse shortcuts
 		while($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res))	{
-			if ($this->editSC && $row['uid']==$this->editSC)	{
-				$this->editSC_rec=$row;
-			}
-
-			if (strcmp($formerGr,$row['sc_group']))	{
-				if ($row['sc_group']!=-100)	{
-					if ($row['sc_group']>=0)	{
-						$onC = 'if (confirm('.$GLOBALS['LANG']->JScharCode($LANG->getLL('shortcut_delAllInCat')).')){document.location=\'alt_shortcut.php?deleteCategory='.$row['sc_group'].'\';}return false;';
-						$this->lines[]='<td>&nbsp;</td><td class="bgColor5"><a href="#" onclick="'.htmlspecialchars($onC).'" title="'.$LANG->getLL('shortcut_delAllInCat',1).'">'.abs($row['sc_group']).'</a></td>';
-					} else {
-						$this->lines[]='<td>&nbsp;</td><td class="bgColor5">'.abs($row['sc_group']).'</td>';
-					}
-				}
-			}
-
 			$mParts = explode('|',$row['module_name']);
 			$row['module_name']=$mParts[0];
 			$row['M_module_name']=$mParts[1];
 			$mParts = explode('_',$row['M_module_name']?$row['M_module_name']:$row['module_name']);
 			$qParts = parse_url($row['url']);
 
+			if (!$BE_USER->isAdmin())	{
+					// Check for module access
+				if (!isset($LANG->moduleLabels['tabs_images'][implode('_',$mParts).'_tab']))	{	// Nice hack to check if the user has access to this module - otherwise the translation label would not have been loaded :-)
+					continue;
+				}
+
+				$page_id = $this->getLinkedPageId($row['url']);
+				if (t3lib_div::testInt($page_id))	{
+						// Check for webmount access
+					if (!$GLOBALS['BE_USER']->isInWebMount($page_id)) continue;
+
+						// Check for record access
+					$pageRow = t3lib_BEfunc::getRecord('pages',$page_id);
+					if (!$GLOBALS['BE_USER']->doesUserHaveAccess($pageRow,$perms=1)) continue;
+				}
+			}
+
+			if ($this->editSC && $row['uid']==$this->editSC)	{
+				$this->editSC_rec=$row;
+			}
+
+			$sc_group = $row['sc_group'];
+			if ($sc_group && strcmp($formerGr,$sc_group))	{
+				if ($sc_group!=-100)	{
+					if ($this->groupLabels[abs($sc_group)] && strcmp('1',$this->groupLabels[abs($sc_group)]))	{
+						$label = $this->groupLabels[abs($sc_group)];
+					} else {
+						$label = $LANG->getLL('shortcut_group_'.abs($sc_group),1);
+						if (!$label)	$label = $LANG->getLL('shortcut_group',1).' '.abs($sc_group);	// Fallback label
+					}
+
+					if ($sc_group>=0)	{
+						$onC = 'if (confirm('.$GLOBALS['LANG']->JScharCode($LANG->getLL('shortcut_delAllInCat')).')){window.location.href=\'alt_shortcut.php?deleteCategory='.$sc_group.'\';}return false;';
+						$this->linesPre[]='<td>&nbsp;</td><td class="bgColor5"><a href="#" onclick="'.htmlspecialchars($onC).'" title="'.$LANG->getLL('shortcut_delAllInCat',1).'">'.$label.'</a></td>';
+					} else {
+						$label = $LANG->getLL('shortcut_global',1).': '.($label ? $label : abs($sc_group));	// Fallback label
+						$this->lines[]='<td>&nbsp;</td><td class="bgColor5">'.$label.'</td>';
+					}
+					unset($label);
+				}
+			}
+
 			$bgColorClass = $row['uid']==$this->editSC ? 'bgColor5' : ($row['sc_group']<0 ? 'bgColor6' : 'bgColor4');
-			$titleA = $this->itemLabel($row['description']&&($row['uid']!=$this->editSC) ? $row['description'] : t3lib_div::fixed_lgd(rawurldecode($qParts['query']),150),$row['module_name'],$row['M_module_name']);
+
+			if ($row['description']&&($row['uid']!=$this->editSC))	{
+				$label = $row['description'];
+			} else {
+				$label = t3lib_div::fixed_lgd(rawurldecode($qParts['query']),150);
+			}
+			$titleA = $this->itemLabel($label,$row['module_name'],$row['M_module_name']);
 
 			$editSH = ($row['sc_group']>=0 || $BE_USER->isAdmin()) ? 'editSh('.intval($row['uid']).');' : "alert('".$LANG->getLL('shortcut_onlyAdmin')."')";
 			$jumpSC = 'jump(unescape(\''.rawurlencode($row['url']).'\'),\''.implode('_',$mParts).'\',\''.$mParts[0].'\');';
 			$onC = 'if (document.shForm.editShortcut_check && document.shForm.editShortcut_check.checked){'.$editSH.'}else{'.$jumpSC.'}return false;';
-			$this->lines[]='<td class="'.$bgColorClass.'"><a href="#" onclick="'.htmlspecialchars($onC).'"><img src="'.$this->getIcon($row['module_name']).'" title="'.htmlspecialchars($titleA).'" alt="" /></a></td>';
+			if ($sc_group>=0)	{	// user defined groups show up first
+				$this->linesPre[]='<td class="'.$bgColorClass.'"><a href="#" onclick="'.htmlspecialchars($onC).'"><img src="'.$this->getIcon($row['module_name']).'" title="'.htmlspecialchars($titleA).'" alt="" /></a></td>';
+			} else {
+				$this->lines[]='<td class="'.$bgColorClass.'"><a href="#" onclick="'.htmlspecialchars($onC).'"><img src="'.$this->getIcon($row['module_name']).'" title="'.htmlspecialchars($titleA).'" alt="" /></a></td>';
+			}
 			if (trim($row['description']))	{
 				$kkey = strtolower(substr($row['description'],0,20)).'_'.$row['uid'];
 				$this->selOpt[$kkey]='<option value="'.htmlspecialchars($jumpSC).'">'.htmlspecialchars(t3lib_div::fixed_lgd_cs($row['description'],50)).'</option>';
@@ -298,33 +392,56 @@ class SC_alt_shortcut {
 		$this->editPageIdFunc();
 
 		if (!$this->editLoaded && t3lib_extMgm::isLoaded('cms'))	{
-				$editIdCode = '<td nowrap="nowrap">'.$LANG->getLL('shortcut_editID',1).': <input type="text" value="'.($this->editError?htmlspecialchars($this->editPage):'').'" name="editPage"'.$this->doc->formWidth(5).' onchange="submitEditPage(this.value);" />'.
+				$editIdCode = '<td nowrap="nowrap">'.$LANG->getLL('shortcut_editID',1).': <input type="text" value="'.($this->editError?htmlspecialchars($this->editPage):'').'" name="editPage"'.$this->doc->formWidth(15).' onchange="submitEditPage(this.value);" />'.
 					($this->editError?'&nbsp;<strong><span class="typo3-red">'.htmlspecialchars($this->editError).'</span></strong>':'').
 					(is_array($this->theEditRec)?'&nbsp;<strong>'.$LANG->getLL('shortcut_loadEdit',1).' \''.t3lib_BEfunc::getRecordTitle('pages',$this->theEditRec,1).'\'</strong> ('.htmlspecialchars($this->editPath).')':'').
+					($this->searchFor?'&nbsp;'.$LANG->getLL('shortcut_searchFor',1).' <strong>\''.htmlspecialchars($this->searchFor).'\'</strong>':'').
 					'</td>';
 		} else $editIdCode = '';
 
 			// Adding CSH:
 		$editIdCode.= '<td>&nbsp;'.t3lib_BEfunc::cshItem('xMOD_csh_corebe', 'shortcuts', $GLOBALS['BACK_PATH'],'',TRUE).'</td>';
 
+			// Compile it all:
 		$this->content.='
 
-
-			<!--
-				Shortcut Display Table:
-			-->
-			<table border="0" cellpadding="0" cellspacing="2" id="typo3-shortcuts">
+			<table border="0" cellpadding="0" cellspacing="0" width="99%">
 				<tr>
-				'.implode('
-				',$this->lines).$editIdCode.'
+					<td>
+						<!--
+							Shortcut Display Table:
+						-->
+						<table border="0" cellpadding="0" cellspacing="2" id="typo3-shortcuts">
+							<tr>
+							'.implode('
+							',$this->lines).$editIdCode.'
+							</tr>
+						</table>
+					</td>
+					<td align="right">
+						'.$this->workspaceSelector().
+							t3lib_BEfunc::cshItem('xMOD_csh_corebe', 'workspaceSelector', $GLOBALS['BACK_PATH'],'',TRUE).'
+					</td>
 				</tr>
 			</table>
-
 			';
 
+			// Launch Edit page:
 		if ($this->theEditRec['uid'])	{
 			$this->content.=$this->doc->wrapScriptTags('top.loadEditId('.$this->theEditRec['uid'].');');
 		}
+
+			// Load alternative table/uid into editing form.
+		if (count($this->alternativeTableUid)==2 && isset($TCA[$this->alternativeTableUid[0]]) && t3lib_div::testInt($this->alternativeTableUid[1]))	{
+			$JSaction = t3lib_BEfunc::editOnClick('&edit['.$this->alternativeTableUid[0].']['.$this->alternativeTableUid[1].']=edit','','dummy.php');
+			$this->content.=$this->doc->wrapScriptTags('function editArbitraryElement() { top.content.'.$JSaction.'; } editArbitraryElement();');
+		}
+
+			// Load search for something.
+		if ($this->searchFor)	{
+			$firstMP = intval($GLOBALS['WEBMOUNTS'][0]);
+			$this->content.= $this->doc->wrapScriptTags('jump(unescape("'.rawurlencode('db_list.php?id='.$firstMP.'&search_field='.rawurlencode($this->searchFor).'&search_levels=4').'"),"web_list","web");');
+		}
 	}
 
 	/**
@@ -341,20 +458,35 @@ class SC_alt_shortcut {
 
 			$opt=array();
 			$opt[]='<option value="0"></option>';
-			for($a=1;$a<=$this->nGroups;$a++)	{
-				$opt[]='<option value="'.$a.'"'.(!strcmp($this->editSC_rec['sc_group'],$a)?' selected="selected"':'').'>'.$LANG->getLL('shortcut_group',1).' '.$a.'</option>';
-			}
-			if ($BE_USER->isAdmin())	{
-				for($a=1;$a<=$this->nGlobals;$a++)	{
-					$opt[]='<option value="-'.$a.'"'.(!strcmp($this->editSC_rec['sc_group'],'-'.$a)?' selected="selected"':'').'>'.$LANG->getLL('shortcut_GLOBAL',1).': '.$a.'</option>';
+
+			foreach($this->groupLabels as $k=>$v)	{
+				if ($v && strcmp('1',$v))	{
+					$label = $v;
+				} else {
+					$label = $LANG->getLL('shortcut_group_'.$k,1);
+					if (!$label)	$label = $LANG->getLL('shortcut_group',1).' '.$k;	// Fallback label
 				}
-				$opt[]='<option value="-100"'.(!strcmp($this->editSC_rec['sc_group'],'-100')?' selected="selected"':'').'>'.$LANG->getLL('shortcut_GLOBAL',1).': '.$LANG->getLL('shortcut_ALL',1).'</option>';
+				$opt[]='<option value="'.$k.'"'.(!strcmp($this->editSC_rec['sc_group'],$k)?' selected="selected"':'').'>'.$label.'</option>';
+			}
+
+			if ($BE_USER->isAdmin())	{
+				foreach($this->groupLabels as $k=>$v)	{
+					if ($v && strcmp('1',$v))	{
+						$label = $v;
+					} else {
+						$label = $LANG->getLL('shortcut_group_'.$k,1);
+						if (!$label)	$label = $LANG->getLL('shortcut_group',1).' '.$k;	// Fallback label
+					}
+					$label = $LANG->getLL('shortcut_global',1).': '.$label;	// Add a prefix for global groups
+
+					$opt[]='<option value="-'.$k.'"'.(!strcmp($this->editSC_rec['sc_group'],'-'.$k)?' selected="selected"':'').'>'.$label.'</option>';
+				}
+				$opt[]='<option value="-100"'.(!strcmp($this->editSC_rec['sc_group'],'-100')?' selected="selected"':'').'>'.$LANG->getLL('shortcut_global',1).': '.$LANG->getLL('shortcut_all',1).'</option>';
 			}
 
 				// border="0" hspace="2" width="21" height="16" - not XHTML compliant in <input type="image" ...>
 			$manageForm='
 
-
 				<!--
 					Shortcut Editing Form:
 				-->
@@ -373,14 +505,18 @@ class SC_alt_shortcut {
 
 				';
 		} else $manageForm='';
-//debug(count($opt));
+
 		if (!$this->editLoaded && count($this->selOpt)>1)	{
 			$this->lines[]='<td>&nbsp;</td>';
 			$this->lines[]='<td><select name="_selSC" onchange="eval(this.options[this.selectedIndex].value);this.selectedIndex=0;">'.implode('',$this->selOpt).'</select></td>';
 		}
+
+			// $this->linesPre contains elements with sc_group>=0
+		$this->lines = array_merge($this->linesPre,$this->lines);
+
 		if (count($this->lines))	{
 			if (!$BE_USER->getTSConfigVal('options.mayNotCreateEditShortcuts'))	{
-				$this->lines=array_merge(array('<td><input type="checkbox" name="editShortcut_check" value="1"'.($this->editSC?' checked="checked"':'').' />'.$LANG->getLL('shortcut_edit',1).'&nbsp;</td>'),$this->lines);
+				$this->lines=array_merge(array('<td><input type="checkbox" id="editShortcut_check" name="editShortcut_check" value="1"'.($this->editSC?' checked="checked"':'').' /><label for="editShortcut_check">'.$LANG->getLL('shortcut_edit',1).'</label>&nbsp;</td>'),$this->lines);
 				$this->lines[]='<td>'.$manageForm.'</td>';
 			}
 			$this->lines[]='<td><img src="clear.gif" width="10" height="1" alt="" /></td>';
@@ -402,29 +538,41 @@ class SC_alt_shortcut {
 		$this->editPage = trim(strtolower($this->editPage));
 		$this->editError = '';
 		$this->theEditRec = '';
+		$this->searchFor = '';
 		if ($this->editPage)	{
-			$where = ' AND ('.$BE_USER->getPagePermsClause(2).' OR '.$BE_USER->getPagePermsClause(16).')';
-			if (t3lib_div::testInt($this->editPage))	{
-				$this->theEditRec = t3lib_BEfunc::getRecord ('pages',$this->editPage,'*',$where);
-			} else {
-				$records = t3lib_BEfunc::getRecordsByField('pages','alias',$this->editPage,$where);
-				if (is_array($records))	{
-					reset($records);
-					$this->theEditRec = current($records);
+
+				// First, test alternative value consisting of [table]:[uid] and if not found, proceed with traditional page ID resolve:
+			$this->alternativeTableUid = explode(':',$this->editPage);
+			if (!(count($this->alternativeTableUid)==2 && $BE_USER->isAdmin()))	{	// We restrict it to admins only just because I'm not really sure if alt_doc.php properly checks permissions of passed records for editing. If alt_doc.php does that, then we can remove this.
+
+				$where = ' AND ('.$BE_USER->getPagePermsClause(2).' OR '.$BE_USER->getPagePermsClause(16).')';
+				if (t3lib_div::testInt($this->editPage))	{
+					$this->theEditRec = t3lib_BEfunc::getRecordWSOL('pages',$this->editPage,'*',$where);
+				} else {
+					$records = t3lib_BEfunc::getRecordsByField('pages','alias',$this->editPage,$where);
+					if (is_array($records))	{
+						reset($records);
+						$this->theEditRec = current($records);
+						t3lib_BEfunc::workspaceOL('pages',$this->theEditRec);
+					}
 				}
-			}
-			if (!is_array($this->theEditRec) || !$BE_USER->isInWebMount($this->theEditRec['uid']))	{
-				unset($this->theEditRec);
-				$this->editError=$LANG->getLL('shortcut_notEditable');
-			} else {
-					// Visual path set:
-				$perms_clause = $BE_USER->getPagePermsClause(1);
-				$this->editPath = t3lib_BEfunc::getRecordPath($this->theEditRec['pid'], $perms_clause, 30);
+				if (!is_array($this->theEditRec))	{
+					unset($this->theEditRec);
+					$this->searchFor = $this->editPage;
+				} elseif (!$BE_USER->isInWebMount($this->theEditRec['uid'])) {
+					unset($this->theEditRec);
+					$this->editError=$LANG->getLL('shortcut_notEditable');
+				} else {
 
-				if(!$BE_USER->getTSConfigVal('options.shortcut_onEditId_dontSetPageTree')) {
+						// Visual path set:
+					$perms_clause = $BE_USER->getPagePermsClause(1);
+					$this->editPath = t3lib_BEfunc::getRecordPath($this->theEditRec['pid'], $perms_clause, 30);
 
-						// Expanding page tree:
-					t3lib_BEfunc::openPageTree($this->theEditRec['pid'],!$BE_USER->getTSConfigVal('options.shortcut_onEditId_keepExistingExpanded'));
+					if(!$BE_USER->getTSConfigVal('options.shortcut_onEditId_dontSetPageTree')) {
+
+							// Expanding page tree:
+						t3lib_BEfunc::openPageTree($this->theEditRec['pid'],!$BE_USER->getTSConfigVal('options.shortcut_onEditId_keepExistingExpanded'));
+					}
 				}
 			}
 		}
@@ -437,6 +585,7 @@ class SC_alt_shortcut {
 	 */
 	function printContent()	{
 		$this->content.= $this->doc->endPage();
+		$this->content = $this->doc->insertStylesAndJS($this->content);
 		echo $this->content;
 	}
 
@@ -448,6 +597,77 @@ class SC_alt_shortcut {
 
 
 
+	/***************************
+	 *
+	 * WORKSPACE FUNCTIONS:
+	 *
+	 ***************************/
+
+	/**
+	 * Create selector for workspaces and change workspace if command is given to do that.
+	 *
+	 * @return	string		HTML
+	 */
+	function workspaceSelector()	{
+		global $TYPO3_DB,$BE_USER,$LANG;
+
+			// Changing workspace and if so, reloading entire backend:
+		if (strlen($this->changeWorkspace))	{
+			$BE_USER->setWorkspace($this->changeWorkspace);
+			return $this->doc->wrapScriptTags('top.location.href="alt_main.php";');
+		}
+			// Changing workspace and if so, reloading entire backend:
+		if (strlen($this->changeWorkspacePreview))	{
+			$BE_USER->setWorkspacePreview($this->changeWorkspacePreview);
+		}
+
+			// Create options array:
+		$options = array();
+		if ($BE_USER->checkWorkspace(array('uid' => 0)))	{
+			$options[0] = '['.$LANG->getLL('shortcut_onlineWS').']';
+		}
+		if ($BE_USER->checkWorkspace(array('uid' => -1)))	{
+			$options[-1] = '['.$LANG->getLL('shortcut_offlineWS').']';
+		}
+
+			// Add custom workspaces (selecting all, filtering by BE_USER check):
+		$workspaces = $TYPO3_DB->exec_SELECTgetRows('uid,title,adminusers,members,reviewers','sys_workspace','pid=0'.t3lib_BEfunc::deleteClause('sys_workspace'),'','title');
+		if (count($workspaces))	{
+			foreach ($workspaces as $rec)	{
+				if ($BE_USER->checkWorkspace($rec))	{
+					$options[$rec['uid']] = $rec['uid'].': '.$rec['title'];
+				}
+			}
+		}
+
+			// Build selector box:
+		if (count($options))	{
+			foreach($options as $value => $label)	{
+				$selected = ((int)$BE_USER->workspace===$value ? ' selected="selected"' : '');
+				$options[$value] = '<option value="'.htmlspecialchars($value).'"'.$selected.'>'.htmlspecialchars($label).'</option>';
+			}
+		} else {
+			$options[] = '<option value="-99">'.$LANG->getLL('shortcut_noWSfound',1).'</option>';
+		}
+
+		$selector = '';
+			// Preview:
+		if ($BE_USER->workspace!==0)	{
+			$selector.= 'Frontend Preview: <input type="checkbox" onclick="changeWorkspacePreview('.($BE_USER->user['workspace_preview'] ? 0 : 1).')"; '.($BE_USER->user['workspace_preview'] ? 'checked="checked"' : '').'/>&nbsp;';
+		}
+
+		$selector.= '<a href="mod/user/ws/index.php" target="content">'.
+					t3lib_iconWorks::getIconImage('sys_workspace',array(),$this->doc->backPath,'align="top"').
+					'</a>'.
+					'<select name="_workspaceSelector" onchange="changeWorkspace(this.options[this.selectedIndex].value);">'.implode('',$options).'</select>';
+
+		return $selector;
+	}
+
+
+
+
+
 
 
 	/***************************
@@ -464,6 +684,9 @@ class SC_alt_shortcut {
 	 * @return	void
 	 */
 	function mIconFilename($Ifilename,$backPath)	{
+			// Change icon of fileadmin references - otherwise it doesn't differ with Web->List
+		$Ifilename = str_replace ('mod/file/list/list.gif', 'mod/file/file.gif', $Ifilename);
+
 		if (t3lib_div::isAbsPath($Ifilename))	{
 			$Ifilename = '../'.substr($Ifilename,strlen(PATH_site));
 		}
@@ -515,6 +738,16 @@ class SC_alt_shortcut {
 		$label.=': '.$inlabel;
 		return $label;
 	}
+
+	/**
+	 * Return the ID of the page in the URL if found.
+	 *
+	 * @param	string		The URL of the current shortcut link
+	 * @return	string		If a page ID was found, it is returned. Otherwise: 0
+	 */
+	function getLinkedPageId($url)	{
+		return preg_replace('/.*[\?&]id=([^&]+).*/', '$1', $url);
+	}
 }
 
 // Include extension?
