@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -30,7 +30,7 @@
  * This script lets users choose a new database element to create.
  * Includes a wizard mode for visually pointing out the position of new pages
  *
- * $Id: db_new.php,v 1.15 2004/09/13 22:57:22 typo3 Exp $
+ * $Id: db_new.php 1421 2006-04-10 09:27:15Z mundaun $
  * Revised for TYPO3 3.6 November/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -48,13 +48,13 @@
  *
  *  128: class SC_db_new
  *  157:     function init()
- *  217:     function main()
- *  269:     function pagesOnly()
- *  287:     function regularNew()
- *  442:     function printContent()
- *  456:     function linkWrap($code,$table,$pid,$addContentTable=0)
- *  476:     function isTableAllowedForThisPage($pid_row, $checkTable)
- *  506:     function showNewRecLink($table,$allowedNewTables='')
+ *  224:     function main()
+ *  276:     function pagesOnly()
+ *  294:     function regularNew()
+ *  458:     function printContent()
+ *  473:     function linkWrap($code,$table,$pid,$addContentTable=0)
+ *  493:     function isTableAllowedForThisPage($pid_row, $checkTable)
+ *  523:     function showNewRecLink($table,$allowedNewTables='')
  *
  * TOTAL FUNCTIONS: 10
  * (This index is automatically created/updated by the extension "extdeveval")
@@ -160,6 +160,12 @@ class SC_db_new {
 			// page-selection permission clause (reading)
 		$this->perms_clause = $BE_USER->getPagePermsClause(1);
 
+			// this will hide records from display - it has nothing todo with user rights!!
+		if ($pidList = $GLOBALS['BE_USER']->getTSConfigVal('options.hideRecords.pages')) {
+			if ($pidList = $GLOBALS['TYPO3_DB']->cleanIntList($pidList)) {
+				$this->perms_clause .= ' AND pages.uid NOT IN ('.$pidList.')';
+			}
+		}
 			// Setting GPvars:
 		$this->id = intval(t3lib_div::_GP('id'));	// The page id to operate from
 		$this->returnUrl = t3lib_div::_GP('returnUrl');
@@ -184,6 +190,7 @@ class SC_db_new {
 			// If a page-record was returned, the user had read-access to the page.
 		if ($this->pageinfo['uid'])	{
 				// Get record of parent page
+
 			$this->pidInfo=t3lib_BEfunc::getRecord('pages',$this->pageinfo['pid']);
 				// Checking the permissions for the user with regard to the parent page: Can he create new pages, new content record, new page after?
 			if ($BE_USER->doesUserHaveAccess($this->pageinfo,8))	{
@@ -296,7 +303,11 @@ class SC_db_new {
 		$tRows = array();
 
 			// New pages INSIDE this pages
-		if ($this->newPagesInto && $this->isTableAllowedForThisPage($this->pageinfo, 'pages') && $BE_USER->check('tables_modify','pages'))	{
+		if ($this->newPagesInto
+			&& $this->isTableAllowedForThisPage($this->pageinfo, 'pages')
+			&& $BE_USER->check('tables_modify','pages')
+			&& $BE_USER->workspaceCreateNewRecord($this->pageinfo['_ORIG_uid']?$this->pageinfo['_ORIG_uid']:$this->id, 'pages')
+			)	{
 
 				// Create link to new page inside:
 			$t = 'pages';
@@ -335,6 +346,7 @@ class SC_db_new {
 							&& $this->isTableAllowedForThisPage($this->pageinfo, $t)
 							&& $BE_USER->check('tables_modify',$t)
 							&& (($v['ctrl']['rootLevel'] xor $this->id) || $v['ctrl']['rootLevel']==-1)
+							&& $BE_USER->workspaceCreateNewRecord($this->pageinfo['_ORIG_uid']?$this->pageinfo['_ORIG_uid']:$this->id, $t)
 							)	{
 
 							// Create new link for record:
@@ -378,7 +390,11 @@ class SC_db_new {
 		}
 
 			// New pages AFTER this pages
-		if ($this->newPagesAfter && $this->isTableAllowedForThisPage($this->pidInfo,'pages') && $BE_USER->check('tables_modify','pages'))	{
+		if ($this->newPagesAfter
+			&& $this->isTableAllowedForThisPage($this->pidInfo,'pages')
+			&& $BE_USER->check('tables_modify','pages')
+			&& $BE_USER->workspaceCreateNewRecord($this->pidInfo['uid'], 'pages')
+			)	{
 
 				// Create link to new page after
 			$t = 'pages';
@@ -441,6 +457,7 @@ class SC_db_new {
 	 */
 	function printContent()	{
 		$this->content.= $this->doc->endPage();
+		$this->content = $this->doc->insertStylesAndJS($this->content);
 		echo $this->content;
 	}
 
