@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Include file extending db_list.inc for use with the web_layout module
  *
- * $Id: class.tx_cms_layout.php,v 1.16 2004/09/13 22:57:35 typo3 Exp $
+ * $Id: class.tx_cms_layout.php 1421 2006-04-10 09:27:15Z mundaun $
  * Revised for TYPO3 3.6 November/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -38,65 +38,66 @@
  *
  *
  *
- *  114: class tx_cms_layout extends recordList
+ *  115: class tx_cms_layout extends recordList
  *
  *              SECTION: Renderings
- *  179:     function getTable($table,$id)
- *  239:     function getTable_pages($id)
- *  376:     function getTable_tt_content($id)
- *  736:     function getTable_fe_users($id)
- *  762:     function getTable_sys_note($id)
- *  853:     function getTable_tt_board($id)
- *  933:     function getTable_tt_address($id)
- *  963:     function getTable_tt_links($id)
- *  989:     function getTable_tt_guest($id)
- * 1004:     function getTable_tt_news($id)
- * 1025:     function getTable_tt_calender($id)
- * 1075:     function getTable_tt_products($id)
+ *  180:     function getTable($table,$id)
+ *  240:     function getTable_pages($id)
+ *  378:     function getTable_tt_content($id)
+ *  754:     function getTable_fe_users($id)
+ *  780:     function getTable_sys_note($id)
+ *  873:     function getTable_tt_board($id)
+ *  955:     function getTable_tt_address($id)
+ *  985:     function getTable_tt_links($id)
+ * 1011:     function getTable_tt_guest($id)
+ * 1026:     function getTable_tt_news($id)
+ * 1047:     function getTable_tt_calender($id)
+ * 1097:     function getTable_tt_products($id)
  *
  *              SECTION: Generic listing of items
- * 1121:     function makeOrdinaryList($table, $id, $fList, $icon=0, $addWhere='')
- * 1200:     function dataFields($fieldArr,$table,$row,$out=array())
- * 1251:     function headerFields($fieldArr,$table,$out=array())
+ * 1143:     function makeOrdinaryList($table, $id, $fList, $icon=0, $addWhere='')
+ * 1224:     function dataFields($fieldArr,$table,$row,$out=array())
+ * 1275:     function headerFields($fieldArr,$table,$out=array())
  *
  *              SECTION: Additional functions; Pages
- * 1293:     function pages_getTree($theRows,$pid,$qWhere,$treeIcons,$depth)
- * 1325:     function pages_drawItem($row,$fieldArr)
+ * 1317:     function pages_getTree($theRows,$pid,$qWhere,$treeIcons,$depth)
+ * 1350:     function pages_drawItem($row,$fieldArr)
  *
  *              SECTION: Additional functions; Content Elements
- * 1435:     function tt_content_drawColHeader($colName,$editParams,$newParams)
- * 1487:     function tt_content_drawHeader($row,$space=0,$disableMoveAndNewButtons=FALSE,$langMode=FALSE)
- * 1617:     function tt_content_drawItem($row, $isRTE=FALSE)
- * 1780:     function getNonTranslatedTTcontentUids($defLanguageCount,$id,$lP)
- * 1810:     function newLanguageButton($defLanguageCount,$lP)
- * 1831:     function infoGif($infoArr)
- * 1847:     function newContentElementOnClick($id,$colPos,$sys_language)
- * 1865:     function linkEditContent($str,$row)
- * 1883:     function linkRTEbutton($row)
- * 1904:     function languageSelector($id)
- * 1939:     function getResult($result)
+ * 1461:     function tt_content_drawColHeader($colName,$editParams,$newParams)
+ * 1513:     function tt_content_drawHeader($row,$space=0,$disableMoveAndNewButtons=FALSE,$langMode=FALSE)
+ * 1643:     function tt_content_drawItem($row, $isRTE=FALSE)
+ * 1806:     function getNonTranslatedTTcontentUids($defLanguageCount,$id,$lP)
+ * 1836:     function newLanguageButton($defLanguageCount,$lP)
+ * 1857:     function infoGif($infoArr)
+ * 1873:     function newContentElementOnClick($id,$colPos,$sys_language)
+ * 1891:     function linkEditContent($str,$row)
+ * 1909:     function linkRTEbutton($row)
+ * 1930:     function languageSelector($id)
+ * 1967:     function getResult($result)
  *
  *              SECTION: Additional functions; Message board items (tt_board)
- * 2008:     function tt_board_getTree($theRows,$parent,$pid,$qWhere,$treeIcons)
- * 2043:     function tt_board_drawItem($table,$row,$re)
+ * 2036:     function tt_board_getTree($theRows,$parent,$pid,$qWhere,$treeIcons)
+ * 2071:     function tt_board_drawItem($table,$row,$re)
  *
  *              SECTION: Various helper functions
- * 2090:     function numberOfRecords($table,$pid)
- * 2109:     function renderText($input)
- * 2123:     function getIcon($table,$row)
- * 2146:     function getProcessedValue($table,$fieldList,$row,&$info)
- * 2166:     function isDisabled($table,$row)
- * 2184:     function wordWrapper($content,$max=50,$char=' -')
- * 2201:     function noEditIcon($label='noEditItems')
- * 2210:     function cleanTableNames()
- * 2246:     function isRTEforField($table,$row,$field)
- * 2275:     function getSpecConfForField($table,$row,$field)
+ * 2118:     function numberOfRecords($table,$pid)
+ * 2137:     function renderText($input)
+ * 2151:     function getIcon($table,$row)
+ * 2174:     function getProcessedValue($table,$fieldList,$row,&$info)
+ * 2194:     function isDisabled($table,$row)
+ * 2212:     function wordWrapper($content,$max=50,$char=' -')
+ * 2229:     function noEditIcon($label='noEditItems')
+ * 2238:     function cleanTableNames()
+ * 2274:     function isRTEforField($table,$row,$field)
+ * 2304:     function getSpecConfForField($table,$row,$field)
  *
  *              SECTION: External renderings
- * 2312:     function getPageInfoBox($rec,$edit=0)
- * 2481:     function getTableMenu($id)
+ * 2341:     function getPageInfoBox($rec,$edit=0)
+ * 2510:     function getTableMenu($id)
+ * 2575:     function strip_tags($content, $fillEmptyContent=false)
  *
- * TOTAL FUNCTIONS: 42
+ * TOTAL FUNCTIONS: 43
  * (This index is automatically created/updated by the extension "extdeveval")
  *
  */
@@ -122,7 +123,7 @@ class tx_cms_layout extends recordList {
 	var $pI_showStat=1;						// If true, hit statistics are shown in the page info box.
 	var $nextThree = 3;						// The number of successive records to edit when showing content elements.
 	var $pages_noEditColumns=0;				// If true, disables the edit-column icon for tt_content elements
-	var $option_showBigButtons=1;			// If true, shows bit buttons for editing page header, moving, creating elements etc. in the columns view.
+	var $option_showBigButtons=1;			// If true, shows big buttons for editing page properties, moving, creating elements etc. in the columns view.
 	var $option_newWizard=1;				// If true, new-wizards are linked to rather than the regular new-element list.
 	var $ext_function=0;					// If set to "1", will link a big button to content element wizard.
 	var $doEdit=1;							// If true, elements will have edit icons (probably this is whethere the user has permission to edit the page content). Set externally.
@@ -249,6 +250,7 @@ class tx_cms_layout extends recordList {
 		} else {
 			$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'pages', 'uid='.intval($id).$delClause);
 			$row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result);
+			t3lib_BEfunc::workspaceOL('pages', $row);
 		}
 
 			// If there was found a page:
@@ -266,10 +268,10 @@ class tx_cms_layout extends recordList {
 					$this->fieldArray = explode(',','title,uid,'.implode(',',array_keys($tableNames)));
 				break;
 				case 2:
-					$this->fieldArray = explode(',','title,uid,lastUpdated,newUntil,no_cache,cache_timeout,php_tree_stop,TSconfig,storage_pid,is_siteroot');
+					$this->fieldArray = explode(',','title,uid,lastUpdated,newUntil,no_cache,cache_timeout,php_tree_stop,TSconfig,storage_pid,is_siteroot,fe_login_mode');
 				break;
 				default:
-					$this->fieldArray = explode(',','title,uid,alias,starttime,endtime,fe_group,target,url,shortcut');
+					$this->fieldArray = explode(',','title,uid,alias,starttime,endtime,fe_group,target,url,shortcut,shortcut_mode');
 				break;
 			}
 
@@ -296,7 +298,7 @@ class tx_cms_layout extends recordList {
 
 					// Getting children:
 				$theRows = Array();
-				$theRows = $this->pages_getTree($theRows,$row['uid'],$delClause,'',$depth);
+				$theRows = $this->pages_getTree($theRows,$row['uid'],$delClause.t3lib_BEfunc::versioningPlaceholderClause('pages'),'',$depth);
 				if ($GLOBALS['BE_USER']->doesUserHaveAccess($row,2))	$editUids[]=$row['uid'];
 				$out.=$this->pages_drawItem($row,$this->fieldArray);
 
@@ -382,7 +384,7 @@ class tx_cms_layout extends recordList {
 		$RTE = $GLOBALS['BE_USER']->isRTE();
 		$lMarg=1;
 		$showHidden = $this->tt_contentConfig['showHidden']?'':t3lib_BEfunc::BEenableFields('tt_content');
-		$pageTitleParamForAltDoc='&recTitle='.rawurlencode(t3lib_BEfunc::getRecordTitle('pages',t3lib_BEfunc::getRecord('pages',$id),1));
+		$pageTitleParamForAltDoc='&recTitle='.rawurlencode(t3lib_BEfunc::getRecordTitle('pages',t3lib_BEfunc::getRecordWSOL('pages',$id),1));
 
 			// Get labels for CTypes and tt_content element fields in general:
 		$this->CType_labels =array();
@@ -429,7 +431,7 @@ class tx_cms_layout extends recordList {
 
 						// If it turns out that there are not content elements in the column, then display a big button which links directly to the wizard script:
 					if ($this->doEdit && $this->option_showBigButtons && !intval($key) && !$GLOBALS['TYPO3_DB']->sql_num_rows($result))	{
-						$onClick = "document.location='db_new_content_el.php?id=".$id.'&colPos='.intval($key).'&sys_language_uid='.$lP.'&uid_pid='.$id.'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';";
+						$onClick = "window.location.href='db_new_content_el.php?id=".$id.'&colPos='.intval($key).'&sys_language_uid='.$lP.'&uid_pid='.$id.'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';";
 						$theNewButton = $GLOBALS['SOBE']->doc->t3Button($onClick,$GLOBALS['LANG']->getLL('newPageContent'));
 						$content[$key].= '<img src="clear.gif" width="1" height="5" alt="" /><br />'.$theNewButton;
 					}
@@ -437,21 +439,25 @@ class tx_cms_layout extends recordList {
 						// Traverse any selected elements and render their display code:
 					$rowArr = $this->getResult($result);
 
-					foreach($rowArr as $row)	{
-						$singleElementHTML = '';
-						if (!$lP) $defLanguageCount[$key][] = $row['uid'];
+					foreach($rowArr as $rKey => $row)	{
+						t3lib_BEfunc::workspaceOL('tt_content', $row);
 
-						$editUidList.= $row['uid'].',';
-						$singleElementHTML.= $this->tt_content_drawHeader($row,$this->tt_contentConfig['showInfo']?15:5, $this->defLangBinding && $lP>0, TRUE);
+						if ((int)$row['t3ver_state']!=2)	{
+							$singleElementHTML = '';
+							if (!$lP) $defLanguageCount[$key][] = $row['uid'];
 
-						$isRTE = $RTE && $this->isRTEforField('tt_content',$row,'bodytext');
-						$singleElementHTML.= $this->tt_content_drawItem($row,$isRTE);
+							$editUidList.= $row['uid'].',';
+							$singleElementHTML.= $this->tt_content_drawHeader($row,$this->tt_contentConfig['showInfo']?15:5, $this->defLangBinding && $lP>0, TRUE);
 
-						if ($this->defLangBinding && $this->tt_contentConfig['languageMode'])	{
-							$defLangBinding[$key][$lP][$row[($lP ? 'l18n_parent' : 'uid')]] = $singleElementHTML;
-						} else {
-							$content[$key].= $singleElementHTML;
-						}
+							$isRTE = $RTE && $this->isRTEforField('tt_content',$row,'bodytext');
+							$singleElementHTML.= '<div '.($row['_ORIG_uid'] ? ' class="ver-element"' :'').'>'.$this->tt_content_drawItem($row,$isRTE).'</div>';
+
+							if ($this->defLangBinding && $this->tt_contentConfig['languageMode'])	{
+								$defLangBinding[$key][$lP][$row[($lP ? 'l18n_parent' : 'uid')]] = $singleElementHTML;
+							} else {
+								$content[$key].= $singleElementHTML;
+							}
+						} else { unset($rowArr[$rKey]); }
 					}
 
 						// Add new-icon link, header:
@@ -528,6 +534,7 @@ class tx_cms_layout extends recordList {
 					if ($lP)	{
 
 						list($lpRecord) = t3lib_BEfunc::getRecordsByField('pages_language_overlay','pid',$id,'AND sys_language_uid='.intval($lP));
+						t3lib_BEfunc::workspaceOL('pages_language_overlay',$lpRecord);
 						$params='&edit[pages_language_overlay]['.$lpRecord['uid'].']=edit&overrideVals[pages_language_overlay][sys_language_uid]='.$lP;
 						$lPLabel = $GLOBALS['SOBE']->doc->wrapClickMenuOnIcon(t3lib_iconWorks::getIconImage('pages_language_overlay',$lpRecord,$this->backPath,'  class="absmiddle"'),'pages_language_overlay',$lpRecord['uid']).
 							$viewLink.
@@ -599,7 +606,13 @@ class tx_cms_layout extends recordList {
 			if ($this->tt_contentConfig['sys_language_uid']==0 || !$this->defLangBinding)	{
 
 					// Initialize:
-				$showLanguage = $this->defLangBinding && $this->tt_contentConfig['sys_language_uid']==0 ? ' AND sys_language_uid IN (0,-1)' : ' AND sys_language_uid='.$this->tt_contentConfig['sys_language_uid'];
+				if ($this->defLangBinding && $this->tt_contentConfig['sys_language_uid']==0)	{
+					$showLanguage = ' AND sys_language_uid IN (0,-1)';
+					$lP = 0;
+				} else {
+					$showLanguage = ' AND sys_language_uid='.$this->tt_contentConfig['sys_language_uid'];
+					$lP = $this->tt_contentConfig['sys_language_uid'];
+				}
 
 				$cList = explode(',',$this->tt_contentConfig['showSingleCol']);
 				$content=array();
@@ -626,34 +639,39 @@ class tx_cms_layout extends recordList {
 
 						// If it turns out that there are not content elements in the column, then display a big button which links directly to the wizard script:
 					if ($this->doEdit && $this->option_showBigButtons && !intval($key) && !$GLOBALS['TYPO3_DB']->sql_num_rows($result))	{
-						$onClick="document.location='db_new_content_el.php?id=".$id.'&colPos='.intval($key).'&sys_language_uid='.$lP.'&uid_pid='.$id.'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';";
+						$onClick="window.location.href='db_new_content_el.php?id=".$id.'&colPos='.intval($key).'&sys_language_uid='.$lP.'&uid_pid='.$id.'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';";
 						$theNewButton=$GLOBALS['SOBE']->doc->t3Button($onClick,$GLOBALS['LANG']->getLL('newPageContent'));
 						$theNewButton='<img src="clear.gif" width="1" height="5" alt="" /><br />'.$theNewButton;
 					} else $theNewButton='';
 
 						// Traverse any selected elements:
-					foreach($rowArr as $row)	{
-						$c++;
-						$editUidList.=$row['uid'].',';
-						$isRTE=$RTE && $this->isRTEforField('tt_content',$row,'bodytext');
+					foreach($rowArr as $rKey => $row)	{
+						t3lib_BEfunc::workspaceOL('tt_content', $row);
 
-							// Create row output:
-						$rowOut.='
-							<tr>
-								<td></td>
-								<td valign="top">'.$this->tt_content_drawHeader($row).'</td>
-								<td></td>
-								<td valign="top">'.$this->tt_content_drawItem($row,$isRTE).'</td>
-							</tr>';
+						if ((int)$row['t3ver_state']!=2)	{
 
-							// If the element was not the last element, add a divider line:
-						if ($c != $GLOBALS['TYPO3_DB']->sql_num_rows($result))	{
+							$c++;
+							$editUidList.=$row['uid'].',';
+							$isRTE=$RTE && $this->isRTEforField('tt_content',$row,'bodytext');
+
+								// Create row output:
 							$rowOut.='
-							<tr>
-								<td></td>
-								<td colspan="3"><img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/stiblet_medium2.gif','width="468" height="1"').' class="c-divider" alt="" /></td>
-							</tr>';
-						}
+								<tr>
+									<td></td>
+									<td valign="top">'.$this->tt_content_drawHeader($row).'</td>
+									<td>&nbsp;</td>
+									<td'.($row['_ORIG_uid'] ? ' class="ver-element"' :'').' valign="top">'.$this->tt_content_drawItem($row,$isRTE).'</td>
+								</tr>';
+
+								// If the element was not the last element, add a divider line:
+							if ($c != $GLOBALS['TYPO3_DB']->sql_num_rows($result))	{
+								$rowOut.='
+								<tr>
+									<td></td>
+									<td colspan="3"><img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/stiblet_medium2.gif','width="468" height="1"').' class="c-divider" alt="" /></td>
+								</tr>';
+							}
+						} else { unset($rowArr[$rKey]); }
 					}
 
 						// Add spacer between sections in the vertical list
@@ -701,16 +719,16 @@ class tx_cms_layout extends recordList {
 			$bArray=array();
 
 			if (!$GLOBALS['SOBE']->current_sys_language)	{
-				if ($this->ext_CALC_PERMS&2)	$bArray[0]=$GLOBALS['SOBE']->doc->t3Button(t3lib_BEfunc::editOnClick('&edit[pages]['.$id."]=edit",$this->backPath,''),$GLOBALS['LANG']->getLL('editPageTitle'));
+				if ($this->ext_CALC_PERMS&2)	$bArray[0]=$GLOBALS['SOBE']->doc->t3Button(t3lib_BEfunc::editOnClick('&edit[pages]['.$id."]=edit",$this->backPath,''),$GLOBALS['LANG']->getLL('editPageProperties'));
 			} else {
 				if ($this->doEdit && $GLOBALS['BE_USER']->check('tables_modify','pages_language_overlay'))	{
 					list($languageOverlayRecord) = t3lib_BEfunc::getRecordsByField('pages_language_overlay','pid',$id,'AND sys_language_uid='.intval($GLOBALS['SOBE']->current_sys_language));
-					$bArray[0]=$GLOBALS['SOBE']->doc->t3Button(t3lib_BEfunc::editOnClick('&edit[pages_language_overlay]['.$languageOverlayRecord['uid']."]=edit",$this->backPath,''),$GLOBALS['LANG']->getLL('editPageTitle_curLang'));
+					$bArray[0]=$GLOBALS['SOBE']->doc->t3Button(t3lib_BEfunc::editOnClick('&edit[pages_language_overlay]['.$languageOverlayRecord['uid']."]=edit",$this->backPath,''),$GLOBALS['LANG']->getLL('editPageProperties_curLang'));
 				}
 			}
-			if ($this->ext_CALC_PERMS&4 || $this->ext_CALC_PERMS&2)	$bArray[1]=$GLOBALS['SOBE']->doc->t3Button("document.location='".$this->backPath."move_el.php?table=pages&uid=".$id.'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';",$GLOBALS['LANG']->getLL('move_page'));
-			if ($this->ext_CALC_PERMS&8)	$bArray[2]=$GLOBALS['SOBE']->doc->t3Button("document.location='".$this->backPath."db_new.php?id=".$id.'&pagesOnly=1&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';",$GLOBALS['LANG']->getLL('newPage2'));
-			if ($this->doEdit && $this->ext_function==1) $bArray[3]=$GLOBALS['SOBE']->doc->t3Button("document.location='db_new_content_el.php?id=".$id.'&sys_language_uid='.$GLOBALS['SOBE']->current_sys_language.'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';",$GLOBALS['LANG']->getLL('newPageContent2'));
+			if ($this->ext_CALC_PERMS&4 || $this->ext_CALC_PERMS&2)	$bArray[1]=$GLOBALS['SOBE']->doc->t3Button("window.location.href='".$this->backPath."move_el.php?table=pages&uid=".$id.'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';",$GLOBALS['LANG']->getLL('move_page'));
+			if ($this->ext_CALC_PERMS&8)	$bArray[2]=$GLOBALS['SOBE']->doc->t3Button("window.location.href='".$this->backPath."db_new.php?id=".$id.'&pagesOnly=1&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';",$GLOBALS['LANG']->getLL('newPage2'));
+			if ($this->doEdit && $this->ext_function==1) $bArray[3]=$GLOBALS['SOBE']->doc->t3Button("window.location.href='db_new_content_el.php?id=".$id.'&sys_language_uid='.$GLOBALS['SOBE']->current_sys_language.'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';",$GLOBALS['LANG']->getLL('newPageContent2'));
 			$out = '
 				<table border="0" cellpadding="4" cellspacing="0" class="typo3-page-buttons">
 					<tr>
@@ -776,8 +794,8 @@ class tx_cms_layout extends recordList {
 			// If page ids were found, select all sys_notes from the page ids:
 		$out='';
 		if (count($tree->ids))		{
-			$delClause = t3lib_BEfunc::deleteClause('sys_note');
-			$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'sys_note', 'pid IN ('.implode(',',$tree->ids).') AND (!personal OR cruser='.intval($GLOBALS['BE_USER']->user['uid']).')'.$delClause);
+			$delClause = t3lib_BEfunc::deleteClause('sys_note').t3lib_BEfunc::versioningPlaceholderClause('sys_note');
+			$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'sys_note', 'pid IN ('.implode(',',$tree->ids).') AND (personal=0 OR cruser='.intval($GLOBALS['BE_USER']->user['uid']).')'.$delClause);
 			$dbCount = $GLOBALS['TYPO3_DB']->sql_num_rows($result);
 
 				// If sys_notes were found, render them:
@@ -801,6 +819,8 @@ class tx_cms_layout extends recordList {
 					// Items
 				$this->eCounter=$this->firstElementNumber;
 				while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result))	{
+					t3lib_BEfunc::workspaceOL('sys_note', $row);
+
 					list($flag,$code) = $this->fwd_rwd_nav();
 					$out.=$code;
 					if ($flag)	{
@@ -853,7 +873,7 @@ class tx_cms_layout extends recordList {
 	function getTable_tt_board($id)	{
 
 			// Initialize:
-		$delClause = t3lib_BEfunc::deleteClause('tt_board');
+		$delClause = t3lib_BEfunc::deleteClause('tt_board').t3lib_BEfunc::versioningPlaceholderClause('tt_board');
 		$queryParts = $this->makeQueryArray('tt_board', $id, 'AND parent=0');
 		$this->setTotalItems($queryParts);
 		$dbCount = 0;
@@ -894,6 +914,8 @@ class tx_cms_layout extends recordList {
 				// Items
 			$this->eCounter=$this->firstElementNumber;
 			while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result))	{
+				t3lib_BEfunc::workspaceOL('tt_board', $row);
+
 				list($flag,$code) = $this->fwd_rwd_nav();
 				$out.=$code;
 
@@ -1148,6 +1170,8 @@ class tx_cms_layout extends recordList {
 				// Render Items
 			$this->eCounter = $this->firstElementNumber;
 			while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result))	{
+				t3lib_BEfunc::workspaceOL($table, $row);
+
 				list($flag,$code) = $this->fwd_rwd_nav();
 				$out.= $code;
 				if ($flag)	{
@@ -1212,7 +1236,7 @@ class tx_cms_layout extends recordList {
 					if ($fieldName==$thumbsCol)	{	// If the column is a thumbnail column:
 						$out[$fieldName] = $this->thumbCode($row,$table,$fieldName);
 					} else {	// ... otherwise just render the output:
-						$out[$fieldName] = nl2br(htmlspecialchars(trim(t3lib_div::fixed_lgd_cs(t3lib_BEfunc::getProcessedValue($table,$fieldName,$row[$fieldName]),250))));
+						$out[$fieldName] = nl2br(htmlspecialchars(trim(t3lib_div::fixed_lgd_cs(t3lib_BEfunc::getProcessedValue($table,$fieldName,$row[$fieldName],0,0,0,$row['uid']),250))));
 					}
 				} else {	// Each field is separated by <br /> and shown in the same cell (If not a TCA field, then explode the field name with ";" and check each value there as a TCA configured field)
 					$theFields = explode(';',$fieldName);
@@ -1222,7 +1246,7 @@ class tx_cms_layout extends recordList {
 						if ($TCA[$table]['columns'][$fName2])	{
 							 $out[$fieldName].= '<b>'.$GLOBALS['LANG']->sL($TCA[$table]['columns'][$fName2]['label'],1).'</b>'.
 							 					'&nbsp;&nbsp;'.
-												htmlspecialchars(t3lib_div::fixed_lgd_cs(t3lib_BEfunc::getProcessedValue($table,$fName2,$row[$fName2]),25)).
+												htmlspecialchars(t3lib_div::fixed_lgd_cs(t3lib_BEfunc::getProcessedValue($table,$fName2,$row[$fName2],0,0,0,$row['uid']),25)).
 												'<br />';
 						}
 					}
@@ -1297,6 +1321,7 @@ class tx_cms_layout extends recordList {
 			$c=0;
 			$rc = $GLOBALS['TYPO3_DB']->sql_num_rows($res);
 			while($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res))	{
+				t3lib_BEfunc::workspaceOL('pages', $row);
 				$c++;
 				$row['treeIcons'] = $treeIcons.'<img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/ol/join'.($rc==$c?'bottom':'').'.gif','width="18" height="16"').' alt="" />';
 				$theRows[]=$row;
@@ -1404,6 +1429,7 @@ class tx_cms_layout extends recordList {
 				break;
 			}
 		}
+		$this->addElement_tdParams['title'] = ($row['_CSSCLASS'] ? ' class="'.$row['_CSSCLASS'].'"' : '');
 		return $this->addelement(1,'',$theData);
 	}
 
@@ -1532,7 +1558,7 @@ class tx_cms_layout extends recordList {
 				if (!$disableMoveAndNewButtons)	{
 						// New content element:
 					if ($this->option_newWizard)	{
-						$onClick="document.location='db_new_content_el.php?id=".$row['pid'].'&sys_language_uid='.$row['sys_language_uid'].'&colPos='.$row['colPos'].'&uid_pid='.(-$row['uid']).'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';";
+						$onClick="window.location.href='db_new_content_el.php?id=".$row['pid'].'&sys_language_uid='.$row['sys_language_uid'].'&colPos='.$row['colPos'].'&uid_pid='.(-$row['uid']).'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';";
 					} else {
 						$params='&edit[tt_content]['.(-$row['uid']).']=new';
 						$onClick = t3lib_BEfunc::editOnClick($params,$this->backPath);
@@ -1565,12 +1591,12 @@ class tx_cms_layout extends recordList {
 				$hiddenField = $TCA['tt_content']['ctrl']['enablecolumns']['disabled'];
 				if ($hiddenField && $TCA['tt_content']['columns'][$hiddenField] && (!$TCA['tt_content']['columns'][$hiddenField]['exclude'] || $GLOBALS['BE_USER']->check('non_exclude_fields','tt_content:'.$hiddenField)))	{
 					if ($row[$hiddenField])	{
-						$params='&data[tt_content]['.$row['uid'].']['.$hiddenField.']=0';
+						$params='&data[tt_content]['.($row['_ORIG_uid'] ? $row['_ORIG_uid'] : $row['uid']).']['.$hiddenField.']=0';
 						$out.='<a href="'.htmlspecialchars($GLOBALS['SOBE']->doc->issueCommand($params)).'">'.
 								'<img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/button_unhide.gif','width="11" height="10"').' title="'.$GLOBALS['LANG']->getLL('unHide',1).'" alt="" />'.
 								'</a>';
 					} else {
-						$params='&data[tt_content]['.$row['uid'].']['.$hiddenField.']=1';
+						$params='&data[tt_content]['.($row['_ORIG_uid'] ? $row['_ORIG_uid'] : $row['uid']).']['.$hiddenField.']=1';
 						$out.='<a href="'.htmlspecialchars($GLOBALS['SOBE']->doc->issueCommand($params)).'">'.
 								'<img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/button_hide.gif','width="11" height="10"').' title="'.$GLOBALS['LANG']->getLL('hide',1).'" alt="" />'.
 								'</a>';
@@ -1646,7 +1672,7 @@ class tx_cms_layout extends recordList {
 				if ($row['CType']=='text' || $row['CType']=='textpic')	{
 					if ($row['bodytext'])	{
 						$this->getProcessedValue('tt_content','text_align,text_face,text_size,text_color,text_properties',$row,$infoArr);
-						$out.=	$this->infoGif($infoArr).
+						$out.= $this->infoGif($infoArr).
 								$this->linkEditContent($this->renderText($row['bodytext']),$row).'<br />';
 					}
 				}
@@ -1816,7 +1842,7 @@ class tx_cms_layout extends recordList {
 			}
 
 				// Copy for language:
-			$onClick = "document.location='".$GLOBALS['SOBE']->doc->issueCommand($params)."'; return false;";
+			$onClick = "window.location.href='".$GLOBALS['SOBE']->doc->issueCommand($params)."'; return false;";
 			$theNewButton = $GLOBALS['SOBE']->doc->t3Button($onClick,$GLOBALS['LANG']->getLL('newPageContent_copyForLang').' ['.count($defLanguageCount).']');
 			return $theNewButton;
 		}
@@ -1846,7 +1872,7 @@ class tx_cms_layout extends recordList {
 	 */
 	function newContentElementOnClick($id,$colPos,$sys_language)	{
 		if ($this->option_newWizard)	{
-			$onClick="document.location='db_new_content_el.php?id=".$id.'&colPos='.$colPos.'&sys_language_uid='.$sys_language.'&uid_pid='.$id.'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';";
+			$onClick="window.location.href='db_new_content_el.php?id=".$id.'&colPos='.$colPos.'&sys_language_uid='.$sys_language.'&uid_pid='.$id.'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI'))."';";
 		} else {
 			$onClick=t3lib_BEfunc::editOnClick('&edit[tt_content]['.$id.']=new&defVals[tt_content][colPos]='.$colPos.'&defVals[tt_content][sys_language_uid]='.$sys_language,$this->backPath);
 		}
@@ -1887,7 +1913,7 @@ class tx_cms_layout extends recordList {
 		$params['pid'] = $row['pid'];
 		$params['field'] = 'bodytext';
 		$params['returnUrl'] = t3lib_div::linkThisScript();
-		$RTEonClick = "document.location='".$this->backPath."wizard_rte.php?".t3lib_div::implodeArrayForUrl('',array('P'=>$params))."';return false;";
+		$RTEonClick = "window.location.href='".$this->backPath."wizard_rte.php?".t3lib_div::implodeArrayForUrl('',array('P'=>$params))."';return false;";
 		$addButton = $this->option_showBigButtons && $this->doEdit ? $GLOBALS['SOBE']->doc->t3Button($RTEonClick,$GLOBALS['LANG']->getLL('editInRTE')) : '';
 
 		return $addButton;
@@ -1910,8 +1936,10 @@ class tx_cms_layout extends recordList {
 			$langSelItems[0]='
 						<option value="0"></option>';
 			while($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res))	{
-				$langSelItems[$row['uid']]='
-						<option value="'.$row['uid'].'">'.htmlspecialchars($row['title']).'</option>';
+				if ($GLOBALS['BE_USER']->checkLanguageAccess($row['uid']))	{
+					$langSelItems[$row['uid']]='
+							<option value="'.$row['uid'].'">'.htmlspecialchars($row['title']).'</option>';
+				}
 			}
 
 				// Then, subtract the languages which are already on the page:
@@ -1922,7 +1950,7 @@ class tx_cms_layout extends recordList {
 
 				// If any languages are left, make selector:
 			if (count($langSelItems)>1)		{
-				$onChangeContent = 'document.location=\''.$this->backPath.'alt_doc.php?&edit[pages_language_overlay]['.$id.']=new&overrideVals[pages_language_overlay][sys_language_uid]=\'+this.options[this.selectedIndex].value+\'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI')).'\'';
+				$onChangeContent = 'window.location.href=\''.$this->backPath.'alt_doc.php?&edit[pages_language_overlay]['.$id.']=new&overrideVals[pages_language_overlay][sys_language_uid]=\'+this.options[this.selectedIndex].value+\'&returnUrl='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI')).'\'';
 				return $GLOBALS['LANG']->getLL('new_language',1).': <select name="createNewLanguage" onchange="'.htmlspecialchars($onChangeContent).'">
 						'.implode('',$langSelItems).'
 					</select><br /><br />';
@@ -2092,7 +2120,7 @@ class tx_cms_layout extends recordList {
 
 		$c=0;
 		if ($TCA[$table])	{
-			$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery('count(*)', $table, 'pid='.intval($pid).t3lib_BEfunc::deleteClause($table));
+			$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery('count(*)', $table, 'pid='.intval($pid).t3lib_BEfunc::deleteClause($table).t3lib_BEfunc::versioningPlaceholderClause($table));
 			if ($row = $GLOBALS['TYPO3_DB']->sql_fetch_row($result))	{
 				$c=$row[0];
 			}
@@ -2107,7 +2135,7 @@ class tx_cms_layout extends recordList {
 	 * @return	string		Output string
 	 */
 	function renderText($input)	{
-		$input = strip_tags($input);
+		$input = $this->strip_tags($input, true);
 		$input = t3lib_div::fixed_lgd_cs($input,1500);
 		return nl2br(htmlspecialchars(trim($this->wordWrapper($input))));
 	}
@@ -2247,6 +2275,7 @@ class tx_cms_layout extends recordList {
 		$specConf = $this->getSpecConfForField($table,$row,$field);
 		$p = t3lib_BEfunc::getSpecConfParametersFromArray($specConf['rte_transform']['parameters']);
 		if (isset($specConf['richtext']) && (!$p['flag'] || !$row[$p['flag']]))	{
+			t3lib_BEfunc::fixVersioningPid($table,$row);
 			list($tscPID,$thePidValue) = t3lib_BEfunc::getTSCpid($table,$row['uid'],$row['pid']);
 			if ($thePidValue>=0)	{	// If the pid-value is not negative (that is, a pid could NOT be fetched)
 				$RTEsetup = $GLOBALS['BE_USER']->getTSConfig('RTE',t3lib_BEfunc::getPagesTSconfig($tscPID));
@@ -2306,13 +2335,13 @@ class tx_cms_layout extends recordList {
 	 * Creates an info-box for the current page (identified by input record).
 	 *
 	 * @param	array		Page record
-	 * @param	boolean		If set, there will be shown an edit icon, linking to editing of the page header.
+	 * @param	boolean		If set, there will be shown an edit icon, linking to editing of the page properties.
 	 * @return	string		HTML for the box.
 	 */
 	function getPageInfoBox($rec,$edit=0)	{
 		global $LANG;
 
-			// If editing of the page header is allowed:
+			// If editing of the page properties is allowed:
 		if ($edit)	{
 			$params='&edit[pages]['.$rec['uid'].']=edit';
 			$editIcon='<a href="#" onclick="'.htmlspecialchars(t3lib_BEfunc::editOnClick($params,$this->backPath)).'">'.
@@ -2497,7 +2526,7 @@ class tx_cms_layout extends recordList {
 			if ($GLOBALS['BE_USER']->check('tables_select',$tName) && (t3lib_extMgm::isLoaded($tName)||t3lib_div::inList('fe_users,tt_content',$tName) || isset($this->externalTables[$tName])))	{
 
 					// Make query to count records from page:
-				$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery('count(*)', $tName, 'pid='.intval($id).t3lib_BEfunc::deleteClause($tName));
+				$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery('count(*)', $tName, 'pid='.intval($id).t3lib_BEfunc::deleteClause($tName).t3lib_BEfunc::versioningPlaceholderClause($tName));
 				list($c) = $GLOBALS['TYPO3_DB']->sql_fetch_row($result);
 
 					// If records were found (or if "tt_content" is the table...):
@@ -2534,6 +2563,23 @@ class tx_cms_layout extends recordList {
 			// Return the content:
 		return $out;
 	}
+
+	/**
+	 * Enhancement for the strip_tags function that provides the feature to fill in empty tags.
+	 * Example <link email@hostname.com></link> is accepted by TYPO3 but would not displayed in the Backend otherwise.
+	 *
+	 * @param	string		Input string
+	 * @param	boolean		If true, empty tags will be filled with the first attribute of the tag before.
+	 * @return	string		Input string with all HTML and PHP tags stripped
+	 */
+	function strip_tags($content, $fillEmptyContent=false)	{
+		if($fillEmptyContent && strstr($content, '><'))	{
+			$content = preg_replace('/(<[^ >]* )([^ >]*)([^>]*>)(<\/[^>]*>)/', '$1$2$3$2$4', $content);
+		}
+		$content = preg_replace('/<br.?\/?>/', chr(10), $content);
+
+		return strip_tags($content);
+	}
 }
 
 
