@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -28,7 +28,7 @@
  * New content elements wizard
  * (Part of the 'cms' extension)
  *
- * $Id: db_new_content_el.php,v 1.12 2004/09/13 22:57:36 typo3 Exp $
+ * $Id: db_new_content_el.php 1421 2006-04-10 09:27:15Z mundaun $
  * Revised for TYPO3 3.6 November/2003 by Kasper Skaarhoj
  * XHTML compatible.
  *
@@ -39,21 +39,22 @@
  *
  *
  *
- *  100: class ext_posMap extends t3lib_positionMap
- *  110:     function wrapRecordTitle($str,$row)
- *  124:     function onClickInsertRecord($row,$vv,$moveUid,$pid,$sys_lang=0)
+ *  101: class ext_posMap extends t3lib_positionMap
+ *  111:     function wrapRecordTitle($str,$row)
+ *  125:     function onClickInsertRecord($row,$vv,$moveUid,$pid,$sys_lang=0)
  *
  *
- *  152: class SC_db_new_content_el
- *  175:     function init()
- *  211:     function main()
- *  358:     function printContent()
+ *  153: class SC_db_new_content_el
+ *  176:     function init()
+ *  212:     function main()
+ *  359:     function printContent()
  *
  *              SECTION: OTHER FUNCTIONS:
- *  387:     function getWizardItems()
- *  397:     function wizardArray()
+ *  388:     function getWizardItems()
+ *  398:     function wizardArray()
+ *  549:     function removeInvalidElements(&$wizardItems)
  *
- * TOTAL FUNCTIONS: 7
+ * TOTAL FUNCTIONS: 8
  * (This index is automatically created/updated by the extension "extdeveval")
  *
  */
@@ -126,7 +127,7 @@ class ext_posMap extends t3lib_positionMap {
 
 		$location=$this->backPath.'alt_doc.php?edit[tt_content]['.(is_array($row)?-$row['uid']:$pid).']=new&defVals[tt_content][colPos]='.$vv.'&defVals[tt_content][sys_language_uid]='.$sys_lang.'&returnUrl='.rawurlencode($GLOBALS['SOBE']->R_URI);
 
-		return 'document.location=\''.$location.'\'+document.editForm.defValues.value; return false;';
+		return 'window.location.href=\''.$location.'\'+document.editForm.defValues.value; return false;';
 	}
 }
 
@@ -245,7 +246,7 @@ class SC_db_new_content_el {
 			$this->content.=$this->doc->header($LANG->getLL('newContentElement'));
 			$this->content.=$this->doc->spacer(5);
 
-			$elRow = t3lib_BEfunc::getRecord('pages',$this->id);
+			$elRow = t3lib_BEfunc::getRecordWSOL('pages',$this->id);
 			$hline = t3lib_iconWorks::getIconImage('pages',$elRow,$BACK_PATH,' title="'.htmlspecialchars(t3lib_BEfunc::getRecordIconAltText($elRow,'pages')).'" align="top"');
 			$hline.= t3lib_BEfunc::getRecordTitle('pages',$elRow,1);
 			$this->content.=$this->doc->section('',$hline,0,1);
@@ -274,7 +275,7 @@ class SC_db_new_content_el {
 					$tL=array();
 
 						// Radio button:
-					$oC = "document.editForm.defValues.value=unescape('".rawurlencode($wInfo['params'])."');goToalt_doc();".(!$onClickEvent?"document.location='#sel2';":'');
+					$oC = "document.editForm.defValues.value=unescape('".rawurlencode($wInfo['params'])."');goToalt_doc();".(!$onClickEvent?"window.location.hash='#sel2';":'');
 					$tL[]='<input type="radio" name="tempB" value="'.htmlspecialchars($k).'" onclick="'.htmlspecialchars($this->doc->thisBlur().$oC).'" />';
 
 						// Onclick action for icon/title:
@@ -399,75 +400,90 @@ class SC_db_new_content_el {
 
 		$wizardItems = array(
 			'common' => array('header'=>$LANG->getLL('common')),
-			'common_1' => array(
+			'common_regularText' => array(	// Regular text element
 				'icon'=>'gfx/c_wiz/regular_text.gif',
-				'title'=>$LANG->getLL('common_1_title'),
-				'description'=>$LANG->getLL('common_1_description'),
-				'params'=>'&defVals[tt_content][CType]=text'
+				'title'=>$LANG->getLL('common_regularText_title'),
+				'description'=>$LANG->getLL('common_regularText_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'text'
+				)
 			),
-			'common_2' => array(
-				'icon'=>'gfx/c_wiz/text_image_below.gif',
-				'title'=>$LANG->getLL('common_2_title'),
-				'description'=>$LANG->getLL('common_2_description'),
-				'params'=>'&defVals[tt_content][CType]=textpic&defVals[tt_content][imageorient]=8'
-			),
-			'common_3' => array(
+			'common_textImage' => array(	// Text with image
 				'icon'=>'gfx/c_wiz/text_image_right.gif',
-				'title'=>$LANG->getLL('common_3_title'),
-				'description'=>$LANG->getLL('common_3_description'),
-				'params'=>'&defVals[tt_content][CType]=textpic&defVals[tt_content][imageorient]=17'
+				'title'=>$LANG->getLL('common_textImage_title'),
+				'description'=>$LANG->getLL('common_textImage_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'textpic',
+					'imageorient' => 17
+				)
 			),
-			'common_4' => array(
+			'common_imagesOnly' => array(	// Images only
 				'icon'=>'gfx/c_wiz/images_only.gif',
-				'title'=>$LANG->getLL('common_4_title'),
-				'description'=>$LANG->getLL('common_4_description'),
-				'params'=>'&defVals[tt_content][CType]=image&defVals[tt_content][imagecols]=2'
+				'title'=>$LANG->getLL('common_imagesOnly_title'),
+				'description'=>$LANG->getLL('common_imagesOnly_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'image',
+					'imagecols' => 2
+				)
 			),
-			'common_5' => array(
+			'common_bulletList' => array(	// Bullet list
 				'icon'=>'gfx/c_wiz/bullet_list.gif',
-				'title'=>$LANG->getLL('common_5_title'),
-				'description'=>$LANG->getLL('common_5_description'),
-				'params'=>'&defVals[tt_content][CType]=bullets'
+				'title'=>$LANG->getLL('common_bulletList_title'),
+				'description'=>$LANG->getLL('common_bulletList_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'bullets',
+				)
 			),
-			'common_6' => array(
+			'common_table' => array(	// Table
 				'icon'=>'gfx/c_wiz/table.gif',
-				'title'=>$LANG->getLL('common_6_title'),
-				'description'=>$LANG->getLL('common_6_description'),
-				'params'=>'&defVals[tt_content][CType]=table'
+				'title'=>$LANG->getLL('common_table_title'),
+				'description'=>$LANG->getLL('common_table_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'table',
+				)
 			),
 			'special' => array('header'=>$LANG->getLL('special')),
-			'special_1' => array(
+			'special_filelinks' => array(	// Filelinks
 				'icon'=>'gfx/c_wiz/filelinks.gif',
-				'title'=>$LANG->getLL('special_1_title'),
-				'description'=>$LANG->getLL('special_1_description'),
-				'params'=>'&defVals[tt_content][CType]=uploads'
+				'title'=>$LANG->getLL('special_filelinks_title'),
+				'description'=>$LANG->getLL('special_filelinks_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'uploads',
+				)
 			),
-			'special_2' => array(
+			'special_multimedia' => array(	// Multimedia
 				'icon'=>'gfx/c_wiz/multimedia.gif',
-				'title'=>$LANG->getLL('special_2_title'),
-				'description'=>$LANG->getLL('special_2_description'),
-				'params'=>'&defVals[tt_content][CType]=multimedia'
+				'title'=>$LANG->getLL('special_multimedia_title'),
+				'description'=>$LANG->getLL('special_multimedia_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'multimedia',
+				)
 			),
-			'special_3' => array(
+			'special_sitemap' => array(	// Sitemap
 				'icon'=>'gfx/c_wiz/sitemap2.gif',
-				'title'=>$LANG->getLL('special_3_title'),
-				'description'=>$LANG->getLL('special_3_description'),
-				'params'=>'&defVals[tt_content][CType]=menu&defVals[tt_content][menu_type]=2'
+				'title'=>$LANG->getLL('special_sitemap_title'),
+				'description'=>$LANG->getLL('special_sitemap_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'menu',
+					'menu_type' => 2
+				)
 			),
-			'special_4' => array(
+			'special_plainHTML' => array(	// Plain HTML
 				'icon'=>'gfx/c_wiz/html.gif',
-				'title'=>$LANG->getLL('special_4_title'),
-				'description'=>$LANG->getLL('special_4_description'),
-				'params'=>'&defVals[tt_content][CType]=html'
+				'title'=>$LANG->getLL('special_plainHTML_title'),
+				'description'=>$LANG->getLL('special_plainHTML_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'html',
+				)
 			),
-
-
 			'forms' => array('header'=>$LANG->getLL('forms')),
-			'forms_1' => array(
+			'forms_mail' => array(	// Mail form
 				'icon'=>'gfx/c_wiz/mailform.gif',
-				'title'=>$LANG->getLL('forms_1_title'),
-				'description'=>$LANG->getLL('forms_1_description'),
-				'params'=>'&defVals[tt_content][CType]=mailform&defVals[tt_content][bodytext]='.rawurlencode(trim('
+				'title'=>$LANG->getLL('forms_mail_title'),
+				'description'=>$LANG->getLL('forms_mail_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'mailform',
+					'bodytext' => trim('
 # Example content:
 Name: | *name = input,40 | Enter your name here
 Email: | *email=input,40 |
@@ -477,26 +493,33 @@ Contact me: | tv=check | 1
 |formtype_mail = submit | Send form!
 |html_enabled=hidden | 1
 |subject=hidden| This is the subject
-				'))
+					')
+				)
 			),
-			'forms_2' => array(
+			'forms_search' => array(	// Search form
 				'icon'=>'gfx/c_wiz/searchform.gif',
-				'title'=>$LANG->getLL('forms_2_title'),
-				'description'=>$LANG->getLL('forms_2_description'),
-				'params'=>'&defVals[tt_content][CType]=search'
+				'title'=>$LANG->getLL('forms_search_title'),
+				'description'=>$LANG->getLL('forms_search_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'search',
+				)
 			),
-			'forms_3' => array(
+			'forms_login' => array(	// Login form
 				'icon'=>'gfx/c_wiz/login_form.gif',
-				'title'=>$LANG->getLL('forms_3_title'),
-				'description'=>$LANG->getLL('forms_3_description'),
-				'params'=>'&defVals[tt_content][CType]=login'
+				'title'=>$LANG->getLL('forms_login_title'),
+				'description'=>$LANG->getLL('forms_login_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'login',
+				)
 			),
 			'plugins' => array('header'=>$LANG->getLL('plugins')),
-			'plugins_1' => array(
+			'plugins_general' => array(	// General plugin
 				'icon'=>'gfx/c_wiz/user_defined.gif',
-				'title'=>$LANG->getLL('plugins_1_title'),
-				'description'=>$LANG->getLL('plugins_1_description'),
-				'params'=>'&defVals[tt_content][CType]=list'
+				'title'=>$LANG->getLL('plugins_general_title'),
+				'description'=>$LANG->getLL('plugins_general_description'),
+				'tt_content_defValues' => array(
+					'CType' => 'list',
+				)
 			),
 		);
 
@@ -510,8 +533,78 @@ Contact me: | tv=check | 1
 			}
 		}
 
+			// Remove elements where preset values are not allowed:
+		$this->removeInvalidElements($wizardItems);
+
 		return $wizardItems;
 	}
+
+	/**
+	 * Checks the array for elements which might contain unallowed default values and will unset them!
+	 * Looks for the "tt_content_defValues" key in each element and if found it will traverse that array as fieldname / value pairs and check. The values will be added to the "params" key of the array (which should probably be unset or empty by default).
+	 *
+	 * @param	array		Wizard items, passed by reference
+	 * @return	void
+	 */
+	function removeInvalidElements(&$wizardItems)	{
+		global $TCA;
+
+			// Load full table definition:
+		t3lib_div::loadTCA('tt_content');
+
+			// Get TCEFORM from TSconfig of current page
+		$row = array('pid'=>$this->id);
+		$TCEFORM_TSconfig = t3lib_BEfunc::getTCEFORM_TSconfig('tt_content',$row);
+		$removeItems = t3lib_div::trimExplode(',',$TCEFORM_TSconfig['CType']['removeItems'],1);
+		
+		$headersUsed = Array();
+			// Traverse wizard items:
+		foreach($wizardItems as $key => $cfg)	{
+
+				// Exploding parameter string, if any (old style)
+			if ($wizardItems[$key]['params'])	{
+					// Explode GET vars recursively
+				$tempGetVars = t3lib_div::explodeUrl2Array($wizardItems[$key]['params'],TRUE);
+					// If tt_content values are set, merge them into the tt_content_defValues array, unset them from $tempGetVars and re-implode $tempGetVars into the param string (in case remaining parameters are around).
+				if (is_array($tempGetVars['defVals']['tt_content']))	{
+					$wizardItems[$key]['tt_content_defValues'] = array_merge(is_array($wizardItems[$key]['tt_content_defValues']) ? $wizardItems[$key]['tt_content_defValues'] : array(), $tempGetVars['defVals']['tt_content']);
+					unset($tempGetVars['defVals']['tt_content']);
+					$wizardItems[$key]['params'] = t3lib_div::implodeArrayForUrl('',$tempGetVars);
+				}
+			}
+
+				// If tt_content_defValues are defined...:
+			if (is_array($wizardItems[$key]['tt_content_defValues']))	{
+
+					// Traverse field values:
+				foreach($wizardItems[$key]['tt_content_defValues'] as $fN => $fV)	{
+					if (is_array($TCA['tt_content']['columns'][$fN]))	{
+							// Get information about if the field value is OK:
+						$config = &$TCA['tt_content']['columns'][$fN]['config'];
+						$authModeDeny = $config['type']=='select' && $config['authMode'] && !$GLOBALS['BE_USER']->checkAuthMode('tt_content',$fN,$fV,$config['authMode']);
+
+						if ($authModeDeny || in_array($fV,$removeItems))	{
+								// Remove element all together:
+							unset($wizardItems[$key]);
+							break;
+						} else {
+								// Add the parameter:
+							$wizardItems[$key]['params'].= '&defVals[tt_content]['.$fN.']='.rawurlencode($fV);
+							$tmp = explode('_', $key);
+							$headersUsed[$tmp[0]] = $tmp[0];
+						}
+					}
+				}
+			}
+		}
+			// remove headers without elements
+		foreach ($wizardItems as $key => $cfg)	{
+			$tmp = explode('_',$key);
+			if ($tmp[0] && !$tmp[1] && !in_array($tmp[0], $headersUsed))	{
+				unset($wizardItems[$key]);
+			}
+		}
+	}
 }
 
 // Include extension?
