@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -28,7 +28,7 @@
  * Libraries for pagegen.php
  * The script "pagegen.php" is included by "index_ts.php" when a page is not cached but needs to be rendered.
  *
- * $Id: class.tslib_pagegen.php,v 1.24.2.1 2004/09/19 10:31:16 ingmars Exp $
+ * $Id: class.tslib_pagegen.php 1657 2006-07-26 17:54:15Z mundaun $
  * Revised for TYPO3 3.6 June/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -41,16 +41,16 @@
  *
  *   88: class TSpagegen
  *   95:     function pagegenInit()
- *  225:     function getIncFiles()
- *  258:     function JSeventFunctions()
- *  292:     function renderContent()
- *  319:     function renderContentWithHeader($pageContent)
+ *  271:     function getIncFiles()
+ *  304:     function JSeventFunctions()
+ *  338:     function renderContent()
+ *  365:     function renderContentWithHeader($pageContent)
  *
  *              SECTION: Helper functions
- *  754:     function inline2TempFile($str,$ext)
+ *  827:     function inline2TempFile($str,$ext)
  *
  *
- *  808: class FE_loadDBGroup extends t3lib_loadDBGroup
+ *  881: class FE_loadDBGroup extends t3lib_loadDBGroup
  *
  * TOTAL FUNCTIONS: 6
  * (This index is automatically created/updated by the extension "extdeveval")
@@ -94,7 +94,7 @@ class TSpagegen {
 	 */
 	function pagegenInit()	{
 		if ($GLOBALS['TSFE']->page['content_from_pid']>0)	{
-			$temp_copy_TSFE = $GLOBALS['TSFE'];	// make REAL copy of TSFE object - not reference!
+			$temp_copy_TSFE = clone($GLOBALS['TSFE']);	// make REAL copy of TSFE object - not reference!
 			$temp_copy_TSFE->id = $GLOBALS['TSFE']->page['content_from_pid'];	// Set ->id to the content_from_pid value - we are going to evaluate this pid as was it a given id for a page-display!
 			$temp_copy_TSFE->getPageAndRootlineWithDomain($GLOBALS['TSFE']->config['config']['content_from_pid_allowOutsideDomain']?0:$GLOBALS['TSFE']->domainStartPage);
 			$GLOBALS['TSFE']->contentPid = intval($temp_copy_TSFE->id);
@@ -125,33 +125,67 @@ class TSpagegen {
 		$GLOBALS['TSFE']->debug = ''.$GLOBALS['TSFE']->config['config']['debug'];
 
 			// Base url:
-		if ($GLOBALS['TSFE']->config['config']['baseURL']) {
-			$GLOBALS['TSFE']->baseUrl = (intval($GLOBALS['TSFE']->config['config']['baseURL']) ? t3lib_div::getIndpEnv('TYPO3_SITE_URL') : $GLOBALS['TSFE']->config['config']['baseURL']);
+		if ($GLOBALS['TSFE']->config['config']['baseURL'])	{
+			if ($GLOBALS['TSFE']->config['config']['baseURL']==='1')	{
+					// Deprecated property, going to be dropped.
+				$error = 'Unsupported TypoScript property was found in this template: "config.baseURL="1"
+
+This setting has been deprecated in TYPO 3.8.1 due to security concerns.
+You need to change this value to the URL of your website root, otherwise TYPO3 will not work!
+
+See <a href="http://wiki.typo3.org/index.php/TYPO3_3.8.1" target="_blank">wiki.typo3.org/index.php/TYPO3_3.8.1</a> for more information.';
+
+				$GLOBALS['TSFE']->printError(nl2br($error));
+				exit;
+			} else {
+				$GLOBALS['TSFE']->baseUrl = $GLOBALS['TSFE']->config['config']['baseURL'];
+			}
 			$GLOBALS['TSFE']->anchorPrefix = substr(t3lib_div::getIndpEnv('TYPO3_REQUEST_URL'),strlen(t3lib_div::getIndpEnv('TYPO3_SITE_URL')));
 		}
 
 			// Internal and External target defaults
 		$GLOBALS['TSFE']->intTarget = ''.$GLOBALS['TSFE']->config['config']['intTarget'];
 		$GLOBALS['TSFE']->extTarget = ''.$GLOBALS['TSFE']->config['config']['extTarget'];
-		$GLOBALS['TSFE']->spamProtectEmailAddresses = t3lib_div::intInRange($GLOBALS['TSFE']->config['config']['spamProtectEmailAddresses'],-5,5,0);
-		if ($GLOBALS['TSFE']->spamProtectEmailAddresses)	{
-			$GLOBALS['TSFE']->additionalJavaScript['UnCryptMailto()']='
-  // JS function for uncrypting spam-protected emails:
-function UnCryptMailto(s) {	//
-	var n=0;
-	var r="";
-	for(var i=0; i < s.length; i++) {
-		n=s.charCodeAt(i);
-		if (n>=8364) {n = 128;}
-		r += String.fromCharCode(n-('.$GLOBALS['TSFE']->spamProtectEmailAddresses.'));
+		if ($GLOBALS['TSFE']->config['config']['spamProtectEmailAddresses'] === 'ascii') {
+			$GLOBALS['TSFE']->spamProtectEmailAddresses = 'ascii';
+		} else {
+			$GLOBALS['TSFE']->spamProtectEmailAddresses = t3lib_div::intInRange($GLOBALS['TSFE']->config['config']['spamProtectEmailAddresses'],-10,10,0);
+			if ($GLOBALS['TSFE']->spamProtectEmailAddresses)	{
+				$GLOBALS['TSFE']->additionalJavaScript['UnCryptMailto()']='
+  // decrypt helper function
+function decryptCharcode(n,start,end,offset) {
+	n = n + offset;
+	if (offset > 0 && n > end)	{
+		n = start + (n - end - 1);
+	} else if (offset < 0 && n < start)	{
+		n = end - (start - n - 1);
 	}
-	return r;
+	return String.fromCharCode(n);
 }
-  // JS function for uncrypting spam-protected emails:
-function linkTo_UnCryptMailto(s)	{	//
-	location.href=UnCryptMailto(s);
+  // decrypt string
+function decryptString(enc,offset) {
+	var dec = "";
+	var len = enc.length;
+	for(var i=0; i < len; i++)	{
+		var n = enc.charCodeAt(i);
+		if (n >= 0x2B && n <= 0x39)	{
+			dec += decryptCharcode(n,0x2B,0x3A,offset);	// 0-9 . , - + / :
+		} else if (n >= 0x40 && n <= 0x5A)	{
+			dec += decryptCharcode(n,0x40,0x5A,offset);	// A-Z @
+		} else if (n >= 0x61 && n <= 0x7A)	{
+			dec += decryptCharcode(n,0x61,0x7A,offset);	// a-z
+		} else {
+			dec += enc.charAt(i);
+		}
+	}
+	return dec;
 }
-		';
+  // decrypt spam-protected emails
+function linkTo_UnCryptMailto(s)	{
+	location.href = decryptString(s,'.($GLOBALS['TSFE']->spamProtectEmailAddresses*-1).');
+}
+'				;
+			}
 		}
 
 
@@ -163,7 +197,7 @@ function linkTo_UnCryptMailto(s)	{	//
 
 		if ($GLOBALS['TSFE']->type && $GLOBALS['TSFE']->config['config']['frameReloadIfNotInFrameset'])	{
 			$tdlLD = $GLOBALS['TSFE']->tmpl->linkData($GLOBALS['TSFE']->page,'_top',$GLOBALS['TSFE']->no_cache,'');
-			$GLOBALS['TSFE']->JSCode = 'if(!parent.'.trim($GLOBALS['TSFE']->sPre).' && !parent.view_frame) top.document.location="'.$GLOBALS['TSFE']->baseUrlWrap($tdlLD['totalURL']).'"';
+			$GLOBALS['TSFE']->JSCode = 'if(!parent.'.trim($GLOBALS['TSFE']->sPre).' && !parent.view_frame) top.location.href="'.$GLOBALS['TSFE']->baseUrlWrap($tdlLD['totalURL']).'"';
 		}
 		$GLOBALS['TSFE']->compensateFieldWidth = ''.$GLOBALS['TSFE']->config['config']['compensateFieldWidth'];
 		$GLOBALS['TSFE']->lockFilePath = ''.$GLOBALS['TSFE']->config['config']['lockFilePath'];
@@ -215,6 +249,36 @@ function linkTo_UnCryptMailto(s)	{	//
 		} else {
 			$GLOBALS['TSFE']->linkVars='';
 		}
+
+			// Setting XHTML-doctype from doctype
+		if (!$GLOBALS['TSFE']->config['config']['xhtmlDoctype'])	{
+			$GLOBALS['TSFE']->config['config']['xhtmlDoctype'] = $GLOBALS['TSFE']->config['config']['doctype'];
+		}
+
+		if ($GLOBALS['TSFE']->config['config']['xhtmlDoctype'])	{
+			$GLOBALS['TSFE']->xhtmlDoctype = $GLOBALS['TSFE']->config['config']['xhtmlDoctype'];
+
+				// Checking XHTML-docytpe
+			switch((string)$GLOBALS['TSFE']->config['config']['xhtmlDoctype'])	{
+				case 'xhtml_trans':
+				case 'xhtml_strict':
+				case 'xhtml_frames':
+					$GLOBALS['TSFE']->xhtmlVersion = 100;
+				break;
+				case 'xhtml_basic':
+					$GLOBALS['TSFE']->xhtmlVersion = 105;
+				break;
+				case 'xhtml_11':
+					$GLOBALS['TSFE']->xhtmlVersion = 110;
+				break;
+				case 'xhtml_2':
+					$GLOBALS['TSFE']->xhtmlVersion = 200;
+				break;
+				default:
+					$GLOBALS['TSFE']->xhtmlDoctype = '';
+					$GLOBALS['TSFE']->xhtmlVersion = 0;
+			}
+		}
 	}
 
 	/**
@@ -337,37 +401,51 @@ function linkTo_UnCryptMailto(s)	{	//
 
 			// Setting document type:
 		$docTypeParts = array();
-		$XMLprologue = $GLOBALS['TSFE']->config['config']['xmlprologue'] != 'none';
+			// Part 1: XML prologue
+		switch((string)$GLOBALS['TSFE']->config['config']['xmlprologue'])	{
+			case 'none':
+			break;
+			case 'xml_10':
+				$docTypeParts[]='<?xml version="1.0" encoding="'.$theCharset.'"?>';
+			break;
+			case 'xml_11':
+				$docTypeParts[]='<?xml version="1.1" encoding="'.$theCharset.'"?>';
+			break;
+			case '':
+				if ($GLOBALS['TSFE']->xhtmlVersion)	$docTypeParts[]='<?xml version="1.0" encoding="'.$theCharset.'"?>';
+			break;
+			default:
+				$docTypeParts[]=$GLOBALS['TSFE']->config['config']['xmlprologue'];
+		}
+			// Part 2: DTD
 		if ($GLOBALS['TSFE']->config['config']['doctype'])	{
-
-				// Setting doctypes:
 			switch((string)$GLOBALS['TSFE']->config['config']['doctype'])	{
 				case 'xhtml_trans':
-		 			if ($XMLprologue) $docTypeParts[]='<?xml version="1.0" encoding="'.$theCharset.'"?>';
 					$docTypeParts[]='<!DOCTYPE html
      PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">';
 				break;
 				case 'xhtml_strict':
-		 			if ($XMLprologue) $docTypeParts[]='<?xml version="1.0" encoding="'.$theCharset.'"?>';
 					$docTypeParts[]='<!DOCTYPE html
      PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
 				break;
 				case 'xhtml_frames':
-		 			if ($XMLprologue) $docTypeParts[]='<?xml version="1.0" encoding="'.$theCharset.'"?>';
 					$docTypeParts[]='<!DOCTYPE html
      PUBLIC "-//W3C//DTD XHTML 1.0 Frameset//EN"
      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd">';
+				break;
+				case 'xhtml_basic':
+					$docTypeParts[]='<!DOCTYPE html
+    PUBLIC "-//W3C//DTD XHTML Basic 1.0//EN"
+    "http://www.w3.org/TR/xhtml-basic/xhtml-basic10.dtd">';
 				break;
 				case 'xhtml_11':
-		 			if ($XMLprologue) $docTypeParts[]='<?xml version="1.1" encoding="'.$theCharset.'"?>';
 					$docTypeParts[]='<!DOCTYPE html
      PUBLIC "-//W3C//DTD XHTML 1.1//EN"
      "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">';
 				break;
 				case 'xhtml_2':
-		 			if ($XMLprologue) $docTypeParts[]='<?xml version="2.0" encoding="'.$theCharset.'"?>';
 					$docTypeParts[]='<!DOCTYPE html
 	PUBLIC "-//W3C//DTD XHTML 2.0//EN"
 	"http://www.w3.org/TR/xhtml2/DTD/xhtml2.dtd">';
@@ -376,28 +454,22 @@ function linkTo_UnCryptMailto(s)	{	//
 				break;
 				default:
 					$docTypeParts[] = $GLOBALS['TSFE']->config['config']['doctype'];
-				break;
-			}
-				// Setting <html> tag attributes:
-			switch((string)$GLOBALS['TSFE']->config['config']['doctype'])	{
-				case 'xhtml_trans':
-				case 'xhtml_strict':
-				case 'xhtml_frames':
-	 				$htmlTagAttributes['xmlns'] = 'http://www.w3.org/1999/xhtml';
-					$htmlTagAttributes['xml:lang'] = $htmlLang;
-					$htmlTagAttributes['lang'] = $htmlLang;
-				break;
-				case 'xhtml_11':
-				case 'xhtml_2':
-	 				$htmlTagAttributes['xmlns'] = 'http://www.w3.org/1999/xhtml';
-					$htmlTagAttributes['xml:lang'] = $htmlLang;
-				break;
 			}
 		} else {
 			$docTypeParts[]='<!DOCTYPE html
 	PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">';
 		}
 
+		if ($GLOBALS['TSFE']->xhtmlVersion)	{
+
+				// Setting <html> tag attributes:
+	 		$htmlTagAttributes['xmlns'] = 'http://www.w3.org/1999/xhtml';
+			$htmlTagAttributes['xml:lang'] = $htmlLang;
+			if ($GLOBALS['TSFE']->xhtmlVersion < 110)	{
+					$htmlTagAttributes['lang'] = $htmlLang;
+			}
+		}
+
 			// Swap XML and doctype order around (for MSIE / Opera standards compliance)
 		if ($GLOBALS['TSFE']->config['config']['doctypeSwitch'])	{
 			$docTypeParts = array_reverse($docTypeParts);
@@ -412,8 +484,12 @@ function linkTo_UnCryptMailto(s)	{	//
 		} else {
 			$_attr = '';
 		}
-		$GLOBALS['TSFE']->content.='<html'.($_attr ? ' '.$_attr : '').'>
-<head>';
+		$GLOBALS['TSFE']->content.='<html'.($_attr ? ' '.$_attr : '').'>';
+
+			// Head tag:
+		$headTag = $GLOBALS['TSFE']->pSetup['headTag'] ? $GLOBALS['TSFE']->pSetup['headTag'] : '<head>';
+		$GLOBALS['TSFE']->content.= chr(10).$headTag;
+
 			// Setting charset meta tag:
 		$GLOBALS['TSFE']->content.='
 	<meta http-equiv="Content-Type" content="text/html; charset='.$theCharset.'" />';
@@ -421,10 +497,10 @@ function linkTo_UnCryptMailto(s)	{	//
 $GLOBALS['TSFE']->content.='
 
 <!-- '.($customContent?$customContent.chr(10):'').'
-	This website is brought to you by TYPO3 - get.content.right
-	TYPO3 is a free open source Content Management Framework created by Kasper Skaarhoj and licensed under GNU/GPL.
-	TYPO3 is copyright 1998-2004 of Kasper Skaarhoj. Extensions are copyright of their respective owners.
-	Information and contribution at http://www.typo3.com
+	This website is powered by TYPO3 - inspiring people to share!
+	TYPO3 is a free open source Content Management Framework initially created by Kasper Skaarhoj and licensed under GNU/GPL.
+	TYPO3 is copyright 1998-2006 of Kasper Skaarhoj. Extensions are copyright of their respective owners.
+	Information and contribution at http://typo3.com/ and http://typo3.org/
 -->
 ';
 
@@ -440,13 +516,12 @@ $GLOBALS['TSFE']->content.='
 	<link rel="SHORTCUT ICON" href="'.htmlspecialchars($ss).'" />';
 		}
 
-		/** CSS STYLESHEET handling: */
-		if (is_array($GLOBALS['TSFE']->tmpl->setup['plugin.'])) {
+			// Including CSS files
+		if (is_array($GLOBALS['TSFE']->tmpl->setup['plugin.']))	{
 			$temp_styleLines=array();
-			reset($GLOBALS['TSFE']->tmpl->setup['plugin.']);
-			while(list($k2,$iCSScode)=each($GLOBALS['TSFE']->tmpl->setup['plugin.']))	{
+			foreach ($GLOBALS['TSFE']->tmpl->setup['plugin.'] as $key=>$iCSScode)	{
 				if (is_array($iCSScode) && $iCSScode['_CSS_DEFAULT_STYLE'])	{
-					$temp_styleLines[]='/* default styles for extension "'.substr($k2,0,-1).'" */'.chr(10).$iCSScode['_CSS_DEFAULT_STYLE'];
+					$temp_styleLines[]='/* default styles for extension "'.substr($key,0,-1).'" */'.chr(10).$iCSScode['_CSS_DEFAULT_STYLE'];
 				}
 			}
 			if (count($temp_styleLines))	{
@@ -465,35 +540,34 @@ $GLOBALS['TSFE']->content.='
 			}
 		}
 
-		if ($GLOBALS['TSFE']->pSetup['stylesheet']) {
+		if ($GLOBALS['TSFE']->pSetup['stylesheet'])	{
 			$ss=$GLOBALS['TSFE']->tmpl->getFileName($GLOBALS['TSFE']->pSetup['stylesheet']);
-			if ($ss) {
+			if ($ss)	{
 				$GLOBALS['TSFE']->content.='
 	<link rel="stylesheet" type="text/css" href="'.htmlspecialchars($ss).'" />';
 			}
 		}
-		if (is_array($GLOBALS['TSFE']->pSetup['includeCSS.'])) {
-			reset($GLOBALS['TSFE']->pSetup['includeCSS.']);
-			while(list($k2,$iCSSfile)=each($GLOBALS['TSFE']->pSetup['includeCSS.']))	{
+		if (is_array($GLOBALS['TSFE']->pSetup['includeCSS.']))	{
+			foreach ($GLOBALS['TSFE']->pSetup['includeCSS.'] as $key=>$iCSSfile)	{
 				if (!is_array($iCSSfile))	{
-					$ss = $GLOBALS['TSFE']->tmpl->getFileName($iCSSfile);
-					if ($ss) {
-						if ($GLOBALS['TSFE']->pSetup['includeCSS.'][$k2.'.']['import'])	{
+					$ss=$GLOBALS['TSFE']->tmpl->getFileName($iCSSfile);
+					if ($ss)	{
+						if ($GLOBALS['TSFE']->pSetup['includeCSS.'][$key.'.']['import'])	{
 							if (substr($ss,0,1)!='/')	{	// To fix MSIE 6 that cannot handle these as relative paths (according to Ben v Ende)
 								$ss = t3lib_div::dirname(t3lib_div::getIndpEnv('SCRIPT_NAME')).'/'.$ss;
 							}
 							$GLOBALS['TSFE']->content.='
 	<style type="text/css">
 	<!--
-	@import url("'.htmlspecialchars($ss).'") '.htmlspecialchars($GLOBALS['TSFE']->pSetup['includeCSS.'][$k2.'.']['media']).';
+	@import url("'.htmlspecialchars($ss).'") '.htmlspecialchars($GLOBALS['TSFE']->pSetup['includeCSS.'][$key.'.']['media']).';
 	-->
 	</style>
 							';
 						} else {
 							$GLOBALS['TSFE']->content.='
-	<link rel="'.($GLOBALS['TSFE']->pSetup['includeCSS.'][$k2.'.']['alternate'] ? 'alternate stylesheet' : 'stylesheet').'" type="text/css" href="'.htmlspecialchars($ss).'"'.
-			($GLOBALS['TSFE']->pSetup['includeCSS.'][$k2.'.']['title'] ? ' title="'.htmlspecialchars($GLOBALS['TSFE']->pSetup['includeCSS.'][$k2.'.']['title']).'"' : '').
-			($GLOBALS['TSFE']->pSetup['includeCSS.'][$k2.'.']['media'] ? ' media="'.htmlspecialchars($GLOBALS['TSFE']->pSetup['includeCSS.'][$k2.'.']['media']).'"' : '').
+	<link rel="'.($GLOBALS['TSFE']->pSetup['includeCSS.'][$key.'.']['alternate'] ? 'alternate stylesheet' : 'stylesheet').'" type="text/css" href="'.htmlspecialchars($ss).'"'.
+			($GLOBALS['TSFE']->pSetup['includeCSS.'][$key.'.']['title'] ? ' title="'.htmlspecialchars($GLOBALS['TSFE']->pSetup['includeCSS.'][$key.'.']['title']).'"' : '').
+			($GLOBALS['TSFE']->pSetup['includeCSS.'][$key.'.']['media'] ? ' media="'.htmlspecialchars($GLOBALS['TSFE']->pSetup['includeCSS.'][$key.'.']['media']).'"' : '').
 			' />';
 						}
 					}
@@ -501,7 +575,7 @@ $GLOBALS['TSFE']->content.='
 			}
 		}
 
-		// Stylesheets
+			// Stylesheets
 		$style='';
 		$style.=trim($GLOBALS['TSFE']->pSetup['CSS_inlineStyle']).chr(10);
 
@@ -509,8 +583,7 @@ $GLOBALS['TSFE']->content.='
 			$pageTSConfig = $GLOBALS['TSFE']->getPagesTSconfig();
 			$RTEclasses = $pageTSConfig['RTE.']['classes.'];
 			if (is_array($RTEclasses))	{
-				reset($RTEclasses);
-				while(list($RTEclassName,$RTEvalueArray)=each($RTEclasses))	{
+				foreach ($RTEclasses as $RTEclassName=>$RTEvalueArray)	{
 					if ($RTEvalueArray['value'])	{
 						$style.='
 .'.substr($RTEclassName,0,-1).' {'.$RTEvalueArray['value'].'}';
@@ -520,8 +593,7 @@ $GLOBALS['TSFE']->content.='
 
 			if ($GLOBALS['TSFE']->pSetup['insertClassesFromRTE.']['add_mainStyleOverrideDefs'] && is_array($pageTSConfig['RTE.']['default.']['mainStyleOverride_add.']))	{
 				$mSOa_tList = t3lib_div::trimExplode(',',strtoupper($GLOBALS['TSFE']->pSetup['insertClassesFromRTE.']['add_mainStyleOverrideDefs']),1);
-				reset($pageTSConfig['RTE.']['default.']['mainStyleOverride_add.']);
-				while(list($mSOa_key,$mSOa_value)=each($pageTSConfig['RTE.']['default.']['mainStyleOverride_add.']))	{
+				foreach ($pageTSConfig['RTE.']['default.']['mainStyleOverride_add.'] as $mSOa_key=>$mSOa_value)	{
 					if (!is_array($mSOa_value) && (in_array('*',$mSOa_tList)||in_array($mSOa_key,$mSOa_tList)))	{
 						$style.='
 '.$mSOa_key.' {'.$mSOa_value.'}';
@@ -586,16 +658,32 @@ $GLOBALS['TSFE']->content.='
 			}
 		}
 
+			// JavaScript files
+		if (is_array($GLOBALS['TSFE']->pSetup['includeJS.']))	{
+			foreach ($GLOBALS['TSFE']->pSetup['includeJS.'] as $key=>$JSfile)	{
+				if (!is_array($JSfile))	{
+					$ss = $GLOBALS['TSFE']->tmpl->getFileName($JSfile);
+					if ($ss)	{
+						$type = $GLOBALS['TSFE']->pSetup['includeJS.'][$key.'.']['type'];
+						if (!$type)	$type = 'text/javascript';
+
+						$GLOBALS['TSFE']->content.='
+	<script src="'.htmlspecialchars($ss).'" type="'.htmlspecialchars($type).'"></script>';
+					}
+				}
+			}
+		}
 
 
 
 
-		// Headerdata
+
+			// Headerdata
 		if (is_array($GLOBALS['TSFE']->pSetup['headerData.']))	{
 			$GLOBALS['TSFE']->content.= chr(10).$GLOBALS['TSFE']->cObj->cObjGet($GLOBALS['TSFE']->pSetup['headerData.'],'headerData.');
 		}
 
-			// <title></title> :
+			// Title
 		$titleTagContent = $GLOBALS['TSFE']->tmpl->printTitle(
 			$GLOBALS['TSFE']->altPageTitle?$GLOBALS['TSFE']->altPageTitle:$GLOBALS['TSFE']->page['title'],
 			$GLOBALS['TSFE']->config['config']['noPageTitle'],
@@ -605,10 +693,12 @@ $GLOBALS['TSFE']->content.='
 			$titleTagContent = $GLOBALS['TSFE']->cObj->callUserFunction($GLOBALS['TSFE']->config['config']['titleTagFunction'], array(), $titleTagContent);
 		}
 
-		$GLOBALS['TSFE']->content.='
+		if (strlen($titleTagContent) && intval($GLOBALS['TSFE']->config['config']['noPageTitle'])!==2)	{
+			$GLOBALS['TSFE']->content.='
 	<title>'.htmlspecialchars($titleTagContent).'</title>';
+		}
 		$GLOBALS['TSFE']->content.='
-	<meta name="generator" content="TYPO3 3.7 CMS" />';
+	<meta name="generator" content="TYPO3 '.TYPO3_branch.' CMS" />';
 
 		$conf=$GLOBALS['TSFE']->pSetup['meta.'];
 		if (is_array($conf))	{
@@ -651,10 +741,11 @@ $GLOBALS['TSFE']->content.='
 
 			// Adding default Java Script:
 		$_scriptCode = '
-		browserName = navigator.appName;
-		browserVer = parseInt(navigator.appVersion);
+		var browserName = navigator.appName;
+		var browserVer = parseInt(navigator.appVersion);
+		var version = "";
 		var msie4 = (browserName == "Microsoft Internet Explorer" && browserVer >= 4);
-		if ((browserName == "Netscape" && browserVer >= 3) || msie4 || browserName=="Konqueror") {version = "n3";} else {version = "n2";}
+		if ((browserName == "Netscape" && browserVer >= 3) || msie4 || browserName=="Konqueror" || browserName=="Opera") {version = "n3";} else {version = "n2";}
 			// Blurring links:
 		function blurLink(theObject)	{	//
 			if (msie4)	{theObject.blur();}
@@ -711,11 +802,11 @@ $GLOBALS['TSFE']->content.='
 
 		// Div-sections
 		if ($GLOBALS['TSFE']->divSection)	{
-			$GLOBALS['TSFE']->content.=	chr(10).$GLOBALS['TSFE']->divSection;
+			$GLOBALS['TSFE']->content.= chr(10).$GLOBALS['TSFE']->divSection;
 		}
 
 		// Page content
-		$GLOBALS['TSFE']->content.=chr(10).$pageContent;
+		$GLOBALS['TSFE']->content.= chr(10).$pageContent;
 
 		// Ending page
 		$GLOBALS['TSFE']->content.= chr(10).'</body>';
