@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * This script contains the parent class, 'pibase', providing an API with the most basic methods for frontend plugins
  *
- * $Id: class.tslib_pibase.php,v 1.16 2004/09/13 22:57:36 typo3 Exp $
+ * $Id: class.tslib_pibase.php 1421 2006-04-10 09:27:15Z mundaun $
  * Revised for TYPO3 3.6 June/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -41,57 +41,57 @@
  *  132: class tslib_pibase
  *
  *              SECTION: Init functions
- *  211:     function tslib_pibase()
- *  225:     function pi_setPiVarDefaults()
+ *  214:     function tslib_pibase()
+ *  240:     function pi_setPiVarDefaults()
  *
  *              SECTION: Link functions
- *  264:     function pi_getPageLink($id,$target='',$urlParameters=array())
- *  280:     function pi_linkToPage($str,$id,$target='',$urlParameters=array())
- *  295:     function pi_linkTP($str,$urlParameters=array(),$cache=0,$altPageId=0)
- *  318:     function pi_linkTP_keepPIvars($str,$overrulePIvars=array(),$cache=0,$clearAnyway=0,$altPageId=0)
- *  342:     function pi_linkTP_keepPIvars_url($overrulePIvars=array(),$cache=0,$clearAnyway=0,$altPageId=0)
- *  360:     function pi_list_linkSingle($str,$uid,$cache=FALSE,$mergeArr=array(),$urlOnly=FALSE,$altPageId=0)
- *  388:     function pi_openAtagHrefInJSwindow($str,$winName='',$winParams='width=670,height=500,status=0,menubar=0,scrollbars=1,resizable=1')
+ *  277:     function pi_getPageLink($id,$target='',$urlParameters=array())
+ *  293:     function pi_linkToPage($str,$id,$target='',$urlParameters=array())
+ *  308:     function pi_linkTP($str,$urlParameters=array(),$cache=0,$altPageId=0)
+ *  331:     function pi_linkTP_keepPIvars($str,$overrulePIvars=array(),$cache=0,$clearAnyway=0,$altPageId=0)
+ *  355:     function pi_linkTP_keepPIvars_url($overrulePIvars=array(),$cache=0,$clearAnyway=0,$altPageId=0)
+ *  373:     function pi_list_linkSingle($str,$uid,$cache=FALSE,$mergeArr=array(),$urlOnly=FALSE,$altPageId=0)
+ *  401:     function pi_openAtagHrefInJSwindow($str,$winName='',$winParams='width=670,height=500,status=0,menubar=0,scrollbars=1,resizable=1')
  *
  *              SECTION: Functions for listing, browsing, searching etc.
- *  428:     function pi_list_browseresults($showResultCount=1,$tableParams='')
- *  500:     function pi_list_searchBox($tableParams='')
- *  531:     function pi_list_modeSelector($items=array(),$tableParams='')
- *  569:     function pi_list_makelist($res,$tableParams='')
- *  604:     function pi_list_row($c)
- *  616:     function pi_list_header()
+ *  456:     function pi_list_browseresults($showResultCount=1,$tableParams='',$wrapArr=array(), $pointerName = 'pointer', $hscText = TRUE)
+ *  618:     function pi_list_searchBox($tableParams='')
+ *  649:     function pi_list_modeSelector($items=array(),$tableParams='')
+ *  687:     function pi_list_makelist($res,$tableParams='')
+ *  722:     function pi_list_row($c)
+ *  734:     function pi_list_header()
  *
  *              SECTION: Stylesheet, CSS
- *  647:     function pi_getClassName($class)
- *  659:     function pi_classParam($class)
- *  673:     function pi_setClassStyle($class,$data,$selector='')
- *  684:     function pi_wrapInBaseClass($str)
+ *  765:     function pi_getClassName($class)
+ *  777:     function pi_classParam($class)
+ *  791:     function pi_setClassStyle($class,$data,$selector='')
+ *  802:     function pi_wrapInBaseClass($str)
  *
  *              SECTION: Frontend editing: Edit panel, edit icons
- *  733:     function pi_getEditPanel($row='',$tablename='',$label='',$conf=Array())
- *  775:     function pi_getEditIcon($content,$fields,$title='',$row='',$tablename='',$oConf=array())
+ *  858:     function pi_getEditPanel($row='',$tablename='',$label='',$conf=Array())
+ *  900:     function pi_getEditIcon($content,$fields,$title='',$row='',$tablename='',$oConf=array())
  *
  *              SECTION: Localization, locallang functions
- *  822:     function pi_getLL($key,$alt='',$hsc=FALSE)
- *  843:     function pi_loadLL()
+ *  947:     function pi_getLL($key,$alt='',$hsc=FALSE)
+ *  970:     function pi_loadLL()
  *
  *              SECTION: Database, queries
- *  908:     function pi_list_query($table,$count=0,$addWhere='',$mm_cat='',$groupBy='',$orderBy='',$query='',$returnQueryArray=FALSE)
- * 1000:     function pi_exec_query($table,$count=0,$addWhere='',$mm_cat='',$groupBy='',$orderBy='',$query='')
- * 1015:     function pi_getRecord($table,$uid,$checkPage=0)
- * 1026:     function pi_getPidList($pid_list,$recursive=0)
- * 1051:     function pi_prependFieldsWithTable($table,$fieldList)
- * 1071:     function pi_getCategoryTableContents($table,$pid,$whereClause='',$groupBy='',$orderBy='',$limit='')
+ * 1048:     function pi_list_query($table,$count=0,$addWhere='',$mm_cat='',$groupBy='',$orderBy='',$query='',$returnQueryArray=FALSE)
+ * 1140:     function pi_exec_query($table,$count=0,$addWhere='',$mm_cat='',$groupBy='',$orderBy='',$query='')
+ * 1155:     function pi_getRecord($table,$uid,$checkPage=0)
+ * 1166:     function pi_getPidList($pid_list,$recursive=0)
+ * 1191:     function pi_prependFieldsWithTable($table,$fieldList)
+ * 1211:     function pi_getCategoryTableContents($table,$pid,$whereClause='',$groupBy='',$orderBy='',$limit='')
  *
  *              SECTION: Various
- * 1115:     function pi_isOnlyFields($fList,$lowerThan=-1)
- * 1135:     function pi_autoCache($inArray)
- * 1166:     function pi_RTEcssText($str)
+ * 1255:     function pi_isOnlyFields($fList,$lowerThan=-1)
+ * 1275:     function pi_autoCache($inArray)
+ * 1306:     function pi_RTEcssText($str)
  *
  *              SECTION: FlexForms related functions
- * 1187:     function pi_initPIflexForm()
- * 1205:     function pi_getFFvalue($T3FlexForm_array,$fieldName,$sheet='sDEF',$lang='lDEF',$value='vDEF')
- * 1222:     function pi_getFFvalueFromSheetArray($sheetArray,$fieldNameArr,$value)
+ * 1328:     function pi_initPIflexForm($field='pi_flexform')
+ * 1346:     function pi_getFFvalue($T3FlexForm_array,$fieldName,$sheet='sDEF',$lang='lDEF',$value='vDEF')
+ * 1363:     function pi_getFFvalueFromSheetArray($sheetArray,$fieldNameArr,$value)
  *
  * TOTAL FUNCTIONS: 35
  * (This index is automatically created/updated by the extension "extdeveval")
@@ -132,29 +132,31 @@
 class tslib_pibase {
 
 		// Reserved variables:
-	var $cObj;			// The backReference to the mother cObj object set at call time
+	var $cObj;		// The backReference to the mother cObj object set at call time
 	var $prefixId;		// Should be same as classname of the plugin, used for CSS classes, variables
 	var $scriptRelPath;	// Path to the plugin class script relative to extension directory, eg. 'pi1/class.tx_newfaq_pi1.php'
 	var $extKey;		// Extension key.
-	var $piVars = Array (	// This is the incomming array by name $this->prefixId merged between POST and GET, POST taking precedence. Eg. if the class name is 'tx_myext' then the content of this array will be whatever comes into &tx_myext[...]=...
-		'pointer' => '',			// Used as a pointer for lists
-		'mode' => '',				// List mode
-		'sword' => '',				// Search word
-		'sort' => '',				// [Sorting column]:[ASC=0/DESC=1]
+	var $piVars = Array (	// This is the incoming array by name $this->prefixId merged between POST and GET, POST taking precedence. Eg. if the class name is 'tx_myext' then the content of this array will be whatever comes into &tx_myext[...]=...
+		'pointer' => '',		// Used as a pointer for lists
+		'mode' => '',			// List mode
+		'sword' => '',			// Search word
+		'sort' => '',			// [Sorting column]:[ASC=0/DESC=1]
 	);
 	var $internal = Array(	// Used internally for general storage of values between methods
-		'res_count' => 0,			// Total query count
+		'res_count' => 0,		// Total query count
 		'results_at_a_time' => 20,	// pi_list_browseresults(): Show number of results at a time
-		'maxPages' => 10,			// pi_list_browseresults(): Max number of 'Page 1 - Page 2 - ...' in the list browser
+		'maxPages' => 10,		// pi_list_browseresults(): Max number of 'Page 1 - Page 2 - ...' in the list browser
 		'currentRow' => Array(),	// Current result row
 		'currentTable' => '',		// Current table
 	);
 
-	var $LOCAL_LANG = Array();	// Local Language content
-	var $LOCAL_LANG_loaded = 0;	// Flag that tells if the locallang file has been fetch (or tried to be fetched) already.
-	var $LLkey='default';		// Pointer to the language to use.
-	var $LLtestPrefix='';		// You can set this during development to some value that makes it easy for you to spot all labels that ARe delivered by the getLL function.
-	var $LLtestPrefixAlt='';	// Save as LLtestPrefix, but additional prefix for the alternative value in getLL() function calls
+	var $LOCAL_LANG = Array();		// Local Language content
+	var $LOCAL_LANG_charset = Array();	// Local Language content charset for individual labels (overriding)
+	var $LOCAL_LANG_loaded = 0;		// Flag that tells if the locallang file has been fetch (or tried to be fetched) already.
+	var $LLkey='default';			// Pointer to the language to use.
+	var $altLLkey='';			// Pointer to alternative fall-back language to use.
+	var $LLtestPrefix='';			// You can set this during development to some value that makes it easy for you to spot all labels that ARe delivered by the getLL function.
+	var $LLtestPrefixAlt='';		// Save as LLtestPrefix, but additional prefix for the alternative value in getLL() function calls
 
 	var $pi_isOnlyFields = 'mode,pointer';
 	var $pi_alwaysPrev = 0;
@@ -165,7 +167,8 @@ class tslib_pibase {
 	var $pi_autoCacheFields=array();
 	var $pi_autoCacheEn=0;
 
-	var $pi_USER_INT_obj = 0;	// If set, then links are 1) not using cHash and 2) allowing pages to be cached.
+	var $pi_USER_INT_obj = FALSE;		// If set, then links are 1) not using cHash and 2) not allowing pages to be cached. (Set this for all USER_INT plugins!)
+	var $pi_checkCHash = FALSE;		// If set, then caching is disabled if piVars are incoming while no cHash was set (Set this for all USER plugins!)
 
 	/**
 	 * Should normally be set in the main function with the TypoScript content passed to the method.
@@ -209,11 +212,23 @@ class tslib_pibase {
 	 * @return	void
 	 */
 	function tslib_pibase()	{
+
+			// Setting piVars:
 		if ($this->prefixId)	{
 			$this->piVars = t3lib_div::GParrayMerged($this->prefixId);
+
+				// cHash mode check
+				// IMPORTANT FOR CACHED PLUGINS (USER cObject): As soon as you generate cached plugin output which depends on parameters (eg. seeing the details of a news item) you MUST check if a cHash value is set.
+				// Background: The function call will check if a cHash parameter was sent with the URL because only if it was the page may be cached. If no cHash was found the function will simply disable caching to avoid unpredictable caching behaviour. In any case your plugin can generate the expected output and the only risk is that the content may not be cached. A missing cHash value is considered a mistake in the URL resulting from either URL manipulation, "realurl" "grayzones" etc. The problem is rare (more frequent with "realurl") but when it occurs it is very puzzling!
+			if ($this->pi_checkCHash && count($this->piVars))	{
+				$GLOBALS['TSFE']->reqCHash();
+			}
 		}
 		if ($GLOBALS['TSFE']->config['config']['language'])	{
 			$this->LLkey = $GLOBALS['TSFE']->config['config']['language'];
+			if ($GLOBALS['TSFE']->config['config']['language_alt'])	{
+				$this->altLLkey = $GLOBALS['TSFE']->config['config']['language_alt'];
+			}
 		}
 	}
 
@@ -240,8 +255,6 @@ class tslib_pibase {
 
 
 
-
-
 	/***************************
 	 *
 	 * Link functions
@@ -361,11 +374,11 @@ class tslib_pibase {
 		if ($this->prefixId)	{
 			if ($cache)	{
 				$overrulePIvars=$uid?array('showUid'=>$uid):Array();
-				$overrulePIvars=array_merge($overrulePIvars,$mergeArr);
+				$overrulePIvars=array_merge($overrulePIvars,(array)$mergeArr);
 				$str = $this->pi_linkTP($str,Array($this->prefixId=>$overrulePIvars),$cache,$altPageId);
 			} else {
 				$overrulePIvars=array('showUid'=>$uid?$uid:'');
-				$overrulePIvars=array_merge($overrulePIvars,$mergeArr);
+				$overrulePIvars=array_merge($overrulePIvars,(array)$mergeArr);
 				$str = $this->pi_linkTP_keepPIvars($str,$overrulePIvars,$cache,0,$altPageId);
 			}
 
@@ -409,7 +422,6 @@ class tslib_pibase {
 
 
 
-
 	/***************************
 	 *
 	 * Functions for listing, browsing, searching etc.
@@ -418,74 +430,180 @@ class tslib_pibase {
 
 	/**
 	 * Returns a results browser. This means a bar of page numbers plus a "previous" and "next" link. For each entry in the bar the piVars "pointer" will be pointing to the "result page" to show.
-	 * Using $this->piVars['pointer'] as pointer to the page to display
+	 * Using $this->piVars['pointer'] as pointer to the page to display. Can be overwritten with another string ($pointerName) to make it possible to have more than one pagebrowser on a page)
 	 * Using $this->internal['res_count'], $this->internal['results_at_a_time'] and $this->internal['maxPages'] for count number, how many results to show and the max number of pages to include in the browse bar.
+	 * Using $this->internal['dontLinkActivePage'] as switch if the active (current) page should be displayed as pure text or as a link to itself
+	 * Using $this->internal['showFirstLast'] as switch if the two links named "<< First" and "LAST >>" will be shown and point to the first or last page.
+	 * Using $this->internal['pagefloat']: this defines were the current page is shown in the list of pages in the Pagebrowser. If this var is an integer it will be interpreted as position in the list of pages. If its value is the keyword "center" the current page will be shown in the middle of the pagelist.
+	 * Using $this->internal['showRange']: this var switches the display of the pagelinks from pagenumbers to ranges f.e.: 1-5 6-10 11-15... instead of 1 2 3...
+	 * Using $this->pi_isOnlyFields: this holds a comma-separated list of fieldnames which - if they are among the GETvars - will not disable caching for the page with pagebrowser.
 	 *
-	 * @param	boolean		If set (default) the text "Displaying results..." will be show, otherwise not.
+	 * The third parameter is an array with several wraps for the parts of the pagebrowser. The following elements will be recognized:
+	 * disabledLinkWrap, inactiveLinkWrap, activeLinkWrap, browseLinksWrap, showResultsWrap, showResultsNumbersWrap, browseBoxWrap.
+	 *
+	 * If $wrapArr['showResultsNumbersWrap'] is set, the formatting string is expected to hold template markers (###FROM###, ###TO###, ###OUT_OF###, ###FROM_TO###, ###CURRENT_PAGE###, ###TOTAL_PAGES###)
+	 * otherwise the formatting sting is expected to hold sprintf-markers (%s) for from, to, outof (in that sequence)
+	 *
+	 * 										otherwise wrapping is totally controlled/modified by this array
+	 *
+	 * @param	integer		determines how the results of the pagerowser will be shown. See description below
 	 * @param	string		Attributes for the table tag which is wrapped around the table cells containing the browse links
-	 * @return	string		Output HTML, wrapped in <div>-tags with a class attribute
+	 * @param	array		Array with elements to overwrite the default $wrapper-array.
+	 * @param	string		varname for the pointer.
+	 * @param	boolean		enable htmlspecialchars() for the pi_getLL function (set this to FALSE if you want f.e use images instead of text for links like 'previous' and 'next').
+	 * @return	string		Output HTML-Table, wrapped in <div>-tags with a class attribute (if $wrapArr is not passed,
 	 */
-	function pi_list_browseresults($showResultCount=1,$tableParams='')	{
+	function pi_list_browseresults($showResultCount=1,$tableParams='',$wrapArr=array(), $pointerName = 'pointer', $hscText = TRUE)	{
+
+		// example $wrapArr-array how it could be traversed from an extension
+		/* $wrapArr = array(
+			'browseBoxWrap' => '<div class="browseBoxWrap">|</div>',
+			'showResultsWrap' => '<div class="showResultsWrap">|</div>',
+			'browseLinksWrap' => '<div class="browseLinksWrap">|</div>',
+			'showResultsNumbersWrap' => '<span class="showResultsNumbersWrap">|</span>',
+			'disabledLinkWrap' => '<span class="disabledLinkWrap">|</span>',
+			'inactiveLinkWrap' => '<span class="inactiveLinkWrap">|</span>',
+			'activeLinkWrap' => '<span class="activeLinkWrap">|</span>'
+		); */
 
 			// Initializing variables:
-		$pointer=$this->piVars['pointer'];
-		$count=$this->internal['res_count'];
+		$pointer = intval($this->piVars[$pointerName]);
+		$count = intval($this->internal['res_count']);
 		$results_at_a_time = t3lib_div::intInRange($this->internal['results_at_a_time'],1,1000);
+		$totalPages = ceil($count/$results_at_a_time);
 		$maxPages = t3lib_div::intInRange($this->internal['maxPages'],1,100);
-		$max = t3lib_div::intInRange(ceil($count/$results_at_a_time),1,$maxPages);
-		$pointer=intval($pointer);
-		$links=array();
+		$pi_isOnlyFields = $this->pi_isOnlyFields($this->pi_isOnlyFields);
 
-			// Make browse-table/links:
-		if ($this->pi_alwaysPrev>=0)	{
-			if ($pointer>0)	{
-				$links[]='
-					<td nowrap="nowrap"><p>'.$this->pi_linkTP_keepPIvars($this->pi_getLL('pi_list_browseresults_prev','< Previous',TRUE),array('pointer'=>($pointer-1?$pointer-1:'')),0).'</p></td>';
-			} elseif ($this->pi_alwaysPrev)	{
-				$links[]='
-					<td nowrap="nowrap"><p>'.$this->pi_getLL('pi_list_browseresults_prev','< Previous',TRUE).'</p></td>';
+			// $showResultCount determines how the results of the pagerowser will be shown.
+			// If set to 0: only the result-browser will be shown
+			//	 		 1: (default) the text "Displaying results..." and the result-browser will be shown.
+			//	 		 2: only the text "Displaying results..." will be shown
+		$showResultCount = intval($showResultCount);
+
+			// if this is set, two links named "<< First" and "LAST >>" will be shown and point to the very first or last page.
+		$showFirstLast = $this->internal['showFirstLast'];
+
+			// if this has a value the "previous" button is always visible (will be forced if "showFirstLast" is set)
+		$alwaysPrev = $showFirstLast?1:$this->pi_alwaysPrev;
+
+		if (isset($this->internal['pagefloat'])) {
+			if (strtoupper($this->internal['pagefloat']) == 'CENTER') {
+				$pagefloat = ceil(($maxPages - 1)/2);
+			} else {
+				// pagefloat set as integer. 0 = left, value >= $this->internal['maxPages'] = right
+				$pagefloat = t3lib_div::intInRange($this->internal['pagefloat'],-1,$maxPages-1);
 			}
+		} else {
+			$pagefloat = -1; // pagefloat disabled
 		}
-		for($a=0;$a<$max;$a++)	{
-			$links[]='
-					<td'.($pointer==$a?$this->pi_classParam('browsebox-SCell'):'').' nowrap="nowrap"><p>'.
-				$this->pi_linkTP_keepPIvars(trim($this->pi_getLL('pi_list_browseresults_page','Page',TRUE).' '.($a+1)),array('pointer'=>($a?$a:'')),$this->pi_isOnlyFields($this->pi_isOnlyFields)).
-				'</p></td>';
-		}
-		if ($pointer<ceil($count/$results_at_a_time)-1)	{
-			$links[]='
-					<td nowrap="nowrap"><p>'.
-				$this->pi_linkTP_keepPIvars($this->pi_getLL('pi_list_browseresults_next','Next >',TRUE),array('pointer'=>$pointer+1)).
-				'</p></td>';
+
+			// default values for "traditional" wrapping with a table. Can be overwritten by vars from $wrapArr
+		$wrapper['disabledLinkWrap'] = '<td nowrap="nowrap"><p>|</p></td>';
+		$wrapper['inactiveLinkWrap'] = '<td nowrap="nowrap"><p>|</p></td>';
+		$wrapper['activeLinkWrap'] = '<td'.$this->pi_classParam('browsebox-SCell').' nowrap="nowrap"><p>|</p></td>';
+		$wrapper['browseLinksWrap'] = trim('<table '.$tableParams).'><tr>|</tr></table>';
+		$wrapper['showResultsWrap'] = '<p>|</p>';
+		$wrapper['browseBoxWrap'] = '
+		<!--
+			List browsing box:
+		-->
+		<div '.$this->pi_classParam('browsebox').'>
+			|
+		</div>';
+
+			// now overwrite all entries in $wrapper which are also in $wrapArr
+		$wrapper = array_merge($wrapper,$wrapArr);
+
+		if ($showResultCount != 2) { //show pagebrowser
+			if ($pagefloat > -1) {
+				$lastPage = min($totalPages,max($pointer+1 + $pagefloat,$maxPages));
+				$firstPage = max(0,$lastPage-$maxPages);
+			} else {
+				$firstPage = 0;
+				$lastPage = t3lib_div::intInRange($totalPages,1,$maxPages);
+			}
+			$links=array();
+
+				// Make browse-table/links:
+			if ($showFirstLast) { // Link to first page
+				if ($pointer>0)	{
+					$links[]=$this->cObj->wrap($this->pi_linkTP_keepPIvars($this->pi_getLL('pi_list_browseresults_first','<< First',$hscText),array($pointerName => null),$pi_isOnlyFields),$wrapper['inactiveLinkWrap']);
+				} else {
+					$links[]=$this->cObj->wrap($this->pi_getLL('pi_list_browseresults_first','<< First',$hscText),$wrapper['disabledLinkWrap']);
+				}
+			}
+			if ($alwaysPrev>=0)	{ // Link to previous page
+				if ($pointer>0)	{
+					$links[]=$this->cObj->wrap($this->pi_linkTP_keepPIvars($this->pi_getLL('pi_list_browseresults_prev','< Previous',$hscText),array($pointerName => ($pointer-1?$pointer-1:'')),$pi_isOnlyFields),$wrapper['inactiveLinkWrap']);
+				} elseif ($alwaysPrev)	{
+					$links[]=$this->cObj->wrap($this->pi_getLL('pi_list_browseresults_prev','< Previous',$hscText),$wrapper['disabledLinkWrap']);
+				}
+			}
+			for($a=$firstPage;$a<$lastPage;$a++)	{ // Links to pages
+				if ($this->internal['showRange']) {
+					$pageText = (($a*$results_at_a_time)+1).'-'.min($count,(($a+1)*$results_at_a_time));
+				} else {
+					$pageText = trim($this->pi_getLL('pi_list_browseresults_page','Page',$hscText).' '.($a+1));
+				}
+				if ($pointer == $a) { // current page
+					if ($this->internal['dontLinkActivePage']) {
+						$links[] = $this->cObj->wrap($pageText,$wrapper['activeLinkWrap']);
+					} else {
+						$links[] = $this->cObj->wrap($this->pi_linkTP_keepPIvars($pageText,array($pointerName  => ($a?$a:'')),$pi_isOnlyFields),$wrapper['activeLinkWrap']);
+					}
+				} else {
+					$links[] = $this->cObj->wrap($this->pi_linkTP_keepPIvars($pageText,array($pointerName => ($a?$a:'')),$pi_isOnlyFields),$wrapper['inactiveLinkWrap']);
+				}
+			}
+			if ($pointer<$totalPages-1 || $showFirstLast)	{
+				if ($pointer==$totalPages-1) { // Link to next page
+					$links[]=$this->cObj->wrap($this->pi_getLL('pi_list_browseresults_next','Next >',$hscText),$wrapper['disabledLinkWrap']);
+				} else {
+					$links[]=$this->cObj->wrap($this->pi_linkTP_keepPIvars($this->pi_getLL('pi_list_browseresults_next','Next >',$hscText),array($pointerName => $pointer+1),$pi_isOnlyFields),$wrapper['inactiveLinkWrap']);
+				}
+			}
+			if ($showFirstLast) { // Link to last page
+				if ($pointer<$totalPages-1) {
+					$links[]=$this->cObj->wrap($this->pi_linkTP_keepPIvars($this->pi_getLL('pi_list_browseresults_last','Last >>',$hscText),array($pointerName => $totalPages-1),$pi_isOnlyFields),$wrapper['inactiveLinkWrap']);
+				} else {
+					$links[]=$this->cObj->wrap($this->pi_getLL('pi_list_browseresults_last','Last >>',$hscText),$wrapper['disabledLinkWrap']);
+				}
+			}
+			$theLinks = $this->cObj->wrap(implode(chr(10),$links),$wrapper['browseLinksWrap']);
+		} else {
+			$theLinks = '';
 		}
 
 		$pR1 = $pointer*$results_at_a_time+1;
 		$pR2 = $pointer*$results_at_a_time+$results_at_a_time;
-		$sTables = '
 
-		<!--
-			List browsing box:
-		-->
-		<div'.$this->pi_classParam('browsebox').'>'.
-			($showResultCount ? '
-			<p>'.
-				($this->internal['res_count'] ?
-    			sprintf(
+		if ($showResultCount) {
+			if ($wrapper['showResultsNumbersWrap']) {
+				// this will render the resultcount in a more flexible way using markers (new in TYPO3 3.8.0).
+				// the formatting string is expected to hold template markers (see function header). Example: 'Displaying results ###FROM### to ###TO### out of ###OUT_OF###'
+
+				$markerArray['###FROM###'] = $this->cObj->wrap($this->internal['res_count'] > 0 ? $pR1 : 0,$wrapper['showResultsNumbersWrap']);
+				$markerArray['###TO###'] = $this->cObj->wrap(min($this->internal['res_count'],$pR2),$wrapper['showResultsNumbersWrap']);
+				$markerArray['###OUT_OF###'] = $this->cObj->wrap($this->internal['res_count'],$wrapper['showResultsNumbersWrap']);
+				$markerArray['###FROM_TO###'] = $this->cObj->wrap(($this->internal['res_count'] > 0 ? $pR1 : 0).' '.$this->pi_getLL('pi_list_browseresults_to','to').' '.min($this->internal['res_count'],$pR2),$wrapper['showResultsNumbersWrap']);
+				$markerArray['###CURRENT_PAGE###'] = $this->cObj->wrap($pointer+1,$wrapper['showResultsNumbersWrap']);
+				$markerArray['###TOTAL_PAGES###'] = $this->cObj->wrap($totalPages,$wrapper['showResultsNumbersWrap']);
+				// substitute markers
+				$resultCountMsg = $this->cObj->substituteMarkerArray($this->pi_getLL('pi_list_browseresults_displays','Displaying results ###FROM### to ###TO### out of ###OUT_OF###'),$markerArray);
+			} else {
+				// render the resultcount in the "traditional" way using sprintf
+				$resultCountMsg = sprintf(
 					str_replace('###SPAN_BEGIN###','<span'.$this->pi_classParam('browsebox-strong').'>',$this->pi_getLL('pi_list_browseresults_displays','Displaying results ###SPAN_BEGIN###%s to %s</span> out of ###SPAN_BEGIN###%s</span>')),
-					$this->internal['res_count'] > 0 ? $pR1 : 0,
-					min(array($this->internal['res_count'],$pR2)),
-					$this->internal['res_count']
-				) :
-				$this->pi_getLL('pi_list_browseresults_noResults','Sorry, no items were found.')).'</p>':''
-			).
-		'
+					$count > 0 ? $pR1 : 0,
+					min($count,$pR2),
+					$count);
+			}
+			$resultCountMsg = $this->cObj->wrap($resultCountMsg,$wrapper['showResultsWrap']);
+		} else {
+			$resultCountMsg = '';
+		}
 
-			<'.trim('table '.$tableParams).'>
-				<tr>
-					'.implode('',$links).'
-				</tr>
-			</table>
-		</div>';
+		$sTables = $this->cObj->wrap($resultCountMsg.$theLinks,$wrapper['browseBoxWrap']);
 
 		return $sTables;
 	}
@@ -667,7 +785,7 @@ class tslib_pibase {
 	 * @param	string		$data: CSS data
 	 * @param	string		If $selector is set to any CSS selector, eg 'P' or 'H1' or 'TABLE' then the style $data will regard those HTML-elements only
 	 * @return	void
-	 * @depreciated		I think this function should not be used (and probably isn't used anywhere). It was a part of a concept which was left behind quite quickly.
+	 * @deprecated		I think this function should not be used (and probably isn't used anywhere). It was a part of a concept which was left behind quite quickly.
 	 * @private
 	 */
 	function pi_setClassStyle($class,$data,$selector='')	{
@@ -682,7 +800,13 @@ class tslib_pibase {
 	 * @return	string		HTML content wrapped, ready to return to the parent object.
 	 */
 	function pi_wrapInBaseClass($str)	{
-		return '
+		$content = '<div class="'.str_replace('_','-',$this->prefixId).'">
+		'.$str.'
+	</div>
+	';
+
+		if(!$GLOBALS['TSFE']->config['config']['disablePrefixComment'])	{
+			$content = '
 
 
 	<!--
@@ -690,12 +814,13 @@ class tslib_pibase {
 		BEGIN: Content of extension "'.$this->extKey.'", plugin "'.$this->prefixId.'"
 
 	-->
-	<div class="'.str_replace('_','-',$this->prefixId).'">
-		'.$str.'
-	</div>
+	'.$content.'
 	<!-- END: Content of extension "'.$this->extKey.'", plugin "'.$this->prefixId.'" -->
 
 	';
+		}
+
+		return $content;
 	}
 
 
@@ -821,9 +946,11 @@ class tslib_pibase {
 	 */
 	function pi_getLL($key,$alt='',$hsc=FALSE)	{
 		if (isset($this->LOCAL_LANG[$this->LLkey][$key]))	{
-			$word = $GLOBALS['TSFE']->csConv($this->LOCAL_LANG[$this->LLkey][$key]);
+			$word = $GLOBALS['TSFE']->csConv($this->LOCAL_LANG[$this->LLkey][$key], $this->LOCAL_LANG_charset[$this->LLkey][$key]);	// The "from" charset is normally empty and thus it will convert from the charset of the system language, but if it is set (see ->pi_loadLL()) it will be used.
+		} elseif ($this->altLLkey && isset($this->LOCAL_LANG[$this->altLLkey][$key]))	{
+			$word = $GLOBALS['TSFE']->csConv($this->LOCAL_LANG[$this->altLLkey][$key], $this->LOCAL_LANG_charset[$this->altLLkey][$key]);	// The "from" charset is normally empty and thus it will convert from the charset of the system language, but if it is set (see ->pi_loadLL()) it will be used.
 		} elseif (isset($this->LOCAL_LANG['default'][$key]))	{
-			$word = $this->LOCAL_LANG['default'][$key];
+			$word = $this->LOCAL_LANG['default'][$key];	// No charset conversion because default is english and thereby ASCII
 		} else {
 			$word = $this->LLtestPrefixAlt.$alt;
 		}
@@ -842,16 +969,29 @@ class tslib_pibase {
 	 */
 	function pi_loadLL()	{
 		if (!$this->LOCAL_LANG_loaded && $this->scriptRelPath)	{
-			$basePath = t3lib_extMgm::siteRelPath($this->extKey).dirname($this->scriptRelPath).'/locallang.php';
-			if (@is_file($basePath))	{
-				include('./'.$basePath);
-				$this->LOCAL_LANG = $LOCAL_LANG;
-				if (is_array($this->conf['_LOCAL_LANG.']))	{
-					reset($this->conf['_LOCAL_LANG.']);
-					while(list($k,$lA)=each($this->conf['_LOCAL_LANG.']))	{
-						if (is_array($lA))	{
-							$k = substr($k,0,-1);
-							$this->LOCAL_LANG[$k] = t3lib_div::array_merge_recursive_overrule(is_array($this->LOCAL_LANG[$k])?$this->LOCAL_LANG[$k]:array(), $lA);
+			$basePath = t3lib_extMgm::extPath($this->extKey).dirname($this->scriptRelPath).'/locallang.php';
+
+				// php or xml as source: In any case the charset will be that of the system language.
+				// However, this function guarantees only return output for default language plus the specified language (which is different from how 3.7.0 dealt with it)
+			$this->LOCAL_LANG = t3lib_div::readLLfile($basePath,$this->LLkey);
+			if ($this->altLLkey)	{
+				$tempLOCAL_LANG = t3lib_div::readLLfile($basePath,$this->altLLkey);
+				$this->LOCAL_LANG = array_merge(is_array($this->LOCAL_LANG) ? $this->LOCAL_LANG : array(),$tempLOCAL_LANG);
+			}
+
+				// Overlaying labels from TypoScript (including fictitious language keys for non-system languages!):
+			if (is_array($this->conf['_LOCAL_LANG.']))	{
+				reset($this->conf['_LOCAL_LANG.']);
+				while(list($k,$lA)=each($this->conf['_LOCAL_LANG.']))	{
+					if (is_array($lA))	{
+						$k = substr($k,0,-1);
+						foreach($lA as $llK => $llV)	{
+							if (!is_array($llV))	{
+								$this->LOCAL_LANG[$k][$llK] = $llV;
+								if ($k != 'default')	{
+									$this->LOCAL_LANG_charset[$k][$llK] = $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset'];	// For labels coming from the TypoScript (database) the charset is assumed to be "forceCharset" and if that is not set, assumed to be that of the individual system languages (thus no conversion)
+								}
+							}
 						}
 					}
 				}
@@ -903,7 +1043,7 @@ class tslib_pibase {
 	 * @param	boolean		If set, the function will return the query not as a string but array with the various parts.
 	 * @return	mixed		The query build.
 	 * @access private
-	 * @depreciated		Use pi_exec_query() instead!
+	 * @deprecated		Use pi_exec_query() instead!
 	 */
 	function pi_list_query($table,$count=0,$addWhere='',$mm_cat='',$groupBy='',$orderBy='',$query='',$returnQueryArray=FALSE)	{
 
@@ -949,7 +1089,7 @@ class tslib_pibase {
 			);
 		} else {
 				// Order by data:
-			if (!$orderBy)	{
+			if (!$orderBy && $this->internal['orderBy'])	{
 				if (t3lib_div::inList($this->internal['orderByList'],$this->internal['orderBy']))	{
 					$orderBy = 'ORDER BY '.$table.'.'.$this->internal['orderBy'].($this->internal['descFlag']?' DESC':'');
 				}
@@ -994,7 +1134,7 @@ class tslib_pibase {
 	 * @param	mixed		If an array, then it must contain the keys "table", "mmtable" and (optionally) "catUidList" defining a table to make a MM-relation to in the query (based on fields uid_local and uid_foreign). If not array, the query will be a plain query looking up data in only one table.
 	 * @param	string		If set, this is added as a " GROUP BY ...." part of the query.
 	 * @param	string		If set, this is added as a " ORDER BY ...." part of the query. The default is that an ORDER BY clause is made based on $this->internal['orderBy'] and $this->internal['descFlag'] where the orderBy field must be found in $this->internal['orderByList']
-	 * @param	string		If set, this is taken as the first part of the query instead of what is created internally. Basically this should be a query starting with "FROM [table] WHERE ... AND ...". The $addWhere clauses and all the other stuff is still added. Only the tables and PID selecting clauses are bypassed. May be depreciated in the future!
+	 * @param	string		If set, this is taken as the first part of the query instead of what is created internally. Basically this should be a query starting with "FROM [table] WHERE ... AND ...". The $addWhere clauses and all the other stuff is still added. Only the tables and PID selecting clauses are bypassed. May be deprecated in the future!
 	 * @return	pointer		SQL result pointer
 	 */
 	function pi_exec_query($table,$count=0,$addWhere='',$mm_cat='',$groupBy='',$orderBy='',$query='')	{
@@ -1182,13 +1322,14 @@ class tslib_pibase {
 	/**
 	 * Converts $this->cObj->data['pi_flexform'] from XML string to flexForm array.
 	 *
+	 * @param	string		Field name to convert
 	 * @return	void
 	 */
-	function pi_initPIflexForm()	{
+	function pi_initPIflexForm($field='pi_flexform')	{
 			// Converting flexform data into array:
-		if (!is_array($this->cObj->data['pi_flexform']) && $this->cObj->data['pi_flexform'])	{
-			$this->cObj->data['pi_flexform'] = t3lib_div::xml2array($this->cObj->data['pi_flexform']);
-			if (!is_array($this->cObj->data['pi_flexform']))	$this->cObj->data['pi_flexform']=array();
+		if (!is_array($this->cObj->data[$field]) && $this->cObj->data[$field])	{
+			$this->cObj->data[$field] = t3lib_div::xml2array($this->cObj->data[$field]);
+			if (!is_array($this->cObj->data[$field]))	$this->cObj->data[$field]=array();
 		}
 	}
 
@@ -1203,7 +1344,7 @@ class tslib_pibase {
 	 * @return	string		The content.
 	 */
 	function pi_getFFvalue($T3FlexForm_array,$fieldName,$sheet='sDEF',$lang='lDEF',$value='vDEF')	{
-		$sheetArray = $T3FlexForm_array['data'][$sheet][$lang];
+		$sheetArray = is_array($T3FlexForm_array) ? $T3FlexForm_array['data'][$sheet][$lang] : '';
 		if (is_array($sheetArray))	{
 			return $this->pi_getFFvalueFromSheetArray($sheetArray,explode('/',$fieldName),$value);
 		}
