@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Contains class with layout/output function for TYPO3 Backend Scripts
  *
- * $Id: template.php,v 1.33 2004/09/13 22:57:22 typo3 Exp $
+ * $Id: template.php 1507 2006-06-07 09:52:20Z mundaun $
  * Revised for TYPO3 3.6 2/2003 by Kasper Skaarhoj
  * XHTML-trans compliant
  *
@@ -36,78 +36,86 @@
 /**
  * [CLASS/FUNCTION INDEX of SCRIPT]
  *
- *  137: function fw($str)
+ *  145: function fw($str)
  *
  *
- *  161: class template
- *  216:     function template()
+ *  169: class template
+ *  224:     function template()
  *
  *              SECTION: EVALUATION FUNCTIONS
- *  290:     function wrapClickMenuOnIcon($str,$table,$uid='',$listFr=1,$addParams='',$enDisItems='', $returnOnClick=FALSE)
- *  307:     function viewPageIcon($id,$backPath,$addParams='hspace="3"')
- *  333:     function issueCommand($params,$rUrl='')
- *  348:     function isCMlayers()
- *  358:     function thisBlur()
- *  368:     function helpStyle()
- *  385:     function getHeader($table,$row,$path,$noViewPageIcon=0,$tWrap=array('',''))
- *  411:     function getFileheader($title,$path,$iconfile)
- *  426:     function makeShortcutIcon($gvList,$setList,$modName,$motherModName="")
- *  459:     function makeShortcutUrl($gvList,$setList)
- *  480:     function formWidth($size=48,$textarea=0,$styleOverride='')
- *  505:     function formWidthText($size=48,$styleOverride='',$wrap='')
- *  522:     function redirectUrls($thisLocation='')
- *  546:     function formatTime($tstamp,$type)
- *  559:     function parseTime()
+ *  298:     function wrapClickMenuOnIcon($str,$table,$uid='',$listFr=1,$addParams='',$enDisItems='', $returnOnClick=FALSE)
+ *  315:     function viewPageIcon($id,$backPath,$addParams='hspace="3"')
+ *  341:     function issueCommand($params,$rUrl='')
+ *  356:     function isCMlayers()
+ *  366:     function thisBlur()
+ *  376:     function helpStyle()
+ *  393:     function getHeader($table,$row,$path,$noViewPageIcon=0,$tWrap=array('',''))
+ *  419:     function getFileheader($title,$path,$iconfile)
+ *  434:     function makeShortcutIcon($gvList,$setList,$modName,$motherModName="")
+ *  467:     function makeShortcutUrl($gvList,$setList)
+ *  488:     function formWidth($size=48,$textarea=0,$styleOverride='')
+ *  513:     function formWidthText($size=48,$styleOverride='',$wrap='')
+ *  530:     function redirectUrls($thisLocation='')
+ *  554:     function formatTime($tstamp,$type)
+ *  571:     function parseTime()
  *
  *              SECTION: PAGE BUILDING FUNCTIONS.
- *  592:     function startPage($title)
- *  667:     function endPage()
- *  696:     function header($text)
- *  717:     function section($label,$text,$nostrtoupper=FALSE,$sH=FALSE,$type=0,$allowHTMLinHeader=FALSE)
- *  741:     function divider($dist)
- *  757:     function spacer($dist)
- *  776:     function sectionHeader($label,$sH=FALSE,$addAttrib='')
- *  793:     function sectionBegin()
- *  814:     function sectionEnd()
- *  834:     function middle()
- *  843:     function endPageJS()
- *  860:     function docBodyTagBegin()
- *  870:     function docStyle()
- *  912:     function insertStylesAndJS($content)
- *  932:     function initCharset()
- *  944:     function generator()
+ *  604:     function startPage($title)
+ *  686:     function endPage()
+ *  720:     function header($text)
+ *  741:     function section($label,$text,$nostrtoupper=FALSE,$sH=FALSE,$type=0,$allowHTMLinHeader=FALSE)
+ *  765:     function divider($dist)
+ *  781:     function spacer($dist)
+ *  800:     function sectionHeader($label,$sH=FALSE,$addAttrib='')
+ *  817:     function sectionBegin()
+ *  838:     function sectionEnd()
+ *  858:     function middle()
+ *  867:     function endPageJS()
+ *  884:     function docBodyTagBegin()
+ *  894:     function docStyle()
+ *  936:     function insertStylesAndJS($content)
+ *  956:     function initCharset()
+ *  968:     function generator()
  *
  *              SECTION: OTHER ELEMENTS
- *  977:     function icons($type, $styleAttribValue='')
- * 1006:     function t3Button($onClick,$label)
- * 1017:     function dfw($string)
- * 1027:     function rfw($string)
- * 1037:     function wrapInCData($string)
- * 1054:     function wrapScriptTags($string, $linebreak=TRUE)
- * 1092:     function table($arr, $layout='')
- * 1134:     function menuTable($arr1,$arr2=array(), $arr3=array())
- * 1167:     function funcMenu($content,$menu)
- * 1185:     function clearCacheMenu($id,$addSaveOptions=0)
- * 1221:     function getContextMenuCode()
- * 1408:     function getTabMenu($mainParams,$elementName,$currentValue,$menuItems,$script='',$addparams='')
- * 1438:     function getTabMenuRaw($menuItems)
- * 1507:     function getDynTabMenu($menuItems,$identString,$toggle=0,$foldout=FALSE,$newRowCharLimit=50,$noWrap=1,$fullWidth=FALSE,$defaultTabIndex=1)
- * 1630:     function getDynTabMenuJScode()
- * 1687:     function getVersionSelector($id,$noAction=FALSE)
+ * 1001:     function icons($type, $styleAttribValue='')
+ * 1030:     function t3Button($onClick,$label)
+ * 1041:     function dfw($string)
+ * 1051:     function rfw($string)
+ * 1061:     function wrapInCData($string)
+ * 1078:     function wrapScriptTags($string, $linebreak=TRUE)
+ * 1117:     function table($arr, $layout='')
+ * 1159:     function menuTable($arr1,$arr2=array(), $arr3=array())
+ * 1192:     function funcMenu($content,$menu)
+ * 1210:     function clearCacheMenu($id,$addSaveOptions=0)
+ * 1246:     function getContextMenuCode()
+ * 1251:     function showClickmenu(table, uid, listFr, enDisItems, backPath, addParams)
+ * 1280:     function showClickmenu_noajax(url)
+ * 1287:     function showClickmenu_ajax(t3ajax)
+ * 1472:     function getDragDropCode($table)
+ * 1483:     function cancelDragEvent(event)
+ * 1496:     function mouseMoveEvent (event)
+ * 1509:     function dragElement(id,elementID)
+ * 1528:     function dropElement(id)
+ * 1577:     function getTabMenu($mainParams,$elementName,$currentValue,$menuItems,$script='',$addparams='')
+ * 1607:     function getTabMenuRaw($menuItems)
+ * 1676:     function getDynTabMenu($menuItems,$identString,$toggle=0,$foldout=FALSE,$newRowCharLimit=50,$noWrap=1,$fullWidth=FALSE,$defaultTabIndex=1)
+ * 1801:     function getDynTabMenuJScode()
+ * 1892:     function getVersionSelector($id,$noAction=FALSE)
  *
  *
- * 1761: class bigDoc extends template
+ * 2060: class bigDoc extends template
  *
  *
- * 1770: class noDoc extends template
+ * 2069: class noDoc extends template
  *
  *
- * 1779: class smallDoc extends template
+ * 2078: class smallDoc extends template
  *
  *
- * 1788: class mediumDoc extends template
+ * 2087: class mediumDoc extends template
  *
- * TOTAL FUNCTIONS: 49
+ * TOTAL FUNCTIONS: 57
  * (This index is automatically created/updated by the extension "extdeveval")
  *
  */
@@ -116,7 +124,7 @@
 
 if (!defined('TYPO3_MODE'))	die("Can't include this file directly.");
 
-
+require_once(PATH_t3lib.'class.t3lib_ajax.php');
 
 
 
@@ -128,11 +136,11 @@ if (!defined('TYPO3_MODE'))	die("Can't include this file directly.");
 
 
 /**
- * Depreciated fontwrap function. Is just transparent now.
+ * Deprecated fontwrap function. Is just transparent now.
  *
  * @param	string		Input string
  * @return	string		Output string (in the old days this was wrapped in <font> tags)
- * @depreciated
+ * @deprecated
  */
 function fw($str)	{
 	return $str;
@@ -288,9 +296,9 @@ class template {
 	 * @return	string		The link-wrapped input string.
 	 */
 	function wrapClickMenuOnIcon($str,$table,$uid='',$listFr=1,$addParams='',$enDisItems='', $returnOnClick=FALSE)	{
-		$backPath = '&backPath='.rawurlencode($this->backPath).'|'.t3lib_div::shortMD5($this->backPath.'|'.$GLOBALS['TYPO3_CONF_VARS']['SYS']['encryptionKey']);
-		$onClick = 'top.loadTopMenu(\''.$this->backPath.'alt_clickmenu.php?item='.rawurlencode($table.'|'.$uid.'|'.$listFr.'|'.$enDisItems).$backPath.$addParams.'\');'.$this->thisBlur().'return false;';
-		return $returnOnClick ? $onClick : '<a href="#" onclick="'.htmlspecialchars($onClick).'">'.$str.'</a>';
+		$backPath = rawurlencode($this->backPath).'|'.t3lib_div::shortMD5($this->backPath.'|'.$GLOBALS['TYPO3_CONF_VARS']['SYS']['encryptionKey']);
+		$onClick = 'showClickmenu("'.$table.'","'.$uid.'","'.$listFr.'","'.$enDisItems.'","'.str_replace('&','&amp;',addcslashes($backPath,'"')).'","'.str_replace('&','&amp;',addcslashes($addParams,'"')).'");return false;';
+		return $returnOnClick ? $onClick : '<a href="#" onclick="'.htmlspecialchars($onClick).'"'.($GLOBALS['TYPO3_CONF_VARS']['BE']['useOnContextMenuHandler'] ? ' oncontextmenu="'.htmlspecialchars($onClick).'"' : '').'>'.$str.'</a>';
 	}
 
 	/**
@@ -346,7 +354,7 @@ class template {
 	 * @return	boolean
 	 */
 	function isCMlayers()	{
-		return !$GLOBALS['BE_USER']->uc['disableCMlayers'] && $GLOBALS['CLIENT']['FORMSTYLE'] && $GLOBALS['CLIENT']['SYSTEM']!='mac';
+		return !$GLOBALS['BE_USER']->uc['disableCMlayers'] && $GLOBALS['CLIENT']['FORMSTYLE'] && !($GLOBALS['CLIENT']['SYSTEM']=='mac' && $GLOBALS['CLIENT']['BROWSER']=='Opera');
 	}
 
 	/**
@@ -432,11 +440,11 @@ class template {
 			$mMN="&motherModName='+top.currentModuleLoaded+'";
 		} elseif ($motherModName)	{
 			$mMN='&motherModName='.rawurlencode($motherModName);
-		} else $mMN="";
+		} else $mMN='';
 
 		$onClick = 'if (top.shortcutFrame && confirm('.
 					$GLOBALS['LANG']->JScharCode($GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:labels.makeShortcut')).
-					')){top.shortcutFrame.document.location=\''.$backPath.'alt_shortcut.php?modName='.rawurlencode($modName).
+					')){top.shortcutFrame.location.href=\''.$backPath.'alt_shortcut.php?modName='.rawurlencode($modName).
 					'&URL='.rawurlencode($pathInfo['path']."?".$storeUrl).
 					$mMN.
 					'\';}return false;';
@@ -472,7 +480,7 @@ class template {
 	 * For CSS compliant browsers (recommended) a ' style="width: ...px;"' is returned.
 	 *
 	 * @param	integer		A relative number which multiplied with approx. 10 will lead to the width in pixels
-	 * @param	boolean		A flag you can set for textareas - DEPRECIATED, use ->formWidthText() for textareas!!!
+	 * @param	boolean		A flag you can set for textareas - DEPRECATED, use ->formWidthText() for textareas!!!
 	 * @param	string		A string which will be returned as attribute-value for style="" instead of the calculated width (if CSS is enabled)
 	 * @return	string		Tag attributes for an <input> tag (regarding width)
 	 * @see formWidthText()
@@ -544,10 +552,14 @@ class template {
 	 * @return	string		Formatted timestamp
 	 */
 	function formatTime($tstamp,$type)	{
+		$dateStr = '';
 		switch($type)	{
-			case 1: return date($GLOBALS['TYPO3_CONF_VARS']['SYS']['hhmm'],$tstamp); break;
-			case 10: return date($GLOBALS['TYPO3_CONF_VARS']['SYS']['ddmmyy'],$tstamp); break;
+			case 1: $dateStr = date($GLOBALS['TYPO3_CONF_VARS']['SYS']['hhmm'],$tstamp);
+			break;
+			case 10: $dateStr = date($GLOBALS['TYPO3_CONF_VARS']['SYS']['ddmmyy'],$tstamp);
+			break;
 		}
+		return $dateStr;
 	}
 
 	/**
@@ -629,6 +641,12 @@ class template {
 			break;
 		}
 
+		$tabJScode = '';
+		if (!$BE_USER->uc['disableTabInTextarea'])	{
+				// This loads the tabulator-in-textarea feature. It automatically modifies every textarea which is found.
+			$tabJScode = '<script src="'.$this->backPath.'tab.js" type="text/javascript"></script>';
+		}
+
 			// Construct page header.
 		$str = $headerStart.'
 <html>
@@ -639,6 +657,7 @@ class template {
 	<title>'.htmlspecialchars($title).'</title>
 	'.$this->docStyle().'
 	'.$this->JScode.'
+	'.$tabJScode.'
 	'.$this->wrapScriptTags(implode("\n", $this->JScodeArray)).'
 	<!--###POSTJSMARKER###-->
 </head>
@@ -671,14 +690,19 @@ $str.=$this->docBodyTagBegin().
 				t3lib_BEfunc::getSetUpdateSignal().
 				$this->parseTime().
 				($this->form?'
-</form>':'').
-				($this->divClass?'
+</form>':'');
+
+		if ($this->docType!='xhtml_frames') {
+
+			$str .= ($this->divClass?'
 
 <!-- Wrapping DIV-section for whole page END -->
-</div>':'').
-				'
-</body>
-</html>	';
+</div>':'').'
+</body>	';
+
+		}
+
+		$str .= '</html>';
 
 			// Logging: Can't find better place to put it:
 		if (TYPO3_DLOG)	t3lib_div::devLog('END of BACKEND session','',0,array('_FLUSH'=>TRUE));
@@ -751,7 +775,7 @@ $str.=$this->docBodyTagBegin().
 	/**
 	 * Returns a blank <div>-section with a height
 	 *
-	 * @param	integer		Padding-top for the div-section (should be margin-top but konquorer (3.1) don't like it :-(
+	 * @param	integer		Padding-top for the div-section (should be margin-top but konqueror (3.1) doesn't like it :-(
 	 * @return	string		HTML content
 	 */
 	function spacer($dist)	{
@@ -825,11 +849,11 @@ $str.=$this->docBodyTagBegin().
 
 	/**
 	 * Originally it printed a kind of divider.
-	 * Depreciated. Just remove function calls to it or call the divider() function instead.
+	 * Deprecated. Just remove function calls to it or call the divider() function instead.
 	 *
 	 * @return	void
 	 * @internal
-	 * @depreciated
+	 * @deprecated
 	 */
 	function middle()	{
 	}
@@ -939,11 +963,11 @@ $str.=$this->docBodyTagBegin().
 	/**
 	 * Returns generator meta tag
 	 *
-	 * @return	string		<meta> tag with name "GENERATOR"
+	 * @return	string		<meta> tag with name "generator"
 	 */
 	function generator()	{
-		$str = 'TYPO3 '.$GLOBALS['TYPO_VERSION'].', http://typo3.com, &#169; Kasper Sk&#229;rh&#248;j 1998-2004, extensions are copyright of their respective owners.';
-		return '<meta name="GENERATOR" content="'.$str .'" />';
+		$str = 'TYPO3 '.TYPO3_branch.', http://typo3.com, &#169; Kasper Sk&#229;rh&#248;j 1998-2006, extensions are copyright of their respective owners.';
+		return '<meta name="generator" content="'.$str .'" />';
 	}
 
 
@@ -1059,6 +1083,7 @@ $str.=$this->docBodyTagBegin().
 				// remove nl from the beginning
 			$string = preg_replace ('/^\n+/', '', $string);
 				// re-ident to one tab using the first line as reference
+			$match = array();
 			if(preg_match('/^(\t+)/',$string,$match)) {
 				$string = str_replace($match[1],"\t", $string);
 			}
@@ -1184,7 +1209,7 @@ $str.=$this->docBodyTagBegin().
 	 */
 	function clearCacheMenu($id,$addSaveOptions=0)	{
 		global $BE_USER;
-		$opt=$addOptions;
+		$opt=array();
 		if ($addSaveOptions)	{
 			$opt[]='<option value="">'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:rm.menu',1).'</option>';
 			$opt[]='<option value="TBE_EDITOR_checkAndDoSubmit(1);">'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:rm.saveDoc',1).'</option>';
@@ -1204,11 +1229,11 @@ $str.=$this->docBodyTagBegin().
 			} else if (this.options[this.selectedIndex].value.indexOf(\';\')!=-1) {
 				eval(this.options[this.selectedIndex].value);
 			}else{
-				document.location=\''.$this->backPath.'tce_db.php?vC='.$BE_USER->veriCode().'&redirect='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI')).'&cacheCmd=\'+this.options[this.selectedIndex].value;
+				window.location.href=\''.$this->backPath.'tce_db.php?vC='.$BE_USER->veriCode().'&redirect='.rawurlencode(t3lib_div::getIndpEnv('REQUEST_URI')).'&cacheCmd=\'+this.options[this.selectedIndex].value;
 			}';
 		$af_content = '<select name="cacheCmd" onchange="'.htmlspecialchars($onChange).'">'.implode('',$opt).'</select>';
 
-		if (count($opt)>2)	{
+		if (count($opt)>1)	{
 			return $af_content;
 		}
 	}
@@ -1219,10 +1244,55 @@ $str.=$this->docBodyTagBegin().
 	 * @return	array		If values are present: [0] = A <script> section for the HTML page header, [1] = onmousemove/onload handler for HTML tag or alike, [2] = Two empty <div> layers for the context menu
 	 */
 	function getContextMenuCode()	{
-		if ($this->isCMlayers())	{
-			$content='
+		$content = '
 			<script type="text/javascript">
 			/*<![CDATA[*/
+					// is called from most clickmenu links
+				function showClickmenu(table, uid, listFr, enDisItems, backPath, addParams)	{
+					var url = "'.$this->backPath.'alt_clickmenu.php?table=" + table
+					          + "&uid=" + uid
+										+ "&listFr=" + listFr
+										+ "&enDisItems=" + enDisItems
+										+ "&backPath=" + backPath
+										+ "&addParams=" + addParams;
+
+					showClickmenu_raw(url);
+				}
+					// switch - either forwards call to ajax or does the request in the top frame
+				function showClickmenu_raw(url)	{';
+		if($this->isCMlayers()) { // ajax
+			$content .= '
+					url += "&ajax=1";
+					ajax_doRequest(url);';
+		} else { // no ajax
+			$content .= '
+					showClickmenu_noajax(url);';
+		}
+		$content .= '
+				}
+
+	/**
+	 * [Describe function...]
+	 *
+	 * @param	[type]		$url: ...
+	 * @return	[type]		...
+	 */
+				function showClickmenu_noajax(url)	{
+					top.loadTopMenu(url);
+				}';
+		if ($this->isCMlayers())	{
+			$content .= t3lib_ajax::getJScode('showClickmenu_ajax', 'showClickmenu_noajax');
+			$content.='
+					// opens the clickmenu, is called from ajax_doRequest
+				function showClickmenu_ajax(t3ajax)	{
+					if (t3ajax.getElementsByTagName("data")[0])	{
+						var clickmenu = t3ajax.getElementsByTagName("data")[0].getElementsByTagName("clickmenu")[0];
+						var tableData = clickmenu.getElementsByTagName("htmltable")[0].firstChild.data;
+						var cmlevel = clickmenu.getElementsByTagName("cmlevel")[0].firstChild.data;
+						setLayerObj(tableData,cmlevel);
+					}
+				}
+
 				var GLV_gap=10;
 				var GLV_curLayerX=new Array(0,0);
 				var GLV_curLayerY=new Array(0,0);
@@ -1260,8 +1330,8 @@ $str.=$this->docBodyTagBegin().
 					this.ref= bw.ns4?this.el.document:document;
 					this.x= (bw.ns4||bw.op)?this.css.left:this.el.offsetLeft;
 					this.y= (bw.ns4||bw.op)?this.css.top:this.el.offsetTop;
-					this.height= (bw.ie4||bw.ie5||bw.ns6||this.konq||bw.op7)?this.el.offsetHeight:bw.ns4?this.ref.height:bw.op?this.css.pixelHeight:0;
-					this.width= (bw.ie4||bw.ie5||bw.ns6||this.konq||bw.op7)?this.el.offsetWidth:bw.ns4?this.ref.width:bw.op?this.css.pixelWidth:0;
+					this.height= (bw.ie4||bw.dom)?this.el.offsetHeight:bw.ns4?this.ref.height:0;
+					this.width= (bw.ie4||bw.dom)?this.el.offsetWidth:bw.ns4?this.ref.width:0;
 					return this;
 				}
 					// GL_getObjCss(obj)
@@ -1304,8 +1374,8 @@ $str.=$this->docBodyTagBegin().
 
 					if (tempLayerObj && (level==0 || GLV_isVisible[level-1]))	{
 						tempLayerObj.el.innerHTML = html;
-						tempLayerObj.width= (bw.ie4||bw.ie5||bw.ns6||bw.konq||bw.op7)?this.el.offsetWidth:bw.ns4?this.ref.width:bw.op?this.css.pixelWidth:0;
-						tempLayerObj.height= (bw.ie4||bw.ie5||bw.ns6||bw.konq||bw.op7)?this.el.offsetHeight:bw.ns4?this.ref.height:bw.op?this.css.pixelHeight:0;
+						tempLayerObj.width= (bw.ie4||bw.dom)?this.el.offsetWidth:bw.ns4?this.ref.width:0;
+						tempLayerObj.height= (bw.ie4||bw.dom)?this.el.offsetHeight:bw.ns4?this.ref.height:0;
 
 							// konqueror (3.2.2) workaround
 						winHeight = (bw.konq)?window.innerHeight:winHeight;
@@ -1378,16 +1448,115 @@ $str.=$this->docBodyTagBegin().
 							}
 						}
 					}
-				}
-			/*]]>*/
-			</script>
-			';
+				}';
+			$content.='	/*]]>*/
+				</script>';
 			return array(
 				$content,
 				' onmousemove="GL_getMouse(event);" onload="initLayer();"',
 				'<div id="contentMenu0" style="z-index:1; position:absolute;visibility:hidden"></div><div id="contentMenu1" style="z-index:2; position:absolute;visibility:hidden"></div>'
 			);
-		} else return array('','','');
+		} else {
+			$content.='	/*]]>*/
+				</script>';
+			return array($content,'','');
+		}
+	}
+
+	/**
+	 * Returns an array with parts (JavaScript, init-functions, <div>-layers) for use on pages which have the drag and drop functionality (usually pages and folder display trees)
+	 *
+	 * @param	string		indicator of which table the drag and drop function should work on (pages or folders)
+	 * @return	array		If values are present: [0] = A <script> section for the HTML page header, [1] = onmousemove/onload handler for HTML tag or alike, [2] = One empty <div> layer for the follow-mouse drag element
+	 */
+	function getDragDropCode($table)	{
+		$content = '
+			<script type="text/javascript">
+			/*<![CDATA[*/
+			';
+
+		if ($this->isCMlayers())	{
+			$content.= '
+				var dragID = null;
+				var dragIconCSS = null;
+
+				function cancelDragEvent(event) {
+					dragID = null;
+					dragIconCSS.visibility = "hidden";
+					document.onmouseup = null;
+					document.onmousemove = null;
+				}
+
+	/**
+	 * [Describe function...]
+	 *
+	 * @param	[type]		$event: ...
+	 * @return	[type]		...
+	 */
+				function mouseMoveEvent (event) {
+					dragIconCSS.left = GLV_x+5+"px";
+					dragIconCSS.top = GLV_y-5+"px";
+					dragIconCSS.visibility = "visible";
+					return false;
+				}
+
+	/**
+	 * [Describe function...]
+	 *
+	 * @param	[type]		$id,elementID: ...
+	 * @return	[type]		...
+	 */
+				function dragElement(id,elementID) {
+					dragID = id;
+					if (elementID == null)	{
+						elementID = id;
+					}
+					document.getElementById("dragIcon").innerHTML=document.getElementById("dragIconID_"+elementID).innerHTML + document.getElementById("dragTitleID_"+elementID).getElementsByTagName("a")[0].innerHTML;
+					dragIconCSS = new GL_getObjCss("dragIcon");
+					dragIconCSS.whiteSpace = "nowrap";
+					document.onmouseup = cancelDragEvent;
+					document.onmousemove = mouseMoveEvent;
+					return false;
+				}
+
+	/**
+	 * [Describe function...]
+	 *
+	 * @param	[type]		$id: ...
+	 * @return	[type]		...
+	 */
+				function dropElement(id) {
+					if ((dragID != null) && (dragID != id)) {
+						var url = "'.$this->backPath.'alt_clickmenu.php?dragDrop='.$table.'"
+									+ "&srcId=" + dragID
+									+ "&dstId=" + id
+									+ "&backPath='.t3lib_div::shortMD5(''.'|'.$GLOBALS['TYPO3_CONF_VARS']['SYS']['encryptionKey']).'";
+						showClickmenu_raw(url);
+					}
+					cancelDragEvent();
+					return false;
+				}
+				';
+		}
+		else {
+			$content.= '
+				function dragElement(id) { return false; }
+				function dropElement(id) { return false; }
+				';
+		}
+		$content.='
+			/*]]>*/
+			</script>';
+
+		if ($this->isCMlayers())	{
+			return array(
+				$content,
+				'',
+				'<div id="dragIcon" style="z-index:1;position:absolute;visibility:hidden;filter:alpha(opacity=50);-moz-opacity:0.5;opacity:0.5;"><img src="" width="18" height="16"></div>'
+			);
+		} else {
+			return array($content,'','');
+		}
 	}
 
 	/**
@@ -1403,7 +1572,7 @@ $str.=$this->docBodyTagBegin().
 	 * @param	string		$script is the script to send the &id to, if empty it's automatically found
 	 * @param	string		$addParams is additional parameters to pass to the script.
 	 * @return	string		HTML code for tab menu
-	 * @author	Ren Fritz <r.fritz@colorcube.de>
+	 * @author	Rene Fritz <r.fritz@colorcube.de>
 	 */
 	function getTabMenu($mainParams,$elementName,$currentValue,$menuItems,$script='',$addparams='')	{
 		$content='';
@@ -1538,10 +1707,12 @@ $str.=$this->docBodyTagBegin().
 
 				$isActive = strcmp($def['content'],'');
 
+				$mouseOverOut = 'onmouseover="DTM_mouseOver(this);" onmouseout="DTM_mouseOut(this);"';
+
 				if (!$foldout)	{
 						// Create TAB cell:
 					$options[$tabRows][] = '
-							<td class="'.($isActive ? 'tab' : 'disabled').'" id="'.$id.'-'.$index.'-MENU"'.$noWrap.'>'.
+							<td class="'.($isActive ? 'tab' : 'disabled').'" id="'.$id.'-'.$index.'-MENU"'.$noWrap.$mouseOverOut.'>'.
 							($isActive ? '<a href="#" onclick="'.htmlspecialchars($onclick).'"'.($def['linkTitle'] ? ' title="'.htmlspecialchars($def['linkTitle']).'"':'').'>' : '').
 							$def['icon'].
 							($def['label'] ? htmlspecialchars($def['label']) : '&nbsp;').
@@ -1552,8 +1723,9 @@ $str.=$this->docBodyTagBegin().
 				} else {
 						// Create DIV layer for content:
 					$divs[] = '
-						<div class="'.($isActive ? 'tab' : 'disabled').'" id="'.$id.'-'.$index.'-MENU">'.
+						<div class="'.($isActive ? 'tab' : 'disabled').'" id="'.$id.'-'.$index.'-MENU"'.$mouseOverOut.'>'.
 							($isActive ? '<a href="#" onclick="'.htmlspecialchars($onclick).'"'.($def['linkTitle'] ? ' title="'.htmlspecialchars($def['linkTitle']).'"':'').'>' : '').
+							$def['icon'].
 							($def['label'] ? htmlspecialchars($def['label']) : '&nbsp;').
 							($isActive ? '</a>' : '').
 							'</div>';
@@ -1572,7 +1744,7 @@ $str.=$this->docBodyTagBegin().
 					';
 					if ($toggle==1)	{
 						$JSinit[] = '
-							if (top.DTM_currentTabs["'.$id.'-'.$index.'"]) { DTM_toggle("'.$id.'","'.$index.'"); }
+							if (top.DTM_currentTabs["'.$id.'-'.$index.'"]) { DTM_toggle("'.$id.'","'.$index.'",1); }
 						';
 					}
 
@@ -1611,7 +1783,6 @@ $str.=$this->docBodyTagBegin().
 				<script type="text/javascript">
 					DTM_array["'.$id.'"] = new Array();
 					'.implode('',$JSinit).'
-
 					'.($toggle<=0 ? 'DTM_activate("'.$id.'", top.DTM_currentTabs["'.$id.'"]?top.DTM_currentTabs["'.$id.'"]:'.intval($defaultTabIndex).', 0);' : '').'
 				</script>
 
@@ -1632,6 +1803,12 @@ $str.=$this->docBodyTagBegin().
 			<script type="text/javascript">
 			/*<![CDATA[*/
 				var DTM_array = new Array();
+				var DTM_origClass = new String();
+
+					// if tabs are used in a popup window the array might not exists
+				if(!top.DTM_currentTabs) {
+					top.DTM_currentTabs = new Array();
+				}
 
 				function DTM_activate(idBase,index,doToogle)	{	//
 						// Hiding all:
@@ -1648,29 +1825,57 @@ $str.=$this->docBodyTagBegin().
 					if (document.getElementById(idBase+"-"+index+"-DIV"))	{
 						if (doToogle && document.getElementById(idBase+"-"+index+"-DIV").style.display == "block")	{
 							document.getElementById(idBase+"-"+index+"-DIV").style.display = "none";
-							document.getElementById(idBase+"-"+index+"-MENU").attributes.getNamedItem("class").nodeValue = "tab";
+							if(DTM_origClass=="") {
+								document.getElementById(idBase+"-"+index+"-MENU").attributes.getNamedItem("class").nodeValue = "tab";
+							} else {
+								DTM_origClass = "tab";
+							}
 							top.DTM_currentTabs[idBase] = -1;
 						} else {
 							document.getElementById(idBase+"-"+index+"-DIV").style.display = "block";
-							document.getElementById(idBase+"-"+index+"-MENU").attributes.getNamedItem("class").nodeValue = "tabact";
+							if(DTM_origClass=="") {
+								document.getElementById(idBase+"-"+index+"-MENU").attributes.getNamedItem("class").nodeValue = "tabact";
+							} else {
+								DTM_origClass = "tabact";
+							}
 							top.DTM_currentTabs[idBase] = index;
 						}
 					}
 				}
-				function DTM_toggle(idBase,index)	{	//
+				function DTM_toggle(idBase,index,isInit)	{	//
 						// Showing one:
 					if (document.getElementById(idBase+"-"+index+"-DIV"))	{
 						if (document.getElementById(idBase+"-"+index+"-DIV").style.display == "block")	{
 							document.getElementById(idBase+"-"+index+"-DIV").style.display = "none";
-							document.getElementById(idBase+"-"+index+"-MENU").attributes.getNamedItem("class").nodeValue = "tab";
+							if(isInit) {
+								document.getElementById(idBase+"-"+index+"-MENU").attributes.getNamedItem("class").nodeValue = "tab";
+							} else {
+								DTM_origClass = "tab";
+							}
 							top.DTM_currentTabs[idBase+"-"+index] = 0;
 						} else {
 							document.getElementById(idBase+"-"+index+"-DIV").style.display = "block";
-							document.getElementById(idBase+"-"+index+"-MENU").attributes.getNamedItem("class").nodeValue = "tabact";
+							if(isInit) {
+								document.getElementById(idBase+"-"+index+"-MENU").attributes.getNamedItem("class").nodeValue = "tabact";
+							} else {
+								DTM_origClass = "tabact";
+							}
 							top.DTM_currentTabs[idBase+"-"+index] = 1;
 						}
 					}
 				}
+
+				function DTM_mouseOver(obj) {	//
+						DTM_origClass = obj.attributes.getNamedItem(\'class\').nodeValue;
+						obj.attributes.getNamedItem(\'class\').nodeValue += "_over";
+				}
+
+				function DTM_mouseOut(obj) {	//
+						obj.attributes.getNamedItem(\'class\').nodeValue = DTM_origClass;
+						DTM_origClass = "";
+				}
+
+
 			/*]]>*/
 			</script>
 		';
@@ -1686,59 +1891,153 @@ $str.=$this->docBodyTagBegin().
 	 */
 	function getVersionSelector($id,$noAction=FALSE)	{
 
-		if ($id>0 && t3lib_extMgm::isLoaded('version'))	{
+		if ($id>0)	{
+			if (t3lib_extMgm::isLoaded('version') && $GLOBALS['BE_USER']->workspace==0)	{
 
-				// Get Current page record:
-			$curPage = t3lib_BEfunc::getRecord('pages',$id);
-				// If the selected page is not online, find the right ID
-			$onlineId = ($curPage['pid']==-1 ? $curPage['t3ver_oid'] : $id);
-				// Select all versions of online version:
-			$versions = t3lib_BEfunc::selectVersionsOfRecord('pages', $onlineId, 'uid,pid,t3ver_label,t3ver_oid,t3ver_id');
+					// Get Current page record:
+				$curPage = t3lib_BEfunc::getRecord('pages',$id);
+					// If the selected page is not online, find the right ID
+				$onlineId = ($curPage['pid']==-1 ? $curPage['t3ver_oid'] : $id);
+					// Select all versions of online version:
+				$versions = t3lib_BEfunc::selectVersionsOfRecord('pages', $onlineId, 'uid,pid,t3ver_label,t3ver_oid,t3ver_wsid,t3ver_id');
 
-				// If more than one was found...:
-			if (count($versions)>1)	{
+					// If more than one was found...:
+				if (count($versions)>1)	{
 
-					// Create selector box entries:
-				$opt = array();
-				foreach($versions as $vRow)	{
-					$opt[] = '<option value="'.htmlspecialchars(t3lib_div::linkThisScript(array('id'=>$vRow['uid']))).'"'.($id==$vRow['uid']?' selected="selected"':'').'>'.
-							htmlspecialchars($vRow['t3ver_label'].' [v#'.$vRow['t3ver_id'].']'.($vRow['uid']==$onlineId ? ' =>'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:ver.online').'<=':'')).
-							'</option>';
-				}
+						// Create selector box entries:
+					$opt = array();
+					foreach($versions as $vRow)	{
+						$opt[] = '<option value="'.htmlspecialchars(t3lib_div::linkThisScript(array('id'=>$vRow['uid']))).'"'.($id==$vRow['uid']?' selected="selected"':'').'>'.
+								htmlspecialchars($vRow['t3ver_label'].' [v#'.$vRow['t3ver_id'].', WS:'.$vRow['t3ver_wsid'].']'.($vRow['uid']==$onlineId ? ' =>'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:ver.online').'<=':'')).
+								'</option>';
+					}
 
-					// Add management link:
-				$opt[] = '<option value="'.htmlspecialchars(t3lib_div::linkThisScript(array('id'=>$id))).'">---</option>';
-				$opt[] = '<option value="'.htmlspecialchars($this->backPath.t3lib_extMgm::extRelPath('version').'cm1/index.php?table=pages&uid='.$onlineId).'">'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:ver.mgm',1).'</option>';
+						// Add management link:
+					$opt[] = '<option value="'.htmlspecialchars(t3lib_div::linkThisScript(array('id'=>$id))).'">---</option>';
+					$opt[] = '<option value="'.htmlspecialchars($this->backPath.t3lib_extMgm::extRelPath('version').'cm1/index.php?table=pages&uid='.$onlineId).'">'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:ver.mgm',1).'</option>';
 
-					// Create onchange handler:
-				$onChange = "document.location=this.options[this.selectedIndex].value;";
+						// Create onchange handler:
+					$onChange = "window.location.href=this.options[this.selectedIndex].value;";
 
-					// Controls:
-				if ($id==$onlineId)	{
-					$controls = '<img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/blinkarrow_left.gif','width="5" height="9"').' class="absmiddle" alt="" /> <b>'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:ver.online',1).'</b>';
-				} elseif (!$noAction) {
-					$controls = '<a href="'.$this->issueCommand('&cmd[pages]['.$onlineId.'][version][swapWith]='.$id.'&cmd[pages]['.$onlineId.'][version][action]=swap&cmd[pages]['.$onlineId.'][version][swapContent]=1',t3lib_div::linkThisScript(array('id'=>$onlineId))).'">'.
-							'<img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/insert2.gif','width="14" height="14"').' style="margin-right: 2px;" class="absmiddle" alt="" title="'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:ver.swapPage',1).'" />'.
-							'<b>'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:ver.swap',1).'</b></a>';
+						// Controls:
+					if ($id==$onlineId)	{
+						$controls = '<img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/blinkarrow_left.gif','width="5" height="9"').' class="absmiddle" alt="" /> <b>'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:ver.online',1).'</b>';
+					} elseif (!$noAction) {
+						$controls = '<a href="'.$this->issueCommand('&cmd[pages]['.$onlineId.'][version][swapWith]='.$id.'&cmd[pages]['.$onlineId.'][version][action]=swap',t3lib_div::linkThisScript(array('id'=>$onlineId))).'" class="nobr">'.
+								'<img'.t3lib_iconWorks::skinImg($this->backPath,'gfx/insert1.gif','width="14" height="14"').' style="margin-right: 2px;" class="absmiddle" alt="" title="'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:ver.swapPage',1).'" />'.
+								'<b>'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:ver.swap',1).'</b></a>';
+					}
+
+						// Write out HTML code:
+					return '
+
+						<!--
+							Version selector:
+						-->
+						<table border="0" cellpadding="0" cellspacing="0" id="typo3-versionSelector">
+							<tr>
+								<td>'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:ver.selVer',1).'</td>
+								<td>
+									<select onchange="'.htmlspecialchars($onChange).'">
+										'.implode('',$opt).'
+									</select></td>
+								<td>'.$controls.'</td>
+							</tr>
+						</table>
+					';
 				}
+			} elseif ($GLOBALS['BE_USER']->workspace!==0) {
 
 					// Write out HTML code:
-				return '
+				switch($GLOBALS['BE_USER']->workspace)	{
+					case 0:
+						$wsTitle = 'LIVE';
+					break;
+					case -1:
+						$wsTitle = 'Draft';
+					break;
+					default:
+						$wsTitle = $GLOBALS['BE_USER']->workspaceRec['title'];
+					break;
+				}
 
-					<!--
-						Version selector:
-					-->
-					<table border="0" cellpadding="0" cellspacing="0" id="typo3-versionSelector">
-						<tr>
-							<td>'.$GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:ver.selVer',1).'</td>
-							<td>
-								<select onchange="'.htmlspecialchars($onChange).'">
-									'.implode('',$opt).'
-								</select></td>
-							<td>'.$controls.'</td>
-						</tr>
-					</table>
-				';
+				if (t3lib_BEfunc::isPidInVersionizedBranch($id)=='branchpoint')	{
+					return '
+
+						<!--
+							Version selector:
+						-->
+						<table border="0" cellpadding="0" cellspacing="0" id="typo3-versionSelector">
+							<tr>
+								<td>Workspace: "'.htmlspecialchars($wsTitle).'"</td>
+								<td><em>Inside branch, no further versioning possible</em></td>
+							</tr>
+						</table>
+					';
+				} else {
+						// Get Current page record:
+					$curPage = t3lib_BEfunc::getRecord('pages',$id);
+						// If the selected page is not online, find the right ID
+					$onlineId = ($curPage['pid']==-1 ? $curPage['t3ver_oid'] : $id);
+						// The version of page:
+					$verPage = t3lib_BEfunc::getWorkspaceVersionOfRecord($GLOBALS['BE_USER']->workspace, 'pages', $onlineId);
+
+					if (!$verPage)	{
+
+						if (!count(t3lib_BEfunc::countVersionsOfRecordsOnPage($GLOBALS['BE_USER']->workspace, $onlineId)))	{
+							if ($GLOBALS['BE_USER']->workspaceVersioningTypeAccess(0))	{
+
+								$onClick = $this->issueCommand('&cmd[pages]['.$onlineId.'][version][action]=new&cmd[pages]['.$onlineId.'][version][treeLevels]=0',t3lib_div::linkThisScript(array('id'=>$onlineId)));
+								$onClick = 'window.location.href=\''.$onClick.'\'; return false;';
+									// Write out HTML code:
+								return '
+
+									<!--
+										No version yet, create one?
+									-->
+									<table border="0" cellpadding="0" cellspacing="0" id="typo3-versionSelector">
+										<tr>
+											<td>Workspace: "'.htmlspecialchars($wsTitle).'"</td>
+											<td>
+												<input type="submit" value="New version of page" name="_" onclick="'.htmlspecialchars($onClick).'" /></td>
+										</tr>
+									</table>
+								';
+							}
+						} else {
+							return '
+
+								<!--
+									Version selector:
+								-->
+								<table border="0" cellpadding="0" cellspacing="0" id="typo3-versionSelector">
+									<tr>
+										<td>Workspace: "'.htmlspecialchars($wsTitle).'"</td>
+										<td><em>Versions found on page, no "Page" versioning possible</em></td>
+									</tr>
+								</table>
+							';
+						}
+					} elseif ($verPage['t3ver_swapmode']==0) {
+						$onClick = $this->issueCommand('&cmd[pages]['.$onlineId.'][version][action]=swap&cmd[pages]['.$onlineId.'][version][swapWith]='.$verPage['uid'],t3lib_div::linkThisScript(array('id'=>$onlineId)));
+						$onClick = 'window.location.href=\''.$onClick.'\'; return false;';
+
+							// Write out HTML code:
+						return '
+
+							<!--
+								Version selector:
+							-->
+							<table border="0" cellpadding="0" cellspacing="0" id="typo3-versionSelector">
+								<tr>
+									<td>Workspace: "'.htmlspecialchars($wsTitle).'"</td>
+									<td>
+										<input type="submit" value="Publish page" name="_" onclick="'.htmlspecialchars($onClick).'" /></td>
+								</tr>
+							</table>
+						';
+					}
+				}
 			}
 		}
 	}
@@ -1811,4 +2110,4 @@ $LANG->init($BE_USER->uc['lang']);
 // The template is loaded
 // ******************************
 $TBE_TEMPLATE = t3lib_div::makeInstance('template');
-?>
\ No newline at end of file
+?>
