@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -27,7 +27,7 @@
 /**
  * Wizard for inserting TSconfig in form fields. (page,user or TS)
  *
- * $Id: wizard_tsconfig.php,v 1.11 2004/09/13 22:57:22 typo3 Exp $
+ * $Id: wizard_tsconfig.php 1421 2006-04-10 09:27:15Z mundaun $
  * Revised for TYPO3 3.6 November/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -38,27 +38,31 @@
  *
  *
  *
- *   94: class ext_TSparser extends t3lib_tsparser_ext
- *  102:     function makeHtmlspecialchars($P)
+ *   98: class ext_TSparser extends t3lib_tsparser_ext
+ *  106:     function makeHtmlspecialchars($P)
  *
  *
- *  123: class SC_wizard_tsconfig
- *  145:     function init()
- *  252:     function main()
- *  279:     function printContent()
- *  290:     function browseTSprop($mode,$show)
+ *  127: class SC_wizard_tsconfig
+ *  149:     function init()
+ *  202:     function setValue(field,value)
+ *  232:     function mixerField(cmd,objString)
+ *  258:     function str_replace(match,replace,string)
+ *  280:     function jump(show,objString)
+ *  295:     function main()
+ *  320:     function printContent()
+ *  333:     function browseTSprop($mode,$show)
  *
  *              SECTION: Module functions
- *  376:     function getObjTree()
- *  406:     function setObj(&$objTree,$strArr,$params)
- *  426:     function revertFromSpecialChars($str)
- *  439:     function doLink($params)
- *  452:     function removePointerObjects($objArray)
- *  471:     function linkToObj($str,$uid,$objString='')
- *  484:     function printTable($table,$objString,$objTree)
- *  565:     function linkProperty($str,$propertyVal,$prefix,$datatype)
+ *  419:     function getObjTree()
+ *  449:     function setObj(&$objTree,$strArr,$params)
+ *  469:     function revertFromSpecialChars($str)
+ *  482:     function doLink($params)
+ *  495:     function removePointerObjects($objArray)
+ *  514:     function linkToObj($str,$uid,$objString='')
+ *  527:     function printTable($table,$objString,$objTree)
+ *  608:     function linkProperty($str,$propertyName,$prefix,$datatype)
  *
- * TOTAL FUNCTIONS: 13
+ * TOTAL FUNCTIONS: 17
  * (This index is automatically created/updated by the extension "extdeveval")
  *
  */
@@ -154,13 +158,11 @@ class SC_wizard_tsconfig {
 		$this->show = t3lib_div::_GP('show');
 		$this->objString = t3lib_div::_GP('objString');
 		$this->onlyProperty = t3lib_div::_GP('onlyProperty');
-
 			// Preparing some JavaScript code:
 		if (!is_array($this->P['fieldChangeFunc']))	$this->P['fieldChangeFunc']=array();
 		unset($this->P['fieldChangeFunc']['alert']);
-		reset($this->P['fieldChangeFunc']);
 		$update='';
-		while(list($k,$v)=each($this->P['fieldChangeFunc']))	{
+		foreach($this->P['fieldChangeFunc'] as $k=>$v)	{
 			$update.= '
 			window.opener.'.$v;
 		}
@@ -180,27 +182,58 @@ class SC_wizard_tsconfig {
 		';
 
 		$this->doc->JScode.=$this->doc->wrapScriptTags('
-			function checkReference()	{	//
+			function checkReference_name()	{	// Checks if the input field containing the name exists in the document
 				if (window.opener && window.opener.document && window.opener.document.'.$this->P['formName'].' && window.opener.document.'.$this->P['formName'].'["'.$this->P['itemName'].'"] )	{
 					return window.opener.document.'.$this->P['formName'].'["'.$this->P['itemName'].'"];
-				} else {
-					close();
 				}
 			}
-			function setValue(input)	{	//
-				var field = checkReference();
-				if (field)	{
-					field.value=input+"\n"+field.value;
+			function checkReference_value()	{	// Checks if the input field containing the value exists in the document
+				if (window.opener && window.opener.document && window.opener.document.'.$this->P['formName'].' && window.opener.document.'.$this->P['formName'].'["'.$this->P['itemValue'].'"] )	{
+					return window.opener.document.'.$this->P['formName'].'["'.$this->P['itemValue'].'"];
+				}
+			}
+
+	/**
+	 * [Describe function...]
+	 *
+	 * @param	[type]		$field,value: ...
+	 * @return	[type]		...
+	 */
+			function setValue(field,value)	{
+				var nameField = checkReference_name();
+				var valueField = checkReference_value();
+				if (nameField)	{
+					if (valueField)	{	// This applies to the TS Object Browser module
+						nameField.value=field;
+						valueField.value=value;
+					} else {		// This applies to the Info/Modify module and the Page TSconfig field
+						if (value) {
+							nameField.value=field+"="+value+"\n"+nameField.value;
+						} else {
+							nameField.value=field+"\n"+nameField.value;
+						}
+					}
 					'.$update.'
 					window.opener.focus();
 				}
 				close();
 			}
-			function getValue()	{	//
-				var field = checkReference();
-				if (field)	return field.value;
+			function getValue()	{	// This is never used. Remove it?
+				var field = checkReference_name();
+				if (field)	{
+					return field.value;
+				} else {
+					close();
+				}
 			}
-			function mixerField(cmd,objString)	{	//
+
+	/**
+	 * [Describe function...]
+	 *
+	 * @param	[type]		$cmd,objString: ...
+	 * @return	[type]		...
+	 */
+			function mixerField(cmd,objString)	{
 				var temp;
 				switch(cmd)	{
 					case "Indent":
@@ -219,7 +252,14 @@ class SC_wizard_tsconfig {
 					break;
 				}
 			}
-			function str_replace(match,replace,string)	{	//
+
+	/**
+	 * [Describe function...]
+	 *
+	 * @param	[type]		$match,replace,string: ...
+	 * @return	[type]		...
+	 */
+			function str_replace(match,replace,string)	{
 				var input = ""+string;
 				var matchStr = ""+match;
 				if (!matchStr)	{return string;}
@@ -234,8 +274,15 @@ class SC_wizard_tsconfig {
 				output+=""+input.substr(pointer);
 				return output;
 			}
-			function jump(show,objString)	{	//
-				document.location = "'.t3lib_div::linkThisScript(array('show'=>'','objString'=>'')).'&show="+show+"&objString="+objString;
+
+	/**
+	 * [Describe function...]
+	 *
+	 * @param	[type]		$show,objString: ...
+	 * @return	[type]		...
+	 */
+			function jump(show,objString)	{
+				window.location.href = "'.t3lib_div::linkThisScript(array('show'=>'','objString'=>'')).'&show="+show+"&objString="+objString;
 			}
 		');
 
@@ -267,8 +314,6 @@ class SC_wizard_tsconfig {
 			<a href="'.htmlspecialchars('http://typo3.org/documentation/document-library/doc_core_tsconfig/').'" target="_blank">'.$LANG->getLL('tsprop_tsconfig',1).'</a>
 			',0,1);
 		}
-			// Ending page:
-		$this->content.=$this->doc->endPage();
 	}
 
 	/**
@@ -277,6 +322,8 @@ class SC_wizard_tsconfig {
 	 * @return	void
 	 */
 	function printContent()	{
+		$this->content.= $this->doc->endPage();
+		$this->content = $this->doc->insertStylesAndJS($this->content);
 		echo $this->content;
 	}
 
@@ -321,7 +368,7 @@ class SC_wizard_tsconfig {
 				$links[]='<a href="#" onclick="mixerField(\'Outdent\');return false;">'.$LANG->getLL('tsprop_mixer_outdent',1).'</a>';
 				$links[]='<a href="#" onclick="mixerField(\'Wrap\',unescape(\''.rawurlencode($obj_string).'\'));return false;">'.$LANG->getLL('tsprop_mixer_wrap',1).'</a>';
 				$links[]='<a href="#" onclick="mixerField(\'Transfer\');return false;">'.$LANG->getLL('tsprop_mixer_transfer',1).'</a>';
-				$out.='<textarea rows="5" name="mixer" wrap="off"'.$this->doc->formWidthText(48,'','off').'></textarea>';
+				$out.='<textarea rows="5" name="mixer" wrap="off"'.$this->doc->formWidthText(48,'','off').' class="fixed-font enable-tab"></textarea>';
 				$out.='<br /><strong>'.implode('&nbsp; | &nbsp;',$links).'</strong>';
 				$out.='<hr />';
 			}
@@ -562,27 +609,25 @@ class SC_wizard_tsconfig {
 	 * @param	string		Data type
 	 * @return	string		Linked $str
 	 */
-	function linkProperty($str,$propertyVal,$prefix,$datatype)	{
+	function linkProperty($str,$propertyName,$prefix,$datatype)	{
 		$out='';
 
 			// Setting preset value:
 		if (strstr($datatype,'boolean'))	{
-			$propertyVal.='=1';	// add preset "=1" to boolean values.
-		} else {
-			$propertyVal.='=';	// add preset "="
+			$propertyVal='1';	// preset "1" to boolean values.
 		}
 
 			// Adding mixer features; The plus icon:
 		if(!$this->onlyProperty)	{
-			$aOnClick = 'document.editform.mixer.value=unescape(\'  '.rawurlencode($propertyVal).'\')+\'\n\'+document.editform.mixer.value; return false;';
+			$aOnClick = 'document.editform.mixer.value=unescape(\'  '.rawurlencode($propertyName.'='.$propertyVal).'\')+\'\n\'+document.editform.mixer.value; return false;';
 			$out.= '<a href="#" onclick="'.htmlspecialchars($aOnClick).'">'.
 					'<img'.t3lib_iconWorks::skinImg($this->doc->backPath,'gfx/plusbullet2.gif','width="18" height="16"').' title="'.$GLOBALS['LANG']->getLL('tsprop_addToList',1).'" align="top" alt="" />'.
 					'</a>';
-			$propertyVal = $prefix.'.'.$propertyVal;
+			$propertyName = $prefix.'.'.$propertyName;
 		}
 
 			// Wrap string:
-		$aOnClick = 'setValue(unescape(\''.rawurlencode($propertyVal).'\')); return false;';
+		$aOnClick = 'setValue(unescape(\''.rawurlencode($propertyName).'\'),unescape(\''.rawurlencode($propertyVal).'\')); return false;';
 		$out.= '<a href="#" onclick="'.htmlspecialchars($aOnClick).'">'.$str.'</a>';
 
 			// Return link:
